@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -18,7 +29,7 @@
  *
  *    Tests all the reg tests:
  *
- *        alltests_reg  command
+ *        alltests_reg command
  *
  *    where
  *        <command> == "generate" to make the golden files in /tmp/golden
@@ -27,7 +38,8 @@
  *        <command> == "display" to make local files and display
  *
  *    You can also run each test individually with any one of these
- *    arguments.
+ *    arguments.  Warning: if you run this with "display", a very
+ *    large number of images will be displayed on the screen.
  */
 
 #include <string.h>
@@ -36,10 +48,16 @@
 static const char *tests[] = {
                               "alphaops_reg",
                               "alphaxform_reg",
+                         /*   "bilateral2_reg", */
                               "binarize_reg",
+                              "blackwhite_reg",
+                              "blend3_reg",
+                              "blend4_reg",
+                              "colorcontent_reg",
                               "coloring_reg",
                               "colormask_reg",
                               "colorquant_reg",
+                              "colorspace_reg",
                               "compare_reg",
                               "convolve_reg",
                               "dewarp_reg",
@@ -47,15 +65,25 @@ static const char *tests[] = {
                               "dna_reg",
                               "dwamorph1_reg",
                               "enhance_reg",
+                              "findcorners_reg",
                               "findpattern_reg",
-                              "fpix_reg",
-                              "gifio_reg",
+                              "fpix1_reg",
+                              "fpix2_reg",
+                         /*   "gifio_reg",  */
                               "graymorph2_reg",
                               "hardlight_reg",
+                              "insert_reg",
                               "ioformats_reg",
+                              "jpegio_reg",
                               "kernel_reg",
+                              "label_reg",
                               "maze_reg",
+                              "multitype_reg",
+                              "nearline_reg",
+                              "newspaper_reg",
                               "overlap_reg",
+                              "paint_reg",
+                              "paintmask_reg",
                               "pdfseg_reg",
                               "pixa2_reg",
                               "pixserial_reg",
@@ -65,13 +93,14 @@ static const char *tests[] = {
                               "psioseg_reg",
                               "rankbin_reg",
                               "rankhisto_reg",
+                              "rasteropip_reg",
                               "rotateorth_reg",
                               "rotate1_reg",
                               "rotate2_reg",
                               "scale_reg",
                               "seedspread_reg",
                               "selio_reg",
-                              "shear_reg",
+                              "shear1_reg",
                               "shear2_reg",
                               "skew_reg",
                               "splitcomp_reg",
@@ -80,23 +109,22 @@ static const char *tests[] = {
                               "translate_reg",
                               "warper_reg",
                               "writetext_reg",
+                              "xformbox_reg",
                              };
 
 static const char *header = {"\n=======================\n"
                              "Regression Test Results\n"
                              "======================="};
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 char        *str, *results_file;
 char         command[256], buf[256];
 l_int32      i, ntests, dotest, nfail, ret, start, stop;
 SARRAY      *sa;
-L_TIMER      timer;
 static char  mainName[] = "alltests_reg";
 
-
     if (argc != 2)
         return ERROR_INT(" Syntax alltests_reg [generate | compare | display]",
                          mainName, 1);
@@ -123,7 +151,11 @@ static char  mainName[] = "alltests_reg";
 
     nfail = 0;
     for (i = 0; i < ntests; i++) {
+#ifndef  _WIN32
         snprintf(command, sizeof(command) - 2, "./%s %s", tests[i], argv[1]);
+#else  /* windows interprets '/' as a commandline flag */
+        snprintf(command, sizeof(command) - 2, "%s %s", tests[i], argv[1]);
+#endif  /* ! _WIN32 */
         ret = system(command);
         if (ret) {
             snprintf(buf, sizeof(buf), "Failed to complete %s\n", tests[i]);
