@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -27,8 +38,8 @@ PIX *PixTest2(PIX *pixs, l_int32 size, l_float32 factor, l_int32 nx,
 void PixTest3(PIX *pixs, l_int32 size, l_float32 factor,
               l_int32 nx, l_int32 ny, l_int32 paircount, L_REGPARAMS *rp);
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 PIX          *pixs, *pixt1, *pixt2;
 L_REGPARAMS  *rp;
@@ -88,10 +99,10 @@ PIXA    *pixa;
         /* Get results */
     pixSauvolaBinarize(pixs, size, factor, 1, &pixm, &pixsd, &pixth, &pixd);
     pixa = pixaCreate(0);
-    pixSaveTiled(pixm, pixa, 1, 1, 30, 8);
-    pixSaveTiled(pixsd, pixa, 1, 0, 30, 8);
-    pixSaveTiled(pixth, pixa, 1, 1, 30, 8);
-    pixSaveTiled(pixd, pixa, 1, 0, 30, 8);
+    pixSaveTiled(pixm, pixa, 1.0, 1, 30, 8);
+    pixSaveTiled(pixsd, pixa, 1.0, 0, 30, 8);
+    pixSaveTiled(pixth, pixa, 1.0, 1, 30, 8);
+    pixSaveTiled(pixd, pixa, 1.0, 0, 30, 8);
     pixt = pixaDisplay(pixa, 0, 0);
     regTestWritePixAndCheck(rp, pixt, IFF_JFIF_JPEG);
     if (rp->index < 5)
@@ -105,7 +116,7 @@ PIXA    *pixa;
     pixDestroy(&pixt);
     return pixd;
 }
-   
+
 
 PIX *
 PixTest2(PIX          *pixs,
@@ -135,8 +146,8 @@ PIXA    *pixa;
     regTestWritePixAndCheck(rp, pixd, IFF_PNG);
     if (rp->index < 5 && rp->display) {
         pixa = pixaCreate(0);
-        pixSaveTiled(pixth, pixa, 1, 1, 30, 8);
-        pixSaveTiled(pixd, pixa, 1, 0, 30, 8);
+        pixSaveTiled(pixth, pixa, 1.0, 1, 30, 8);
+        pixSaveTiled(pixd, pixa, 1.0, 0, 30, 8);
         pixt = pixaDisplay(pixa, 0, 0);
         pixDisplayWithTitle(pixt, 100, 400, NULL, rp->display);
         pixDestroy(&pixt);
@@ -146,7 +157,7 @@ PIXA    *pixa;
     pixDestroy(&pixth);
     return pixd;
 }
- 
+
 
 void
 PixTest3(PIX          *pixs,
