@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -30,11 +41,11 @@
 static const char *filein = "test24.jpg";
 static const l_int32 WIDTH = 150;
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 char          textstr[256];
-l_int32       w, h, d, i, same;
+l_int32       w, h, d, i;
 l_uint32      srcval, dstval;
 l_float32     scalefact, sat, fract;
 L_BMF        *bmf8;
@@ -64,7 +75,7 @@ L_REGPARAMS  *rp;
         pixaAddPix(pixa, pixt0, L_INSERT);
     }
     pixt1 = pixaDisplayTiledAndScaled(pixa, 32, w, 5, 0, 10, 2);
-    pixSaveTiled(pixt1, pixaf, 1, 1, 20, 32);
+    pixSaveTiled(pixt1, pixaf, 1.0, 1, 20, 32);
     regTestWritePixAndCheck(rp, pixt1, IFF_PNG);  /* 0 */
     pixDisplayWithTitle(pixt1, 0, 100, "TRC Gamma", rp->display);
     pixDestroy(&pixt1);
@@ -77,7 +88,7 @@ L_REGPARAMS  *rp;
         pixaAddPix(pixa, pixt0, L_INSERT);
     }
     pixt1 = pixaDisplayTiledAndScaled(pixa, 32, w, 5, 0, 10, 2);
-    pixSaveTiled(pixt1, pixaf, 1, 1, 20, 0);
+    pixSaveTiled(pixt1, pixaf, 1.0, 1, 20, 0);
     regTestWritePixAndCheck(rp, pixt1, IFF_PNG);  /* 1 */
     pixDisplayWithTitle(pixt1, 300, 100, "TRC", rp->display);
     pixDestroy(&pixt1);
@@ -90,7 +101,7 @@ L_REGPARAMS  *rp;
         pixaAddPix(pixa, pixt0, L_INSERT);
     }
     pixt1 = pixaDisplayTiledAndScaled(pixa, 32, w, 5, 0, 10, 2);
-    pixSaveTiled(pixt1, pixaf, 1, 1, 20, 0);
+    pixSaveTiled(pixt1, pixaf, 1.0, 1, 20, 0);
     regTestWritePixAndCheck(rp, pixt1, IFF_PNG);  /* 2 */
     pixDisplayWithTitle(pixt1, 600, 100, "Hue", rp->display);
     pixDestroy(&pixt1);
@@ -106,7 +117,7 @@ L_REGPARAMS  *rp;
         numaAddNumber(na, sat);
     }
     pixt1 = pixaDisplayTiledAndScaled(pixa, 32, w, 5, 0, 10, 2);
-    pixSaveTiled(pixt1, pixaf, 1, 1, 20, 0);
+    pixSaveTiled(pixt1, pixaf, 1.0, 1, 20, 0);
     gplotSimple1(na, GPLOT_PNG, "/tmp/enhance.7", "Average Saturation");
     regTestWritePixAndCheck(rp, pixt1, IFF_PNG);  /* 3 */
     pixDisplayWithTitle(pixt1, 900, 100, "Saturation", rp->display);
@@ -121,7 +132,7 @@ L_REGPARAMS  *rp;
         pixaAddPix(pixa, pixt0, L_INSERT);
     }
     pixt1 = pixaDisplayTiledAndScaled(pixa, 32, w, 5, 0, 10, 2);
-    pixSaveTiled(pixt1, pixaf, 1, 1, 20, 0);
+    pixSaveTiled(pixt1, pixaf, 1.0, 1, 20, 0);
     regTestWritePixAndCheck(rp, pixt1, IFF_PNG);  /* 4 */
     pixDisplayWithTitle(pixt1, 0, 400, "Contrast", rp->display);
     pixDestroy(&pixt1);
@@ -134,7 +145,7 @@ L_REGPARAMS  *rp;
         pixaAddPix(pixa, pixt0, L_INSERT);
     }
     pixt1 = pixaDisplayTiledAndScaled(pixa, 32, w, 5, 0, 10, 2);
-    pixSaveTiled(pixt1, pixaf, 1, 1, 20, 0);
+    pixSaveTiled(pixt1, pixaf, 1.0, 1, 20, 0);
     regTestWritePixAndCheck(rp, pixt1, IFF_PNG);  /* 5 */
     pixDisplayWithTitle(pixt1, 300, 400, "Sharp", rp->display);
     pixDestroy(&pixt1);
@@ -152,7 +163,7 @@ L_REGPARAMS  *rp;
         snprintf(textstr, 50, "Fract = %5.1f", fract);
         pixt2 = pixAddSingleTextblock(pixt1, bmf8, textstr, 0xff000000,
                                       L_ADD_BELOW, NULL);
-        pixSaveTiledOutline(pixt2, pixa, 1, (i % 4 == 0) ? 1 : 0, 30, 2, 32);
+        pixSaveTiledOutline(pixt2, pixa, 1.0, (i % 4 == 0) ? 1 : 0, 30, 2, 32);
         pixDestroy(&pixt1);
         pixDestroy(&pixt2);
     }
