@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -36,11 +47,11 @@
  *             Lossy: 0.43 sec, composed of:
  *                       0.22 sec (octree quant with dithering)
  *                       0.21 sec (to compress and write out)
- *             
+ *
  *       gif:  Lossy: 0.34 sec, composed of:
  *                       0.22 sec (octree quant with dithering)
- *                       0.12 sec (to compress and write out) 
- *             (note: no lossless mode; gif can't write out rgb)          
+ *                       0.12 sec (to compress and write out)
+ *             (note: no lossless mode; gif can't write out rgb)
  */
 
 #include <math.h>
@@ -66,12 +77,12 @@ static l_int32 test_mem_gif(const char *fname, l_int32 index);
 
 #define   REDUCTION     1
 
-main(int    argc,
-char **argv)
+int main(int    argc,
+         char **argv)
 {
 l_int32       success;
 L_REGPARAMS  *rp;
- 
+
 #if !HAVE_LIBGIF && !HAVE_LIBUNGIF
     fprintf(stderr, "gifio is not enabled\n"
             "libgif or libungif are required for gifio_reg\n"
@@ -101,7 +112,7 @@ L_REGPARAMS  *rp;
             "\n  ******* Failure on at least one r/w to file ******\n\n");
 
     if (rp->display)
-        pixDisplayMultiple("/tmp/junk_write_display*");
+        pixDisplayMultiple("/tmp/display/file*");
 
     /* ------------ Part 2: Test lossless r/w to memory ------------ */
     success = TRUE;
