@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -31,15 +42,15 @@ static const char  *kdatastr = " 20.3    50   80  50   20 "
                                " 53.7   100  140  100  50 "
                                " 24.9    50   80   50  20 ";
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 char         *str;
 l_int32       i, j, same, ok;
 l_float32     sum, avediff, rmsdiff;
 L_KERNEL     *kel1, *kel2, *kel3, *kel4, *kelx, *kely;
 BOX          *box;
-PIX          *pix, *pixs, *pixb, *pixg, *pixr, *pixd, *pixp, *pixt;
+PIX          *pix, *pixs, *pixb, *pixg, *pixd, *pixp, *pixt;
 PIX          *pixt1, *pixt2, *pixt3;
 PIXA         *pixa;
 SARRAY       *sa;
@@ -55,7 +66,7 @@ L_REGPARAMS  *rp;
     pixd = kernelDisplayInPix(kel1, 41, 2);
     pixWrite("/tmp/pixkern.png", pixd, IFF_PNG);
     regTestCheckFile(rp, "/tmp/pixkern.png");  /* 0 */
-    pixSaveTiled(pixd, pixa, 1, 1, 20, 8);
+    pixSaveTiled(pixd, pixa, 1.0, 1, 20, 8);
     pixDestroy(&pixd);
     kernelDestroy(&kel1);
 
@@ -84,7 +95,7 @@ L_REGPARAMS  *rp;
     l_binaryWrite("/tmp/kernfile.kel", "w", str, strlen(str));
     kel2 = kernelCreateFromFile("/tmp/kernfile.kel");
     pixd = kernelDisplayInPix(kel2, 41, 2);
-    pixSaveTiled(pixd, pixa, 1, 1, 20, 0);
+    pixSaveTiled(pixd, pixa, 1.0, 1, 20, 0);
     pixWrite("/tmp/ker1.png", pixd, IFF_PNG);
     regTestCheckFile(rp, "/tmp/ker1.png");  /* 4 */
     pixDestroy(&pixd);
@@ -111,7 +122,7 @@ L_REGPARAMS  *rp;
     pixSetPixel(pixt, 4, 2, 20);
     kel3 = kernelCreateFromPix(pixt, 1, 2);
     pixd = kernelDisplayInPix(kel3, 41, 2);
-    pixSaveTiled(pixd, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixd, pixa, 1.0, 0, 20, 0);
     pixWrite("/tmp/ker2.png", pixd, IFF_PNG);
     regTestCheckFile(rp, "/tmp/ker2.png");  /* 5 */
     pixDestroy(&pixd);
@@ -121,10 +132,10 @@ L_REGPARAMS  *rp;
         /* Test convolution with kel1 */
     pixs = pixRead("test24.jpg");
     pixg = pixScaleRGBToGrayFast(pixs, 3, COLOR_GREEN);
-    pixSaveTiled(pixg, pixa, 1, 1, 20, 0);
+    pixSaveTiled(pixg, pixa, 1.0, 1, 20, 0);
     kel1 = kernelCreateFromString(5, 5, 2, 2, kdatastr);
     pixd = pixConvolve(pixg, kel1, 8, 1);
-    pixSaveTiled(pixd, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixd, pixa, 1.0, 0, 20, 0);
     pixWrite("/tmp/ker3.png", pixd, IFF_PNG);
     regTestCheckFile(rp, "/tmp/ker3.png");  /* 6 */
     pixDestroy(&pixs);
@@ -138,18 +149,18 @@ L_REGPARAMS  *rp;
     pixg = pixScaleRGBToGrayFast(pixs, 3, COLOR_GREEN);
     kel2 = makeFlatKernel(11, 11, 5, 5);
     pixd = pixConvolve(pixg, kel2, 8, 1);
-    pixSaveTiled(pixd, pixa, 1, 1, 20, 0);
+    pixSaveTiled(pixd, pixa, 1.0, 1, 20, 0);
     pixWrite("/tmp/ker4.png", pixd, IFF_PNG);
     regTestCheckFile(rp, "/tmp/ker4.png");  /* 7 */
     pixt = pixBlockconv(pixg, 5, 5);
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 0);
     pixWrite("/tmp/ker5.png", pixt, IFF_PNG);
     regTestCheckFile(rp, "/tmp/ker5.png");  /* 8 */
     if (rp->display)
         pixCompareGray(pixd, pixt, L_COMPARE_ABS_DIFF, GPLOT_X11, NULL,
                        NULL, NULL, NULL);
     pixt2 = pixBlockconvTiled(pixg, 5, 5, 3, 6);
-    pixSaveTiled(pixt2, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixt2, pixa, 1.0, 0, 20, 0);
     pixWrite("/tmp/ker5a.png", pixt2, IFF_PNG);
     regTestCheckFile(rp, "/tmp/ker5a.png");  /* 9 */
     pixDestroy(&pixt2);
@@ -171,7 +182,7 @@ L_REGPARAMS  *rp;
         fprintf(stderr, "OK: Tiled results identical to pixConvolve()\n");
     else
         fprintf(stderr, "ERROR: Tiled results not identical to pixConvolve()\n");
-          
+
     pixDestroy(&pixs);
     pixDestroy(&pixg);
     pixDestroy(&pixd);
@@ -192,14 +203,14 @@ L_REGPARAMS  *rp;
     startTimer();
     pixt = pixConvolve(pixs, kel3, 8, 1);
     fprintf(stderr, "Generic convolution time: %5.3f sec\n", stopTimer());
-    pixSaveTiled(pixt, pixa, 1, 1, 20, 0);
+    pixSaveTiled(pixt, pixa, 1.0, 1, 20, 0);
     pixWrite("/tmp/conv1.png", pixt, IFF_PNG);
     regTestCheckFile(rp, "/tmp/conv1.png");  /* 10 */
 
     startTimer();
     pixt2 = pixBlockconv(pixs, 3, 3);
     fprintf(stderr, "Flat block convolution time: %5.3f sec\n", stopTimer());
-    pixSaveTiled(pixt2, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixt2, pixa, 1.0, 0, 20, 0);
     pixWrite("/tmp/conv2.png", pixt2, IFF_PNG);  /* ditto */
     regTestCheckFile(rp, "/tmp/conv2.png");  /* 11 */
 
@@ -210,8 +221,8 @@ L_REGPARAMS  *rp;
 #else
     Sleep(1000);
 #endif  /* _WIN32 */
-    pixp = pixRead("/tmp/grayroot.png");
-    pixSaveTiled(pixp, pixa, 1, 0, 20, 0);
+    pixp = pixRead("/tmp/grayroot0.png");
+    pixSaveTiled(pixp, pixa, 1.0, 0, 20, 0);
     pixWrite("/tmp/conv3.png", pixp, IFF_PNG);
     regTestCheckFile(rp, "/tmp/conv3.png");  /* 12 */
     fprintf(stderr, "Ave diff = %6.4f, RMS diff = %6.4f\n", avediff, rmsdiff);
@@ -265,29 +276,29 @@ L_REGPARAMS  *rp;
 
         /* Test generation and convolution with gaussian kernel */
     pixs = pixRead("test8.jpg");
-    pixSaveTiled(pixs, pixa, 1, 1, 20, 0);
+    pixSaveTiled(pixs, pixa, 1.0, 1, 20, 0);
     kel1 = makeGaussianKernel(5, 5, 3.0, 5.0);
     kernelGetSum(kel1, &sum);
     fprintf(stderr, "Sum for gaussian kernel = %f\n", sum);
     kernelWrite("/tmp/gauss.kel", kel1);
     pixt = pixConvolve(pixs, kel1, 8, 1);
     pixt2 = pixConvolve(pixs, kel1, 16, 0);
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 0);
-    pixSaveTiled(pixt2, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 0);
+    pixSaveTiled(pixt2, pixa, 1.0, 0, 20, 0);
     pixWrite("/tmp/ker6.png", pixt, IFF_PNG);
     regTestCheckFile(rp, "/tmp/ker6.png");  /* 18 */
     pixDestroy(&pixt);
     pixDestroy(&pixt2);
 
     pixt = kernelDisplayInPix(kel1, 25, 2);
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 0);
     pixDestroy(&pixt);
     kernelDestroy(&kel1);
     pixDestroy(&pixs);
 
         /* Test generation and convolution with separable gaussian kernel */
     pixs = pixRead("test8.jpg");
-    pixSaveTiled(pixs, pixa, 1, 1, 20, 0);
+    pixSaveTiled(pixs, pixa, 1.0, 1, 20, 0);
     makeGaussianKernelSep(5, 5, 3.0, 5.0, &kelx, &kely);
     kernelGetSum(kelx, &sum);
     fprintf(stderr, "Sum for x gaussian kernel = %f\n", sum);
@@ -298,18 +309,18 @@ L_REGPARAMS  *rp;
 
     pixt = pixConvolveSep(pixs, kelx, kely, 8, 1);
     pixt2 = pixConvolveSep(pixs, kelx, kely, 16, 0);
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 0);
-    pixSaveTiled(pixt2, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 0);
+    pixSaveTiled(pixt2, pixa, 1.0, 0, 20, 0);
     pixWrite("/tmp/ker7.png", pixt, IFF_PNG);
     regTestCheckFile(rp, "/tmp/ker7.png");  /* 19 */
     pixDestroy(&pixt);
     pixDestroy(&pixt2);
 
     pixt = kernelDisplayInPix(kelx, 25, 2);
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 0);
     pixDestroy(&pixt);
     pixt = kernelDisplayInPix(kely, 25, 2);
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 0);
     pixDestroy(&pixt);
     kernelDestroy(&kelx);
     kernelDestroy(&kely);
@@ -320,20 +331,20 @@ L_REGPARAMS  *rp;
     pixs = pixConvertRGBToLuminance(pixt);
     pixDestroy(&pixt); */
     pixs = pixRead("test8.jpg");
-    pixSaveTiled(pixs, pixa, 1, 1, 20, 0);
+    pixSaveTiled(pixs, pixa, 1.0, 1, 20, 0);
     kel1 = makeDoGKernel(7, 7, 1.5, 2.7);
     kernelGetSum(kel1, &sum);
     fprintf(stderr, "Sum for DoG kernel = %f\n", sum);
     kernelWrite("/tmp/dog.kel", kel1);
     pixt = pixConvolve(pixs, kel1, 8, 0);
 /*    pixInvert(pixt, pixt); */
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 0);
     pixWrite("/tmp/ker8.png", pixt, IFF_PNG);
     regTestCheckFile(rp, "/tmp/ker8.png");  /* 20 */
     pixDestroy(&pixt);
 
     pixt = kernelDisplayInPix(kel1, 20, 2);
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 0);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 0);
     pixDestroy(&pixt);
     kernelDestroy(&kel1);
     pixDestroy(&pixs);
