@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -32,8 +43,8 @@ static const l_int32 y1[NPATHS] = {383, 383, 112, 168, 371, 341};
 static const l_int32  NBOXES = 20;
 static const l_int32  POLARITY = 0;  /* background */
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 l_int32       i, w, h, bx, by, bw, bh, index, rval, gval, bval;
 BOX          *box;
@@ -46,21 +57,21 @@ PTAA         *ptaa;
 L_REGPARAMS  *rp;
 
     if (regTestSetup(argc, argv, &rp))
-	return 1;
+        return 1;
     pixa = pixaCreate(0);
 
     /* ---------------- Shortest path in binary maze ---------------- */
         /* Generate the maze */
     pixm = generateBinaryMaze(200, 200, 20, 20, 0.65, 0.25);
     pixd = pixExpandBinaryReplicate(pixm, 3);
-    pixSaveTiledOutline(pixd, pixa, 1, 1, 20, 2, 32);
+    pixSaveTiledOutline(pixd, pixa, 1.0, 1, 20, 2, 32);
     pixDestroy(&pixd);
 
         /* Find the shortest path between two points */
     pta = pixSearchBinaryMaze(pixm, 20, 20, 170, 170, NULL);
     pixt = pixDisplayPta(NULL, pixm, pta);
     pixd = pixScaleBySampling(pixt, 3., 3.);
-    pixSaveTiledOutline(pixd, pixa, 1, 0, 20, 2, 32);
+    pixSaveTiledOutline(pixd, pixa, 1.0, 0, 20, 2, 32);
     regTestWritePixAndCheck(rp, pixd, IFF_PNG);  /* 0 */
     ptaDestroy(&pta);
     pixDestroy(&pixt);
@@ -83,7 +94,7 @@ L_REGPARAMS  *rp;
 
     pixt = pixDisplayPtaa(pixg, ptaa);
     pixd = pixScaleBySampling(pixt, 2., 2.);
-    pixSaveTiledOutline(pixd, pixa, 1, 1, 20, 2, 32);
+    pixSaveTiledOutline(pixd, pixa, 1.0, 1, 20, 2, 32);
     regTestWritePixAndCheck(rp, pixd, IFF_PNG);  /* 1 */
     ptaaDestroy(&ptaa);
     pixDestroy(&pixg);
@@ -115,7 +126,7 @@ L_REGPARAMS  *rp;
                             rval, gval, bval);
         boxDestroy(&box);
     }
-    pixSaveTiledOutline(pixd, pixa, 1, 1, 20, 2, 32);
+    pixSaveTiledOutline(pixd, pixa, 1.0, 1, 20, 2, 32);
     regTestWritePixAndCheck(rp, pixd, IFF_PNG);  /* 2 */
     pixDestroy(&pixs);
     pixDestroy(&pixd);
