@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -21,8 +32,8 @@
 
 #define   SHOW    0
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 l_int32   w, h;
 BOXA     *boxa;
@@ -39,11 +50,11 @@ PIXCMAP  *cmap, *cmapg;
     pixm = pixRead("pageseg2-seed.png");
     pixd2 = pixScaleToGray2(pixd);
     pixs2 = pixScaleToGray2(pixs);
-    pixSaveTiled(pixd2, pixac, 2, 1, 40, 32);
-    pixSaveTiled(pixs2, pixac, 2, 0, 40, 0);
-    pixSaveTiled(pixm, pixac, 2, 0, 40, 0);
+    pixSaveTiled(pixd2, pixac, 0.5, 1, 40, 32);
+    pixSaveTiled(pixs2, pixac, 0.5, 0, 40, 0);
+    pixSaveTiled(pixm, pixac, 0.5, 0, 40, 0);
     pixCombineMaskedGeneral(pixd2, pixs2, pixm, 100, 100);
-    pixSaveTiled(pixd2, pixac, 2, 1, 40, 0);
+    pixSaveTiled(pixd2, pixac, 0.5, 1, 40, 0);
     pixDisplayWithTitle(pixd2, 100, 100, NULL, SHOW);
     pixDestroy(&pixd2);
     pixDestroy(&pixs2);
@@ -52,10 +63,10 @@ PIXCMAP  *cmap, *cmapg;
     pixm2 = pixExpandBinaryReplicate(pixm, 2);
     pixt1 = pixCopy(NULL, pixd);
     pixCombineMaskedGeneral(pixd, pixs, pixm2, 200, 200);
-    pixSaveTiled(pixd, pixac, 4, 0, 40, 0);
+    pixSaveTiled(pixd, pixac, 0.25, 0, 40, 0);
     pixDisplayWithTitle(pixd, 700, 100, NULL, SHOW);
     pixCombineMasked(pixt1, pixs, pixm2);
-    pixSaveTiled(pixt1, pixac, 4, 0, 40, 0);
+    pixSaveTiled(pixt1, pixac, 0.25, 0, 40, 0);
     pixDestroy(&pixd);
     pixDestroy(&pixt1);
     pixDestroy(&pixs);
@@ -66,9 +77,9 @@ PIXCMAP  *cmap, *cmapg;
     pixs = pixRead("pageseg2-seed.png");
     pixm = pixRead("pageseg2-mask.png");
     pixd = pixSeedfillBinaryRestricted(NULL, pixs, pixm, 8, 50, 175);
-    pixSaveTiled(pixs, pixac, 2, 1, 40, 0);
-    pixSaveTiled(pixm, pixac, 2, 0, 40, 0);
-    pixSaveTiled(pixd, pixac, 2, 0, 40, 0);
+    pixSaveTiled(pixs, pixac, 0.5, 1, 40, 0);
+    pixSaveTiled(pixm, pixac, 0.5, 0, 40, 0);
+    pixSaveTiled(pixd, pixac, 0.5, 0, 40, 0);
     pixDestroy(&pixs);
     pixDestroy(&pixm);
     pixDestroy(&pixd);
@@ -78,13 +89,13 @@ PIXCMAP  *cmap, *cmapg;
     pixGetDimensions(pixs, &w, &h, NULL);
     pixb = pixThresholdToBinary(pixs, 128);
     boxa = pixConnComp(pixb, &pixa, 8);
-    pixSaveTiled(pixs, pixac, 1, 1, 40, 0);
+    pixSaveTiled(pixs, pixac, 1.0, 1, 40, 0);
     cmap = pixcmapGrayToColor(0x6f90c0);
     pixSetColormap(pixs, cmap);
-    pixSaveTiled(pixs, pixac, 1, 0, 40, 0);
+    pixSaveTiled(pixs, pixac, 1.0, 0, 40, 0);
     pixc = pixaDisplayRandomCmap(pixa, w, h);
     pixcmapResetColor(pixGetColormap(pixc), 0, 255, 255, 255);
-    pixSaveTiled(pixc, pixac, 1, 0, 40, 0);
+    pixSaveTiled(pixc, pixac, 1.0, 0, 40, 0);
     pixDestroy(&pixs);
     pixDestroy(&pixb);
     pixDestroy(&pixc);
@@ -93,17 +104,17 @@ PIXCMAP  *cmap, *cmapg;
 
         /* Convert color to gray */
     pixs = pixRead("weasel4.16c.png");
-    pixSaveTiled(pixs, pixac, 1, 1, 20, 0);
+    pixSaveTiled(pixs, pixac, 1.0, 1, 20, 0);
     pixc = pixConvertTo32(pixs);
     pixt1 = pixConvertRGBToGray(pixc, 3., 7., 5.);
-    pixSaveTiled(pixt1, pixac, 1, 0, 20, 0);
+    pixSaveTiled(pixt1, pixac, 1.0, 0, 20, 0);
     pixt2 = pixConvertRGBToGrayFast(pixc);
-    pixSaveTiled(pixt2, pixac, 1, 0, 20, 0);
+    pixSaveTiled(pixt2, pixac, 1.0, 0, 20, 0);
     pixg = pixCopy(NULL, pixs);
     cmap = pixGetColormap(pixs);
     cmapg = pixcmapColorToGray(cmap, 4., 6., 3.);
     pixSetColormap(pixg, cmapg);
-    pixSaveTiled(pixg, pixac, 1, 0, 20, 0);
+    pixSaveTiled(pixg, pixac, 1.0, 0, 20, 0);
     pixDestroy(&pixs);
     pixDestroy(&pixc);
     pixDestroy(&pixt1);
@@ -115,7 +126,6 @@ PIXCMAP  *cmap, *cmapg;
     pixWrite("/tmp/misc1.png", pixd, IFF_PNG);
     pixDestroy(&pixd);
     pixaDestroy(&pixac);
-
     return 0;
 }
 
