@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -28,8 +39,8 @@ static const char *tempmtiff = "/tmp/junkmtiff";
 static const char *tempnewmtiff = "/tmp/junknewmtiff";
 
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 char        *filein, *fileout, *str, *fname, *filename;
 char         buffer[512];
@@ -43,7 +54,7 @@ SARRAY      *savals, *satypes, *sa;
 static char  mainName[] = "mtifftest";
 
     if (argc != 3)
-	exit(ERROR_INT(" Syntax:  mtifftest filein fileout", mainName, 1));
+        return ERROR_INT(" Syntax:  mtifftest filein fileout", mainName, 1);
 
     filein = argv[1];
     fileout = argv[2];
@@ -69,9 +80,9 @@ static char  mainName[] = "mtifftest";
 
 #if 0   /* ------------ Test single-to-multipage I/O  -------------------*/
         /* Use 'filein' to specify a directory of tiff files.
-	 * Read them in and generate a multipage tiff file.
-	 * Then convert that to a G4 compressed and ascii85 encoded
-	 * PS file. */
+         * Read them in and generate a multipage tiff file.
+         * Then convert that to a G4 compressed and ascii85 encoded
+         * PS file. */
     sa = getFilenamesInDirectory(filein);
     sarrayWriteStream(stderr, sa);
     sarraySort(sa, sa, L_SORT_INCREASING);
@@ -81,11 +92,11 @@ static char  mainName[] = "mtifftest";
         fname = sarrayGetString(sa, i, 0);
         filename = genPathname(filein, fname);
         pix = pixRead(filename);
-	if (!pix) continue;
-	if (i == 0)
-	    pixWriteTiff(tempmtiff, pix, IFF_TIFF_G4, "w+");
+        if (!pix) continue;
+        if (i == 0)
+            pixWriteTiff(tempmtiff, pix, IFF_TIFF_G4, "w+");
         else
-	    pixWriteTiff(tempmtiff, pix, IFF_TIFF_G4, "a");
+            pixWriteTiff(tempmtiff, pix, IFF_TIFF_G4, "a");
         pixDestroy(&pix);
         lept_free(filename);
     }
@@ -99,46 +110,46 @@ static char  mainName[] = "mtifftest";
         /* read count of tiff multipage */
     fp = lept_fopen(filein, "rb");
     if (fileFormatIsTiff(fp)) {
-	tiffGetCount(fp, &npages);
-	fprintf(stderr, " Tiff: %d page\n", npages);
+        tiffGetCount(fp, &npages);
+        fprintf(stderr, " Tiff: %d page\n", npages);
     }
-    else 
-	exit(ERROR_INT(" file not tiff", mainName, 1));
+    else
+        return ERROR_INT(" file not tiff", mainName, 1);
     lept_fclose(fp);
 
         /* split into separate page files */
     for (i = 0; i < npages + 1; i++) {   /* read one beyond to catch error */
-	pix = pixReadTiff(filein, i);
-	if (!pix) continue;
-	sprintf(buffer, "/tmp/junkout.%d.tif", i);
-	pixWrite(buffer, pix, IFF_TIFF_G4);
+        pix = pixReadTiff(filein, i);
+        if (!pix) continue;
+        sprintf(buffer, "/tmp/junkout.%d.tif", i);
+        pixWrite(buffer, pix, IFF_TIFF_G4);
         pixDestroy(&pix);
     }
 
         /* read separate page files and write reversed file */
     for (i = npages - 1; i >= 0; i--) {
-	sprintf(buffer, "/tmp/junkout.%d.tif", i);
+        sprintf(buffer, "/tmp/junkout.%d.tif", i);
         pix = pixRead(buffer);
-	if (!pix) continue;
-	if (i == npages - 1)
-	    pixWriteTiff(tempmtiff, pix, IFF_TIFF_G4, "w+");
+        if (!pix) continue;
+        if (i == npages - 1)
+            pixWriteTiff(tempmtiff, pix, IFF_TIFF_G4, "w+");
         else
-	    pixWriteTiff(tempmtiff, pix, IFF_TIFF_G4, "a");
+            pixWriteTiff(tempmtiff, pix, IFF_TIFF_G4, "a");
         pixDestroy(&pix);
     }
 
         /* read reversed file and reverse again */
     pixa = pixaCreate(npages);
     for (i = 0; i < 5; i++) {
-	pix = pixReadTiff(tempmtiff, i);
-	pixaAddPix(pixa, pix, L_INSERT);
+        pix = pixReadTiff(tempmtiff, i);
+        pixaAddPix(pixa, pix, L_INSERT);
     }
     for (i = npages - 1; i >= 0; i--) {
         pix = pixaGetPix(pixa, i, L_CLONE);
-	if (i == npages - 1)
-	    pixWriteTiff(tempnewmtiff, pix, IFF_TIFF_G4, "w+");
+        if (i == npages - 1)
+            pixWriteTiff(tempnewmtiff, pix, IFF_TIFF_G4, "w+");
         else
-	    pixWriteTiff(tempnewmtiff, pix, IFF_TIFF_G4, "a");
+            pixWriteTiff(tempnewmtiff, pix, IFF_TIFF_G4, "a");
         pixDestroy(&pix);
     }
     pixaDestroy(&pixa);
@@ -151,7 +162,7 @@ static char  mainName[] = "mtifftest";
     savals = sarrayCreate(10);
     satypes = sarrayCreate(10);
     nasizes = numaCreate(10);
-    
+
 /*    numaAddNumber(naflags, TIFFTAG_XMLPACKET);  */ /* XMP:  700 */
     numaAddNumber(naflags, 700);
     str = "<xmp>This is a Fake XMP packet</xmp>\n<text>Guess what ...?</text>";
@@ -166,7 +177,7 @@ static char  mainName[] = "mtifftest";
     numaAddNumber(naflags, 270);  /* IMAGEDESCRIPTION */
     sarrayAddString(savals, "One page of text", 1);
     sarrayAddString(satypes, "char*", 1);
-        /* the max sample is used by rendering programs 
+        /* the max sample is used by rendering programs
          * to scale the dynamic range */
     numaAddNumber(naflags, 281);  /* MAXSAMPLEVALUE */
     sarrayAddString(savals, "4", 1);
@@ -189,6 +200,6 @@ static char  mainName[] = "mtifftest";
     pixDestroy(&pix);
 #endif
 
-    exit(0);
+    return 0;
 }
 
