@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -41,8 +52,8 @@
 #include "allheaders.h"
 
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 char       textstr[256];
 l_int32    i, thresh, fgval, bgval;
@@ -59,7 +70,7 @@ PIXA      *pixa;
         scorefract = 0.1 * i;
         pixOtsuAdaptiveThreshold(pixg, 2000, 2000, 0, 0, scorefract,
                                  NULL, &pixb);
-        pixSaveTiledOutline(pixb, pixa, 2, 1, 20, 2, 32);
+        pixSaveTiledOutline(pixb, pixa, 0.5, 1, 20, 2, 32);
         pixSplitDistributionFgBg(pixg, scorefract, 1, &thresh, &fgval, &bgval, 1);
         fprintf(stderr, "thresh = %d, fgval = %d, bgval = %d\n", thresh, fgval,
                  bgval);
@@ -72,7 +83,7 @@ PIXA      *pixa;
 #endif  /* _WIN32 */
 
         pixp = pixRead("/tmp/histplot.png");
-        pixSaveTiled(pixp, pixa, 1, 0, 20, 1);
+        pixSaveTiled(pixp, pixa, 1.0, 0, 20, 1);
         pixt1 = pixaDisplay(pixa, 0, 0);
         snprintf(textstr, sizeof(textstr),
              "Scorefract = %3.1f ........... Thresh = %d", scorefract, thresh);
@@ -93,7 +104,7 @@ PIXA      *pixa;
         scorefract = 0.1 * i;
         pixOtsuAdaptiveThreshold(pixg, 300, 300, 0, 0, scorefract,
                                  NULL, &pixb);
-        pixb2 = pixAddBlackBorder(pixb, 2);
+        pixb2 = pixAddBlackOrWhiteBorder(pixb, 2, 2, 2, 2, L_GET_BLACK_VAL);
         snprintf(textstr, sizeof(textstr),
              "Scorefract = %3.1f", scorefract);
         pixb3 = pixAddSingleTextblock(pixb2, bmf, textstr, 1,
@@ -112,4 +123,3 @@ PIXA      *pixa;
     pixDestroy(&pixg);
     return 0;
 }
-
