@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -21,16 +32,15 @@
 
 #include "allheaders.h"
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 const char   *name;
 l_int32       i, n;
 BOX          *box;
-FILE         *fp;
-PIX          *pix, *pixt0, *pixt1, *pixd;
+PIX          *pix0, *pix1, *pixd;
 PIXA         *pixa;
-SARRAY       *sa1, *sa2, *sa3, *sa4, *sa;
+SARRAY       *sa1, *sa2, *sa3, *sa4;
 L_REGPARAMS  *rp;
 
     if (regTestSetup(argc, argv, &rp))
@@ -46,29 +56,29 @@ L_REGPARAMS  *rp;
     sarrayDestroy(&sa1);
     sarrayDestroy(&sa2);
     sarrayDestroy(&sa4);
-    
+
     /* ---------------- Use replace to fill up a pixa -------------------*/
     pixa = pixaCreate(1);
     pixaExtendArrayToSize(pixa, n);
-    if ((pixt0 = pixRead("marge.jpg")) == NULL)
+    if ((pix0 = pixRead("marge.jpg")) == NULL)
         rp->success = FALSE;
-    pixt1 = pixScaleToSize(pixt0, 144, 108);  /* scale 0.25 */
-    pixDestroy(&pixt0);
-    pixaInitFull(pixa, pixt1, NULL);  /* fill it up */
+    pix1 = pixScaleToSize(pix0, 144, 108);  /* scale 0.25 */
+    pixDestroy(&pix0);
+    pixaInitFull(pixa, pix1, NULL);  /* fill it up */
     pixd = pixaDisplayTiledInRows(pixa, 32, 1000, 1.0, 0, 25, 2);
     pixDisplayWithTitle(pixd, 100, 100, NULL, rp->display);
     pixWrite("/tmp/pix1.jpg", pixd, IFF_JFIF_JPEG);
-    pixDestroy(&pixt1);
+    pixDestroy(&pix1);
     pixDestroy(&pixd);
 
     /* ---------------- And again with jpgs and tifs -------------------*/
     for (i = 0; i < n; i++) {
         name = sarrayGetString(sa3, i, L_NOCOPY);
-        if ((pixt0 = pixRead(name)) == NULL)
+        if ((pix0 = pixRead(name)) == NULL)
             rp->success = FALSE;
-        pixt1 = pixScaleToSize(pixt0, 144, 108);
-        pixaReplacePix(pixa, i, pixt1, NULL);
-        pixDestroy(&pixt0);
+        pix1 = pixScaleToSize(pix0, 144, 108);
+        pixaReplacePix(pixa, i, pix1, NULL);
+        pixDestroy(&pix0);
     }
     pixd = pixaDisplayTiledInRows(pixa, 32, 1000, 1.0, 0, 25, 2);
     pixDisplayWithTitle(pixd, 400, 100, NULL, rp->display);
@@ -76,18 +86,16 @@ L_REGPARAMS  *rp;
     pixDestroy(&pixd);
 
     /* ---------------- And again, reversing the order ------------------*/
-    pix = pixCreate(1, 1, 1);
     box = boxCreate(0, 0, 0, 0);
-    pixaInitFull(pixa, pix, box);
-    pixDestroy(&pix);
+    pixaInitFull(pixa, NULL, box);
     boxDestroy(&box);
     for (i = 0; i < n; i++) {
         name = sarrayGetString(sa3, i, L_NOCOPY);
-        if ((pixt0 = pixRead(name)) == NULL)
+        if ((pix0 = pixRead(name)) == NULL)
             rp->success = FALSE;
-        pixt1 = pixScaleToSize(pixt0, 144, 108);
-        pixaReplacePix(pixa, n - 1 - i, pixt1, NULL);
-        pixDestroy(&pixt0);
+        pix1 = pixScaleToSize(pix0, 144, 108);
+        pixaReplacePix(pixa, n - 1 - i, pix1, NULL);
+        pixDestroy(&pix0);
     }
     pixd = pixaDisplayTiledInRows(pixa, 32, 1000, 1.0, 0, 25, 2);
     pixDisplayWithTitle(pixd, 700, 100, NULL, rp->display);
