@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -38,8 +49,8 @@ char *WeaselNames[] = {(char *)"weasel2.4c.png",
                        (char *)"weasel8.240c.png",
                        (char *)"weasel8.png",
                        (char *)"weasel32.png"};
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 l_int32       i, w, h;
 l_float32     factor, scale;
@@ -176,7 +187,7 @@ L_REGPARAMS  *rp;
     convertJpegToPS("marge.jpg", "/tmp/psio5.ps",
                     "a", 200, 900, 300, 2.0, 5, 1);
     regTestCheckFile(rp, "/tmp/psio5.ps");  /* 5 */
- 
+
         /* Generation using segmentation masks */
     convertSegmentedPagesToPS(".", "lion-page", ".", "lion-mask",
                               10, 0, 100, 2.0, 0.8, 190, "/tmp/psio6.ps");
@@ -195,7 +206,7 @@ L_REGPARAMS  *rp;
         /* Writing compressed from a pixa */
     sa = sarrayCreate(0);
     for (i = 0; i < 11; i++)
-        sarrayAddString(sa, WeaselNames[i], L_COPY); 
+        sarrayAddString(sa, WeaselNames[i], L_COPY);
     pixa = pixaReadFilesSA(sa);
     pixaWriteCompressedToPS(pixa, "/tmp/psio10.ps", 0, 3);
     regTestCheckFile(rp, "/tmp/psio10.ps");  /* 10 */
