@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -32,8 +43,8 @@
 static PIXA *PixSavePlots1(void);
 static PIXA *PixSavePlots2(void);
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 char          fname[256];
 l_int32       i, w, h, nbins, factor;
@@ -100,7 +111,7 @@ L_REGPARAMS  *rp;
     pixDestroy(&pixd);
 
         /* Now test the edge cases for the histogram and rank LUT,
-         * where all the histo data is piled up at one place. 
+         * where all the histo data is piled up at one place.
          * We only require that the result be sensible. */
     for (i = 0; i < 3; i++) {
         if (i == 0)
@@ -151,28 +162,28 @@ PIXA   *pixa;
 
     pixa = pixaCreate(8);
     pixt = pixRead("/tmp/rtnan.png");
-    pixSaveTiled(pixt, pixa, 1, 1, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 1, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnar.png");
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnai.png");
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnarbin.png");
-    pixSaveTiled(pixt, pixa, 1, 1, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 1, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnabb.png");
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnared.png");
-    pixSaveTiled(pixt, pixa, 1, 1, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 1, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnagreen.png");
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnablue.png");
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 8);
     pixDestroy(&pixt);
     return pixa;
 }
@@ -186,31 +197,31 @@ PIXA   *pixa;
 
     pixa = pixaCreate(9);
     pixt = pixRead("/tmp/rtnan1.png");
-    pixSaveTiled(pixt, pixa, 1, 1, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 1, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnai1.png");
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnarbin1.png");
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnan2.png");
-    pixSaveTiled(pixt, pixa, 1, 1, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 1, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnai2.png");
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnarbin2.png");
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnan3.png");
-    pixSaveTiled(pixt, pixa, 1, 1, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 1, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnai3.png");
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 8);
     pixDestroy(&pixt);
     pixt = pixRead("/tmp/rtnarbin3.png");
-    pixSaveTiled(pixt, pixa, 1, 0, 20, 8);
+    pixSaveTiled(pixt, pixa, 1.0, 0, 20, 8);
     pixDestroy(&pixt);
     return pixa;
 }
