@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -37,7 +48,7 @@ static const l_float32  ANGLE1 = 3.14159265 / 12.;
 static const l_float32  ANGLE2 = 3.14159265 / 120.;
 static const l_int32    NTIMES = 24;
 
-static void RotateTest(PIX *pixs, l_int32 reduction, L_REGPARAMS *rp);
+static void RotateTest(PIX *pixs, l_float32 scale, L_REGPARAMS *rp);
 
 
 l_int32 main(int    argc,
@@ -51,44 +62,44 @@ L_REGPARAMS  *rp;
 
     fprintf(stderr, "Test binary image:\n");
     pixs = pixRead(BINARY_IMAGE);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 2 bpp cmapped image with filled cmap:\n");
     pixs = pixRead(TWO_BPP_IMAGE);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 4 bpp cmapped image with unfilled cmap:\n");
     pixs = pixRead(FOUR_BPP_IMAGE1);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 4 bpp cmapped image with filled cmap:\n");
     pixs = pixRead(FOUR_BPP_IMAGE2);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 8 bpp grayscale image:\n");
     pixs = pixRead(EIGHT_BPP_IMAGE);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 8 bpp grayscale cmap image:\n");
     pixs = pixRead(EIGHT_BPP_CMAP_IMAGE1);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 8 bpp color cmap image:\n");
     pixs = pixRead(EIGHT_BPP_CMAP_IMAGE2);
     pixd = pixOctreeColorQuant(pixs, 200, 0);
-    RotateTest(pixs, 4, rp);
+    RotateTest(pixs, 0.25, rp);
     pixDestroy(&pixs);
     pixDestroy(&pixd);
 
     fprintf(stderr, "Test rgb image:\n");
     pixs = pixRead(RGB_IMAGE);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     return regTestCleanup(rp);
@@ -97,7 +108,7 @@ L_REGPARAMS  *rp;
 
 static void
 RotateTest(PIX          *pixs,
-           l_int32       reduction,
+           l_float32     scale,
            L_REGPARAMS  *rp)
 {
 l_int32   w, h, d, i, outformat;
@@ -113,10 +124,10 @@ PIXCMAP  *cmap;
     for (i = 1; i < NTIMES; i++) {
         if ((i % MODSIZE) == 0) {
             if (i == MODSIZE) {
-                pixSaveTiled(pixd, pixa, reduction, 1, 20, 32);
+                pixSaveTiled(pixd, pixa, scale, 1, 20, 32);
                 regTestWritePixAndCheck(rp, pixd, outformat);
             } else {
-                pixSaveTiled(pixd, pixa, reduction, 0, 20, 32);
+                pixSaveTiled(pixd, pixa, scale, 0, 20, 32);
                 regTestWritePixAndCheck(rp, pixd, outformat);
             }
         }
@@ -131,10 +142,10 @@ PIXCMAP  *cmap;
     for (i = 1; i < NTIMES; i++) {
         if ((i % MODSIZE) == 0) {
             if (i == MODSIZE) {
-                pixSaveTiled(pixd, pixa, reduction, 1, 20, 32);
+                pixSaveTiled(pixd, pixa, scale, 1, 20, 32);
                 regTestWritePixAndCheck(rp, pixd, outformat);
             } else {
-                pixSaveTiled(pixd, pixa, reduction, 0, 20, 32);
+                pixSaveTiled(pixd, pixa, scale, 0, 20, 32);
                 regTestWritePixAndCheck(rp, pixd, outformat);
             }
         }
@@ -149,10 +160,10 @@ PIXCMAP  *cmap;
     for (i = 1; i < NTIMES; i++) {
         if ((i % MODSIZE) == 0) {
             if (i == MODSIZE) {
-                pixSaveTiled(pixd, pixa, reduction, 1, 20, 32);
+                pixSaveTiled(pixd, pixa, scale, 1, 20, 32);
                 regTestWritePixAndCheck(rp, pixd, outformat);
             } else {
-                pixSaveTiled(pixd, pixa, reduction, 0, 20, 32);
+                pixSaveTiled(pixd, pixa, scale, 0, 20, 32);
                 regTestWritePixAndCheck(rp, pixd, outformat);
             }
         }
@@ -167,10 +178,10 @@ PIXCMAP  *cmap;
     for (i = 1; i < NTIMES; i++) {
         if ((i % MODSIZE) == 0) {
             if (i == MODSIZE) {
-                pixSaveTiled(pixd, pixa, reduction, 1, 20, 32);
+                pixSaveTiled(pixd, pixa, scale, 1, 20, 32);
                 regTestWritePixAndCheck(rp, pixd, outformat);
             } else {
-                pixSaveTiled(pixd, pixa, reduction, 0, 20, 32);
+                pixSaveTiled(pixd, pixa, scale, 0, 20, 32);
                 regTestWritePixAndCheck(rp, pixd, outformat);
             }
         }
@@ -185,10 +196,10 @@ PIXCMAP  *cmap;
         for (i = 1; i < NTIMES; i++) {
             if ((i % MODSIZE) == 0) {
                 if (i == MODSIZE) {
-                    pixSaveTiled(pixd, pixa, reduction, 1, 20, 32);
+                    pixSaveTiled(pixd, pixa, scale, 1, 20, 32);
                     regTestWritePixAndCheck(rp, pixd, outformat);
                 } else {
-                    pixSaveTiled(pixd, pixa, reduction, 0, 20, 32);
+                    pixSaveTiled(pixd, pixa, scale, 0, 20, 32);
                     regTestWritePixAndCheck(rp, pixd, outformat);
                 }
             }
