@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -34,11 +45,11 @@
 static const l_float32  ANGLE1 = 3.14159265 / 30.;
 static const l_float32  ANGLE2 = 3.14159265 / 7.;
 
-void RotateTest(PIX *pixs, l_int32 reduction, L_REGPARAMS *rp);
+void RotateTest(PIX *pixs, l_float32 scale, L_REGPARAMS *rp);
 
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 PIX          *pixs, *pixd;
 L_REGPARAMS  *rp;
@@ -48,44 +59,44 @@ L_REGPARAMS  *rp;
 
     fprintf(stderr, "Test binary image:\n");
     pixs = pixRead(BINARY_IMAGE);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 2 bpp cmapped image with filled cmap:\n");
     pixs = pixRead(TWO_BPP_IMAGE);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 4 bpp cmapped image with unfilled cmap:\n");
     pixs = pixRead(FOUR_BPP_IMAGE1);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 4 bpp cmapped image with filled cmap:\n");
     pixs = pixRead(FOUR_BPP_IMAGE2);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 8 bpp grayscale image:\n");
     pixs = pixRead(EIGHT_BPP_IMAGE);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 8 bpp grayscale cmap image:\n");
     pixs = pixRead(EIGHT_BPP_CMAP_IMAGE1);
-    RotateTest(pixs, 1, rp);
+    RotateTest(pixs, 1.0, rp);
     pixDestroy(&pixs);
 
     fprintf(stderr, "Test 8 bpp color cmap image:\n");
     pixs = pixRead(EIGHT_BPP_CMAP_IMAGE2);
     pixd = pixOctreeColorQuant(pixs, 200, 0);
-    RotateTest(pixd, 2, rp);
+    RotateTest(pixd, 0.5, rp);
     pixDestroy(&pixs);
     pixDestroy(&pixd);
 
     fprintf(stderr, "Test rgb image:\n");
     pixs = pixRead(RGB_IMAGE);
-    RotateTest(pixs, 4, rp);
+    RotateTest(pixs, 0.25, rp);
     pixDestroy(&pixs);
 
     return regTestCleanup(rp);
@@ -94,7 +105,7 @@ L_REGPARAMS  *rp;
 
 void
 RotateTest(PIX          *pixs,
-           l_int32       reduction,
+           l_float32     scale,
            L_REGPARAMS  *rp)
 {
 l_int32   w, h, d, outformat;
@@ -106,27 +117,27 @@ PIXA     *pixa;
 
     pixa = pixaCreate(0);
     pixt1 = pixRotate(pixs, ANGLE1, L_ROTATE_SHEAR, L_BRING_IN_WHITE, w, h);
-    pixSaveTiled(pixt1, pixa, reduction, 1, 20, 32);
+    pixSaveTiled(pixt1, pixa, scale, 1, 20, 32);
     pixt2 = pixRotate(pixs, ANGLE1, L_ROTATE_SHEAR, L_BRING_IN_BLACK, w, h);
-    pixSaveTiled(pixt2, pixa, reduction, 0, 20, 0);
+    pixSaveTiled(pixt2, pixa, scale, 0, 20, 0);
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
     pixt1 = pixRotate(pixs, ANGLE1, L_ROTATE_SHEAR, L_BRING_IN_WHITE, 0, 0);
-    pixSaveTiled(pixt1, pixa, reduction, 1, 20, 0);
+    pixSaveTiled(pixt1, pixa, scale, 1, 20, 0);
     pixt2 = pixRotate(pixs, ANGLE1, L_ROTATE_SHEAR, L_BRING_IN_BLACK, 0, 0);
-    pixSaveTiled(pixt2, pixa, reduction, 0, 20, 0);
+    pixSaveTiled(pixt2, pixa, scale, 0, 20, 0);
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
     pixt1 = pixRotate(pixs, ANGLE2, L_ROTATE_SHEAR, L_BRING_IN_WHITE, w, h);
-    pixSaveTiled(pixt1, pixa, reduction, 1, 20, 0);
+    pixSaveTiled(pixt1, pixa, scale, 1, 20, 0);
     pixt2 = pixRotate(pixs, ANGLE2, L_ROTATE_SHEAR, L_BRING_IN_BLACK, w, h);
-    pixSaveTiled(pixt2, pixa, reduction, 0, 20, 0);
+    pixSaveTiled(pixt2, pixa, scale, 0, 20, 0);
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
     pixt1 = pixRotate(pixs, ANGLE2, L_ROTATE_SHEAR, L_BRING_IN_WHITE, 0, 0);
-    pixSaveTiled(pixt1, pixa, reduction, 1, 20, 0);
+    pixSaveTiled(pixt1, pixa, scale, 1, 20, 0);
     pixt2 = pixRotate(pixs, ANGLE2, L_ROTATE_SHEAR, L_BRING_IN_BLACK, 0, 0);
-    pixSaveTiled(pixt2, pixa, reduction, 0, 20, 0);
+    pixSaveTiled(pixt2, pixa, scale, 0, 20, 0);
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
     pixd = pixaDisplay(pixa, 0, 0);
@@ -137,15 +148,15 @@ PIXA     *pixa;
 
     pixa = pixaCreate(0);
     pixt1 = pixRotate(pixs, ANGLE2, L_ROTATE_SAMPLING, L_BRING_IN_WHITE, w, h);
-    pixSaveTiled(pixt1, pixa, reduction, 1, 20, 32);
+    pixSaveTiled(pixt1, pixa, scale, 1, 20, 32);
     pixt2 = pixRotate(pixs, ANGLE2, L_ROTATE_SAMPLING, L_BRING_IN_BLACK, w, h);
-    pixSaveTiled(pixt2, pixa, reduction, 0, 20, 0);
+    pixSaveTiled(pixt2, pixa, scale, 0, 20, 0);
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
     pixt1 = pixRotate(pixs, ANGLE2, L_ROTATE_SAMPLING, L_BRING_IN_WHITE, 0, 0);
-    pixSaveTiled(pixt1, pixa, reduction, 1, 20, 0);
+    pixSaveTiled(pixt1, pixa, scale, 1, 20, 0);
     pixt2 = pixRotate(pixs, ANGLE2, L_ROTATE_SAMPLING, L_BRING_IN_BLACK, 0, 0);
-    pixSaveTiled(pixt2, pixa, reduction, 0, 20, 0);
+    pixSaveTiled(pixt2, pixa, scale, 0, 20, 0);
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
 
@@ -154,15 +165,15 @@ PIXA     *pixa;
     else
         pixt1 = pixClone(pixs);
     pixt2 = pixRotate(pixt1, ANGLE2, L_ROTATE_AREA_MAP, L_BRING_IN_WHITE, w, h);
-    pixSaveTiled(pixt2, pixa, reduction, 1, 20, 0);
+    pixSaveTiled(pixt2, pixa, scale, 1, 20, 0);
     pixt3 = pixRotate(pixt1, ANGLE2, L_ROTATE_AREA_MAP, L_BRING_IN_BLACK, w, h);
-    pixSaveTiled(pixt3, pixa, reduction, 0, 20, 0);
+    pixSaveTiled(pixt3, pixa, scale, 0, 20, 0);
     pixDestroy(&pixt2);
     pixDestroy(&pixt3);
     pixt2 = pixRotate(pixt1, ANGLE2, L_ROTATE_AREA_MAP, L_BRING_IN_WHITE, 0, 0);
-    pixSaveTiled(pixt2, pixa, reduction, 1, 20, 0);
+    pixSaveTiled(pixt2, pixa, scale, 1, 20, 0);
     pixt3 = pixRotate(pixt1, ANGLE2, L_ROTATE_AREA_MAP, L_BRING_IN_BLACK, 0, 0);
-    pixSaveTiled(pixt3, pixa, reduction, 0, 20, 0);
+    pixSaveTiled(pixt3, pixa, scale, 0, 20, 0);
     pixDestroy(&pixt2);
     pixDestroy(&pixt3);
     pixDestroy(&pixt1);
