@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -22,10 +33,10 @@
 
 #include "allheaders.h"
 
-#define  REDUCTION     1
+static const l_int32  scalefact = 1.0;
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 l_int32       i, j, x, y, val;
 PIX          *pixsq, *pixs, *pixc, *pixd;
@@ -55,7 +66,7 @@ L_REGPARAMS  *rp;
         y = (117 * i * i * i + 241) % 299;
         pixRasterop(pixc, x - 1, y - 1, 3, 3, PIX_SRC, pixsq, 0, 0);
     }
-    pixSaveTiled(pixc, pixa, REDUCTION, 1, 20, 32);
+    pixSaveTiled(pixc, pixa, scalefact, 1, 20, 32);
     regTestWritePixAndCheck(rp, pixc, IFF_PNG);  /* 0 */
     pixDisplayWithTitle(pixc, 100, 100, "4-cc", rp->display);
     pixDestroy(&pixd);
@@ -68,7 +79,7 @@ L_REGPARAMS  *rp;
         y = (117 * i * i * i + 241) % 299;
         pixRasterop(pixc, x - 1, y - 1, 3, 3, PIX_SRC, pixsq, 0, 0);
     }
-    pixSaveTiled(pixc, pixa, REDUCTION, 0, 20, 0);
+    pixSaveTiled(pixc, pixa, scalefact, 0, 20, 0);
     regTestWritePixAndCheck(rp, pixc, IFF_PNG);  /* 1 */
     pixDisplayWithTitle(pixc, 410, 100, "8-cc", rp->display);
     pixDestroy(&pixd);
@@ -89,7 +100,7 @@ L_REGPARAMS  *rp;
             pixRasterop(pixc, j - 1, i - 1, 3, 3, PIX_SRC, pixsq, 0, 0);
         }
     }
-    pixSaveTiled(pixc, pixa, REDUCTION, 1, 20, 0);
+    pixSaveTiled(pixc, pixa, scalefact, 1, 20, 0);
     regTestWritePixAndCheck(rp, pixc, IFF_PNG);  /* 2 */
     pixDisplayWithTitle(pixc, 100, 430, "4-cc", rp->display);
     pixDestroy(&pixd);
@@ -102,7 +113,7 @@ L_REGPARAMS  *rp;
             pixRasterop(pixc, j - 1, i - 1, 3, 3, PIX_SRC, pixsq, 0, 0);
         }
     }
-    pixSaveTiled(pixc, pixa, REDUCTION, 0, 20, 0);
+    pixSaveTiled(pixc, pixa, scalefact, 0, 20, 0);
     regTestWritePixAndCheck(rp, pixc, IFF_PNG);  /* 3 */
     pixDisplayWithTitle(pixc, 310, 430, "8-cc", rp->display);
     pixDestroy(&pixd);
@@ -121,7 +132,7 @@ L_REGPARAMS  *rp;
     pixRasterop(pixc, 160 - 1, 40 - 1, 3, 3, PIX_SRC, pixsq, 0, 0);
     pixRasterop(pixc, 80 - 1, 80 - 1, 3, 3, PIX_SRC, pixsq, 0, 0);
     pixRasterop(pixc, 40 - 1, 160 - 1, 3, 3, PIX_SRC, pixsq, 0, 0);
-    pixSaveTiled(pixc, pixa, REDUCTION, 1, 20, 0);
+    pixSaveTiled(pixc, pixa, scalefact, 1, 20, 0);
     regTestWritePixAndCheck(rp, pixc, IFF_PNG);  /* 4 */
     pixDisplayWithTitle(pixc, 100, 600, "4-cc", rp->display);
     pixDestroy(&pixd);
@@ -133,7 +144,7 @@ L_REGPARAMS  *rp;
     pixRasterop(pixc, 160 - 1, 40 - 1, 3, 3, PIX_SRC, pixsq, 0, 0);
     pixRasterop(pixc, 80 - 1, 80 - 1, 3, 3, PIX_SRC, pixsq, 0, 0);
     pixRasterop(pixc, 40 - 1, 160 - 1, 3, 3, PIX_SRC, pixsq, 0, 0);
-    pixSaveTiled(pixc, pixa, REDUCTION, 0, 20, 0);
+    pixSaveTiled(pixc, pixa, scalefact, 0, 20, 0);
     regTestWritePixAndCheck(rp, pixc, IFF_PNG);  /* 5 */
     pixDisplayWithTitle(pixc, 310, 660, "8-cc", rp->display);
     pixDestroy(&pixd);
