@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -20,8 +31,6 @@
  *    Uses gthumb for visually identifying problems.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include "allheaders.h"
 
 static const char  *opstr[3] = {"", "interpolated", "sampled"};
@@ -34,8 +43,8 @@ static const char  *dirstr[3] = {"", "to left", "to right"};
 #define  RUN_HORIZ_SHEAR          0
 #define  RUN_VERT_SHEAR           0
 
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 char       buf[256];
 l_int32    w, h, i, j, k, index, op, dir, stretch;
@@ -46,19 +55,19 @@ PIX       *pixs, *pixt, *pixt2, *pixd;
 static char  mainName[] = "warpertest";
 
     if (argc != 1)
-        exit(ERROR_INT("syntax: warpertest", mainName, 1));
+        return ERROR_INT("syntax: warpertest", mainName, 1);
 
     /* --------   Stereoscopic warping --------------*/
 #if RUN_WARP
     pixs = pixRead("german.png");
     pixGetDimensions(pixs, &w, &h, NULL);
-    l_jpegSetNoChromaSampling(1);
     for (i = 0; i < 50; i++) {  /* need to test > 2 widths ! */
         j = 7 * i;
         box = boxCreate(0, 0, w - j, h - j);
         pixt = pixClipRectangle(pixs, box, NULL);
         pixd = pixWarpStereoscopic(pixt, 15, 22, 8, 30, -20, 1);
         snprintf(buf, sizeof(buf), "/tmp/junkpixw.%02d.jpg", i);
+        pixSetChromaSampling(pixd, 0);
         pixWrite(buf, pixd, IFF_JFIF_JPEG);
         pixDestroy(&pixd);
         pixDestroy(&pixt);
@@ -87,7 +96,7 @@ static char  mainName[] = "warpertest";
         Box *box = boxCreate(0, 0, w - j, h - j);
         pixt = pixClipRectangle(pixs, box, NULL);
         pixt2 = pixQuadraticVShear(pixt, dir, 60, -20, op, L_BRING_IN_WHITE);
-        
+
         snprintf(buf, sizeof(buf), "%s, %s", dirstr[dir], opstr[op]);
         pixd = pixAddSingleTextblock(pixt2, bmf, buf, 0xff000000,
                                      L_ADD_BELOW, 0);
@@ -104,7 +113,7 @@ static char  mainName[] = "warpertest";
 #endif
 
     /* --------  Linear Horizontal stretching  --------------*/
-#if RUN_LIN_HORIZ_STRETCH 
+#if RUN_LIN_HORIZ_STRETCH
     pixs = pixRead("german.png");
     bmf = bmfCreate("./fonts", 6);
     for (k = 0; k < 2; k++) {
@@ -217,5 +226,3 @@ static char  mainName[] = "warpertest";
 
     return 0;
 }
-
-
