@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -34,13 +45,11 @@ static const l_int32  MIN_WORD_HEIGHT = 4;
 static const l_int32  MAX_WORD_WIDTH = 500;
 static const l_int32  MAX_WORD_HEIGHT = 100;
 
-
 #define   BUF_SIZE                  512
 #define   RENDER_PAGES              1
 
-
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
 char         filename[BUF_SIZE];
 char        *dirin, *rootname, *fname;
@@ -59,16 +68,16 @@ PIXCMAP     *cmap;
 static char  mainName[] = "wordsinorder";
 
     if (argc != 3 && argc != 5)
-	exit(ERROR_INT(
-	     " Syntax: wordsinorder dirin rootname [firstpage, npages]",
-	     mainName, 1));
+        return ERROR_INT(
+            " Syntax: wordsinorder dirin rootname [firstpage, npages]",
+            mainName, 1);
 
     dirin = argv[1];
     rootname = argv[2];
 
     if (argc == 3) {
         firstpage = 0;
-	npages = 0;
+        npages = 0;
     }
     else {
         firstpage = atoi(argv[3]);
@@ -84,20 +93,20 @@ static char  mainName[] = "wordsinorder";
     for (i = 0; i < nfiles; i++) {
         fname = sarrayGetString(safiles, i, 0);
         if ((pixs = pixRead(fname)) == NULL) {
-            L_WARNING_INT("image file %d not read", mainName, i);
+            L_WARNING("image file %d not read\n", mainName, i);
             continue;
         }
         pixGetWordBoxesInTextlines(pixs, 2, MIN_WORD_WIDTH, MIN_WORD_HEIGHT,
-			           MAX_WORD_WIDTH, MAX_WORD_HEIGHT,
-				   &boxa, &nai);
+                                   MAX_WORD_WIDTH, MAX_WORD_HEIGHT,
+                                   &boxa, &nai);
         boxaaAddBoxa(baa, boxa, L_INSERT);
         numaaAddNuma(naa, nai, L_INSERT);
-        
+
 #if  RENDER_PAGES
             /* Show the results on a 2x reduced image, where each
              * word is outlined and the color of the box depends on the
              * computed textline. */
-	pixt1 = pixReduceRankBinary2(pixs, 2, NULL);
+        pixt1 = pixReduceRankBinary2(pixs, 2, NULL);
         pixGetDimensions(pixt1, &w, &h, NULL);
         pixd = pixCreate(w, h, 8);
         cmap = pixcmapCreateRandom(8, 1, 1);  /* first color is black */
@@ -115,13 +124,13 @@ static char  mainName[] = "wordsinorder";
             boxDestroy(&box);
         }
 
-	snprintf(filename, BUF_SIZE, "%s.%05d", rootname, i);
-	fprintf(stderr, "filename: %s\n", filename);
-	pixWrite(filename, pixd, IFF_PNG);
-	pixDestroy(&pixt1);
-	pixDestroy(&pixt2);
-	pixDestroy(&pixs);
-	pixDestroy(&pixd);
+        snprintf(filename, BUF_SIZE, "%s.%05d", rootname, i);
+        fprintf(stderr, "filename: %s\n", filename);
+        pixWrite(filename, pixd, IFF_PNG);
+        pixDestroy(&pixt1);
+        pixDestroy(&pixt2);
+        pixDestroy(&pixs);
+        pixDestroy(&pixd);
 #endif  /* RENDER_PAGES */
     }
 
