@@ -1,21 +1,32 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
  *  binarize.c
- *                     
+ *
  *  ===================================================================
  *  Image binarization algorithms are found in:
  *    grayquant.c:   standard, simple, general grayscale quantization
@@ -250,7 +261,7 @@ PIX      *pixn, *pixt, *pixd;
     if (sx < 4 || sy < 4)
         return (PIX *)ERROR_PTR("sx and sy must be >= 4", procName, NULL);
     if (mincount > sx * sy) {
-        L_WARNING("mincount too large for tile size", procName);
+        L_WARNING("mincount too large for tile size\n", procName);
         mincount = (sx * sy) / 3;
     }
 
@@ -276,7 +287,7 @@ PIX      *pixn, *pixt, *pixd;
     else
         return pixd;
 }
-    
+
 
 
 /*----------------------------------------------------------------------*
@@ -345,7 +356,7 @@ PIX      *pixn, *pixm, *pixd, *pixt1, *pixt2, *pixt3, *pixt4;
     if (sx < 4 || sy < 4)
         return (PIX *)ERROR_PTR("sx and sy must be >= 4", procName, NULL);
     if (mincount > sx * sy) {
-        L_WARNING("mincount too large for tile size", procName);
+        L_WARNING("mincount too large for tile size\n", procName);
         mincount = (sx * sy) / 3;
     }
 
@@ -399,7 +410,7 @@ PIX      *pixn, *pixm, *pixd, *pixt1, *pixt2, *pixt3, *pixt4;
     else
         return pixd;
 }
-    
+
 
 /*----------------------------------------------------------------------*
  *                           Sauvola binarization                       *
@@ -446,7 +457,7 @@ PIX       **ptileth, **ptiled;
 PIXTILING  *pt;
 
     PROCNAME("pixSauvolaBinarizeTiled");
-    
+
     if (!ppixth && !ppixd)
         return ERROR_INT("no outputs", procName, 1);
     if (ppixth) *ppixth = NULL;
@@ -474,11 +485,11 @@ PIXTILING  *pt;
     yrat = h / ny;
     if (xrat < whsize + 2) {
         nx = w / (whsize + 2);
-        L_WARNING_INT("tile width too small; nx reduced to %d", procName, nx);
+        L_WARNING("tile width too small; nx reduced to %d\n", procName, nx);
     }
     if (yrat < whsize + 2) {
         ny = h / (whsize + 2);
-        L_WARNING_INT("tile height too small; ny reduced to %d", procName, ny);
+        L_WARNING("tile height too small; ny reduced to %d\n", procName, ny);
     }
     if (nx <= 1 && ny <= 1)
         return pixSauvolaBinarize(pixs, whsize, factor, 1, NULL, NULL,
@@ -571,7 +582,7 @@ PIX     *pixg, *pixsc, *pixm, *pixms, *pixth, *pixd;
 
     PROCNAME("pixSauvolaBinarize");
 
-    
+
     if (!ppixm && !ppixsd && !ppixth && !ppixd)
         return ERROR_INT("no outputs", procName, 1);
     if (ppixm) *ppixm = NULL;
@@ -594,8 +605,7 @@ PIX     *pixg, *pixsc, *pixm, *pixms, *pixth, *pixd;
         pixg = pixAddMirroredBorder(pixs, whsize + 1, whsize + 1,
                                     whsize + 1, whsize + 1);
         pixsc = pixClone(pixs);
-    }
-    else {
+    } else {
         pixg = pixClone(pixs);
         pixsc = pixRemoveBorder(pixs, whsize + 1);
     }
@@ -681,7 +691,7 @@ l_float32  *tab;  /* of 2^16 square roots */
 PIX        *pixsd, *pixd;
 
     PROCNAME("pixSauvolaGetThreshold");
-    
+
     if (ppixsd) *ppixsd = NULL;
     if (!pixm || pixGetDepth(pixm) != 8)
         return (PIX *)ERROR_PTR("pixm undefined or not 8 bpp", procName, NULL);
@@ -729,7 +739,7 @@ PIX        *pixsd, *pixd;
             if (usetab)
                 sd = tab[var];
             else
-                sd = (l_float32)sqrt(var);
+                sd = (l_float32)sqrt((l_float32)var);
             if (ppixsd) SET_DATA_BYTE(linesd, j, (l_int32)sd);
             thresh = (l_int32)(mv * (1.0 - factor * (1.0 - sd / 128.)));
             SET_DATA_BYTE(lined, j, thresh);
@@ -759,7 +769,7 @@ l_uint32  *datas, *datat, *datad, *lines, *linet, *lined;
 PIX       *pixd;
 
     PROCNAME("pixApplyLocalThreshold");
-    
+
     if (!pixs || pixGetDepth(pixs) != 8)
         return (PIX *)ERROR_PTR("pixs undefined or not 8 bpp", procName, NULL);
     if (pixGetColormap(pixs))
@@ -789,5 +799,3 @@ PIX       *pixd;
 
     return pixd;
 }
-
-
