@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -22,9 +33,14 @@
  *      Write bmp to file
  *           l_int32       pixWriteStreamBmp()
  *
- *      Read/write to memory   [only on linux]
+ *      Read/write to memory
  *           PIX          *pixReadMemBmp()
  *           l_int32       pixWriteMemBmp()
+ *
+ *    On systems like windows without fmemopen() and open_memstream(),
+ *    we write data to a temp file and read it back for operations
+ *    between pix and compressed-data, such as pixReadMemPng() and
+ *    pixWriteMemPng().
  */
 
 #include <string.h>
@@ -37,6 +53,9 @@
 
 RGBA_QUAD   bwmap[2] = { {255,255,255,0}, {0,0,0,0} };
 
+    /* Colormap size limit */
+static const l_int32  L_MAX_ALLOWED_NUM_COLORS = 256;
+
 #ifndef  NO_CONSOLE_IO
 #define  DEBUG     0
 #endif  /* ~NO_CONSOLE_IO */
@@ -60,11 +79,11 @@ l_uint16   sval;
 l_uint32   ival;
 l_int16    bfType, bfSize, bfFill1, bfReserved1, bfReserved2;
 l_int16    offset, bfFill2, biPlanes, depth, d;
-l_int32    biSize, width, height, xres, yres, compression, ignore;
+l_int32    biSize, width, height, xres, yres, compression;
 l_int32    imagebytes, biClrUsed, biClrImportant;
 l_uint8   *colormapBuf;
 l_int32    colormapEntries;
-l_int32    fileBpl, extrabytes, readerror;
+l_int32    fileBpl, extrabytes;
 l_int32    pixWpl, pixBpl;
 l_int32    i, j, k;
 l_uint8    pel[4];
@@ -79,51 +98,69 @@ PIXCMAP   *cmap;
         return (PIX *)ERROR_PTR("fp not defined", procName, NULL);
 
         /* Read bitmap file header */
-    ignore = fread((char *)&sval, 1, 2, fp);
+    if (fread((char *)&sval, 2, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 1 not read", procName, NULL);
     bfType = convertOnBigEnd16(sval);
     if (bfType != BMP_ID)
         return (PIX *)ERROR_PTR("not bmf format", procName, NULL);
 
-    ignore = fread((char *)&sval, 1, 2, fp);
+    if (fread((char *)&sval, 2, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 2 not read", procName, NULL);
     bfSize = convertOnBigEnd16(sval);
-    ignore = fread((char *)&sval, 1, 2, fp);
+    if (fread((char *)&sval, 2, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 3 not read", procName, NULL);
     bfFill1 = convertOnBigEnd16(sval);
-    ignore = fread((char *)&sval, 1, 2, fp);
+    if (fread((char *)&sval, 2, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 4 not read", procName, NULL);
     bfReserved1 = convertOnBigEnd16(sval);
-    ignore = fread((char *)&sval, 1, 2, fp);
+    if (fread((char *)&sval, 2, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 5 not read", procName, NULL);
     bfReserved2 = convertOnBigEnd16(sval);
-    ignore = fread((char *)&sval, 1, 2, fp);
+    if (fread((char *)&sval, 2, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 6 not read", procName, NULL);
     offset = convertOnBigEnd16(sval);
-    ignore = fread((char *)&sval, 1, 2, fp);
+    if (fread((char *)&sval, 2, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 7 not read", procName, NULL);
     bfFill2 = convertOnBigEnd16(sval);
 
         /* Read bitmap info header */
-    ignore = fread((char *)&ival, 1, 4, fp);
+    if (fread((char *)&ival, 4, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 8 not read", procName, NULL);
     biSize = convertOnBigEnd32(ival);
-    ignore = fread((char *)&ival, 1, 4, fp);
+    if (fread((char *)&ival, 4, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 9 not read", procName, NULL);
     width = convertOnBigEnd32(ival);
-    ignore = fread((char *)&ival, 1, 4, fp);
+    if (fread((char *)&ival, 4, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 10 not read", procName, NULL);
     height = convertOnBigEnd32(ival);
-    ignore = fread((char *)&sval, 1, 2, fp);
+    if (fread((char *)&sval, 2, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 11 not read", procName, NULL);
     biPlanes = convertOnBigEnd16(sval);
-    ignore = fread((char *)&sval, 1, 2, fp);
+    if (fread((char *)&sval, 2, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 12 not read", procName, NULL);
     depth = convertOnBigEnd16(sval);
-    ignore = fread((char *)&ival, 1, 4, fp);
+    if (fread((char *)&ival, 4, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 13 not read", procName, NULL);
     compression = convertOnBigEnd32(ival);
-    ignore = fread((char *)&ival, 1, 4, fp);
+    if (fread((char *)&ival, 4, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 14 not read", procName, NULL);
     imagebytes = convertOnBigEnd32(ival);
-    ignore = fread((char *)&ival, 1, 4, fp);
+    if (fread((char *)&ival, 4, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 15 not read", procName, NULL);
     xres = convertOnBigEnd32(ival);
-    ignore = fread((char *)&ival, 1, 4, fp);
+    if (fread((char *)&ival, 4, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 16 not read", procName, NULL);
     yres = convertOnBigEnd32(ival);
-    ignore = fread((char *)&ival, 1, 4, fp);
+    if (fread((char *)&ival, 4, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 17 not read", procName, NULL);
     biClrUsed = convertOnBigEnd32(ival);
-    ignore = fread((char *)&ival, 1, 4, fp);
+    if (fread((char *)&ival, 4, 1, fp) != 1)
+        return (PIX *)ERROR_PTR("item 18 not read", procName, NULL);
     biClrImportant = convertOnBigEnd32(ival);
 
     if (compression != 0)
         return (PIX *)ERROR_PTR("cannot read compressed BMP files",
-                                procName,NULL);
+                                procName, NULL);
 
         /* A little sanity checking.  It would be nice to check
          * if the number of bytes in the file equals the offset to
@@ -133,21 +170,26 @@ PIXCMAP   *cmap;
          * uncompressed images is either 0 or the size of the file data.
          * (The fact that it can be 0 is perhaps some legacy glitch).  */
     if (width < 1)
-        return (PIX *)ERROR_PTR("width < 1", procName,NULL);
+        return (PIX *)ERROR_PTR("width < 1", procName, NULL);
     if (height < 1)
-        return (PIX *)ERROR_PTR("height < 1", procName,NULL);
-    if (depth < 1 || depth > 32)
-        return (PIX *)ERROR_PTR("depth not in [1 ... 32]", procName,NULL);
+        return (PIX *)ERROR_PTR("height < 1", procName, NULL);
+    if (depth != 1 && depth != 2 && depth != 4 && depth != 8 &&
+        depth != 16 && depth != 24 && depth != 32)
+        return (PIX *)ERROR_PTR("depth not in {1, 2, 4, 8, 16, 24, 32}",
+                                procName,NULL);
     fileBpl = 4 * ((width * depth + 31)/32);
     if (imagebytes != 0 && imagebytes != fileBpl * height)
-        return (PIX *)ERROR_PTR("invalid imagebytes", procName,NULL);
+        return (PIX *)ERROR_PTR("invalid imagebytes", procName, NULL);
     if (offset < BMP_FHBYTES + BMP_IHBYTES)
-        return (PIX *)ERROR_PTR("invalid offset: too small", procName,NULL);
+        return (PIX *)ERROR_PTR("invalid offset: too small", procName, NULL);
     if (offset > BMP_FHBYTES + BMP_IHBYTES + 4 * 256)
-        return (PIX *)ERROR_PTR("invalid offset: too large", procName,NULL);
+        return (PIX *)ERROR_PTR("invalid offset: too large", procName, NULL);
 
         /* Handle the colormap */
     colormapEntries = (offset - BMP_FHBYTES - BMP_IHBYTES) / sizeof(RGBA_QUAD);
+    colormapBuf = NULL;
+    if (colormapEntries > L_MAX_ALLOWED_NUM_COLORS)
+        return (PIX *)ERROR_PTR("colormap too large", procName,NULL);
     if (colormapEntries > 0) {
         if ((colormapBuf = (l_uint8 *)CALLOC(colormapEntries,
                                              sizeof(RGBA_QUAD))) == NULL)
@@ -165,8 +207,10 @@ PIXCMAP   *cmap;
     d = depth;
     if (depth == 24)
         d = 32;
-    if ((pix = pixCreate(width, height, d)) == NULL)
+    if ((pix = pixCreate(width, height, d)) == NULL) {
+        FREE(colormapBuf);
         return (PIX *)ERROR_PTR( "pix not made", procName, NULL);
+    }
     pixSetXRes(pix, (l_int32)((l_float32)xres / 39.37 + 0.5));  /* to ppi */
     pixSetYRes(pix, (l_int32)((l_float32)yres / 39.37 + 0.5));  /* to ppi */
     pixWpl = pixGetWpl(pix);
@@ -174,7 +218,7 @@ PIXCMAP   *cmap;
 
     cmap = NULL;
     if (colormapEntries > 256)
-        L_WARNING("more than 256 colormap entries!", procName);
+        L_WARNING("more than 256 colormap entries!\n", procName);
     if (colormapEntries > 0) {  /* import the colormap to the pix cmap */
         cmap = pixcmapCreate(L_MIN(d, 8));
         FREE(cmap->array);  /* remove generated cmap array */
@@ -195,8 +239,7 @@ PIXCMAP   *cmap;
             }
             data -= pixBpl;
         }
-    }
-    else {  /*  24 bpp file; 32 bpp pix
+    } else {  /*  24 bpp file; 32 bpp pix
              *  Note: for bmp files, pel[0] is blue, pel[1] is green,
              *  and pel[2] is red.  This is opposite to the storage
              *  in the pix, which puts the red pixel in the 0 byte,
@@ -233,28 +276,30 @@ PIXCMAP   *cmap;
              *          3  (R)     2  (G)        1  (B)        0
              *      |-----------|------------|-----------|-----------|
              */
-        readerror = 0;
         extrabytes = fileBpl - 3 * width;
         line = pixGetData(pix) + pixWpl * (height - 1);
         for (i = 0; i < height; i++) {
             for (j = 0; j < width; j++) {
                 pword = line + j;
-                if (fread(&pel, 1, 3, fp) != 3)
-                    readerror = 1;
+                if (fread(&pel, 1, 3, fp) != 3) {
+                    pixDestroy(&pix);
+                    return (PIX *)ERROR_PTR("bmp(1) read fail", procName, NULL);
+                }
                 *((l_uint8 *)pword + COLOR_RED) = pel[2];
                 *((l_uint8 *)pword + COLOR_GREEN) = pel[1];
                 *((l_uint8 *)pword + COLOR_BLUE) = pel[0];
             }
             if (extrabytes) {
-                for (k = 0; k < extrabytes; k++)
-                    ignore = fread(&pel, 1, 1, fp);
+                for (k = 0; k < extrabytes; k++) {
+                    if (fread(&pel, 1, 1, fp) != 1) {
+                        pixDestroy(&pix);
+                        return (PIX *)ERROR_PTR("bmp(2) read fail",
+                                                procName, NULL);
+                    }
+                }
             }
             line -= pixWpl;
         }
-        if (readerror) {
-            pixDestroy(&pix);
-            return (PIX *)ERROR_PTR("BMP read fail", procName, NULL);
-        }
     }
 
     pixEndianByteSwap(pix);
@@ -269,7 +314,6 @@ PIXCMAP   *cmap;
          * We have no need for a 1 bpp pix with a colormap!
          * ---------------------------------------------- */
     if (depth == 1 && cmap) {
-/*        L_INFO("Removing colormap", procName); */
         pixt = pixRemoveColormap(pix, REMOVE_CMAP_BASED_ON_SRC);
         pixDestroy(&pix);
         pix = pixt;  /* rename */
@@ -328,7 +372,7 @@ RGBA_QUAD  *pquad;
     height = pixGetHeight(pix);
     d  = pixGetDepth(pix);
     if (d == 2)
-        L_WARNING("writing 2 bpp bmp file; nobody else can read", procName);
+        L_WARNING("writing 2 bpp bmp file; nobody else can read\n", procName);
     depth = d;
     if (d == 32)
         depth = 24;
@@ -345,19 +389,16 @@ RGBA_QUAD  *pquad;
     if (d == 32) {   /* 24 bpp rgb; no colormap */
         ncolors = 0;
         cmaplen = 0;
-    }
-    else if ((cmap = pixGetColormap(pix))) {   /* existing colormap */
+    } else if ((cmap = pixGetColormap(pix))) {   /* existing colormap */
         ncolors = pixcmapGetCount(cmap);
         cmaplen = ncolors * sizeof(RGBA_QUAD);
         cta = (l_uint8 *)cmap->array;
-    }
-    else {   /* no existing colormap; make a binary or gray one */
+    } else {   /* no existing colormap; make a binary or gray one */
         if (d == 1) {
             cmaplen  = sizeof(bwmap);
             ncolors = 2;
             cta = (l_uint8 *)bwmap;
-        }
-        else {   /* d != 32; output grayscale version */
+        } else {   /* d != 32; output grayscale version */
             ncolors = 1 << depth;
             cmaplen = ncolors * sizeof(RGBA_QUAD);
 
@@ -459,8 +500,7 @@ RGBA_QUAD  *pquad;
                 writeerror = 1;
             data -= pixBpl;
         }
-    }
-    else {  /* 32 bpp pix; 24 bpp file
+    } else {  /* 32 bpp pix; 24 bpp file
              * See the comments in pixReadStreamBMP() to
              * understand the logic behind the pixel ordering below.
              * Note that we have again done an endian swap on
@@ -508,9 +548,9 @@ RGBA_QUAD  *pquad;
 #endif  /* HAVE_CONFIG_H */
 
 #if HAVE_FMEMOPEN
-
 extern FILE *open_memstream(char **data, size_t *size);
 extern FILE *fmemopen(void *data, size_t size, const char *mode);
+#endif  /* HAVE_FMEMOPEN */
 
 /*!
  *  pixReadMemBmp()
@@ -526,20 +566,26 @@ PIX *
 pixReadMemBmp(const l_uint8  *cdata,
               size_t          size)
 {
-l_uint8  *data;
-FILE     *fp;
-PIX      *pix;
+FILE  *fp;
+PIX   *pix;
 
     PROCNAME("pixReadMemBmp");
 
     if (!cdata)
         return (PIX *)ERROR_PTR("cdata not defined", procName, NULL);
 
-    data = (l_uint8 *)cdata;  /* we're really not going to change this */
-    if ((fp = fmemopen(data, size, "r")) == NULL)
+#if HAVE_FMEMOPEN
+    if ((fp = fmemopen((l_uint8 *)cdata, size, "r")) == NULL)
         return (PIX *)ERROR_PTR("stream not opened", procName, NULL);
+#else
+    L_WARNING("work-around: writing to a temp file\n", procName);
+    fp = tmpfile();
+    fwrite(cdata, 1, size, fp);
+    rewind(fp);
+#endif  /* HAVE_FMEMOPEN */
     pix = pixReadStreamBmp(fp);
     fclose(fp);
+    if (!pix) L_ERROR("pix not read\n", procName);
     return pix;
 }
 
@@ -573,38 +619,21 @@ FILE    *fp;
     if (!pix)
         return ERROR_INT("&pix not defined", procName, 1 );
 
+#if HAVE_FMEMOPEN
     if ((fp = open_memstream((char **)pdata, psize)) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
     ret = pixWriteStreamBmp(fp, pix);
+#else
+    L_WARNING("work-around: writing to a temp file\n", procName);
+    fp = tmpfile();
+    ret = pixWriteStreamBmp(fp, pix);
+    rewind(fp);
+    *pdata = l_binaryReadStream(fp, psize);
+#endif  /* HAVE_FMEMOPEN */
     fclose(fp);
     return ret;
 }
 
-#else
-
-PIX *
-pixReadMemBmp(const l_uint8  *cdata,
-              size_t          size)
-{
-    return (PIX *)ERROR_PTR(
-        "bmp read from memory not implemented on this platform",
-        "pixReadMemBmp", NULL);
-}
-
-
-l_int32
-pixWriteMemBmp(l_uint8  **pdata,
-               size_t    *psize,
-               PIX       *pix)
-{
-    return ERROR_INT(
-        "bmp write to memory not implemented on this platform",
-        "pixWriteMemBmp", 1);
-}
-
-#endif  /* HAVE_FMEMOPEN */
-
 /* --------------------------------------------*/
 #endif  /* USE_BMPIO */
 /* --------------------------------------------*/
-
