@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -53,7 +64,7 @@
     /* For more than this number of c.c. in a binarized image of
      * semi-perimeter (w + h) about 5000 or less, the O(n) binsort
      * is faster than the O(nlogn) shellsort.  */
-static const l_int32   MIN_COMPS_FOR_BIN_SORT = 500;
+static const l_int32   MIN_COMPS_FOR_BIN_SORT = 200;
 
 
 /*---------------------------------------------------------------------*
@@ -61,7 +72,7 @@ static const l_int32   MIN_COMPS_FOR_BIN_SORT = 500;
  *---------------------------------------------------------------------*/
 /*!
  *  boxaTransform()
- * 
+ *
  *      Input:  boxa
  *              shiftx, shifty
  *              scalex, scaley
@@ -102,7 +113,7 @@ BOXA    *boxad;
 
 /*!
  *  boxTransform()
- * 
+ *
  *      Input:  box
  *              shiftx, shifty
  *              scalex, scaley
@@ -135,9 +146,9 @@ boxTransform(BOX       *box,
 
 /*!
  *  boxaTransformOrdered()
- * 
+ *
  *      Input:  boxa
- *              shiftx, shifty 
+ *              shiftx, shifty
  *              scalex, scaley
  *              xcen, ycen (center of rotation)
  *              angle (in radians; clockwise is positive)
@@ -202,9 +213,9 @@ BOXA    *boxad;
 
 /*!
  *  boxTransformOrdered()
- * 
+ *
  *      Input:  boxs
- *              shiftx, shifty 
+ *              shiftx, shifty
  *              scalex, scaley
  *              xcen, ycen (center of rotation)
  *              angle (in radians; clockwise is positive)
@@ -288,9 +299,9 @@ BOX       *boxd;
         th = (l_int32)(L_MAX(1.0, scaley * bh + 0.5));
         xcent = (l_int32)(scalex * xcen + 0.5);
         ycent = (l_int32)(scaley * ycen + 0.5);
-        if (angle == 0.0)
+        if (angle == 0.0) {
             boxd = boxCreate(tx, ty, tw, th);
-        else {
+        } else {
             xdif = tx + 0.5 * tw - xcent;
             ydif = ty + 0.5 * th - ycent;
             rw = L_ABS(tw * cosa) + L_ABS(th * sina);
@@ -300,17 +311,16 @@ BOX       *boxd;
             boxd = boxCreate((l_int32)rx, (l_int32)ry, (l_int32)rw,
                              (l_int32)rh);
         }
-    }
-    else if (order == L_SC_TR_RO) {
+    } else if (order == L_SC_TR_RO) {
         tx = (l_int32)(scalex * bx + shiftx + 0.5);
         ty = (l_int32)(scaley * by + shifty + 0.5);
         tw = (l_int32)(L_MAX(1.0, scalex * bw + 0.5));
         th = (l_int32)(L_MAX(1.0, scaley * bh + 0.5));
         xcent = (l_int32)(scalex * xcen + 0.5);
         ycent = (l_int32)(scaley * ycen + 0.5);
-        if (angle == 0.0)
+        if (angle == 0.0) {
             boxd = boxCreate(tx, ty, tw, th);
-        else {
+        } else {
             xdif = tx + 0.5 * tw - xcent;
             ydif = ty + 0.5 * th - ycent;
             rw = L_ABS(tw * cosa) + L_ABS(th * sina);
@@ -320,15 +330,13 @@ BOX       *boxd;
             boxd = boxCreate((l_int32)rx, (l_int32)ry, (l_int32)rw,
                              (l_int32)rh);
         }
-    }
-    else if (order == L_RO_TR_SC) {
+    } else if (order == L_RO_TR_SC) {
         if (angle == 0.0) {
             rx = bx;
             ry = by;
             rw = bw;
             rh = bh;
-        }
-        else {
+        } else {
             xdif = bx + 0.5 * bw - xcen;
             ydif = by + 0.5 * bh - ycen;
             rw = L_ABS(bw * cosa) + L_ABS(bh * sina);
@@ -341,15 +349,13 @@ BOX       *boxd;
         tw = (l_int32)(L_MAX(1.0, scalex * rw + 0.5));
         th = (l_int32)(L_MAX(1.0, scaley * rh + 0.5));
         boxd = boxCreate(tx, ty, tw, th);
-    }
-    else if (order == L_RO_SC_TR) {
+    } else if (order == L_RO_SC_TR) {
         if (angle == 0.0) {
             rx = bx;
             ry = by;
             rw = bw;
             rh = bh;
-        }
-        else {
+        } else {
             xdif = bx + 0.5 * bw - xcen;
             ydif = by + 0.5 * bh - ycen;
             rw = L_ABS(bw * cosa) + L_ABS(bh * sina);
@@ -362,8 +368,7 @@ BOX       *boxd;
         tw = (l_int32)(L_MAX(1.0, scalex * rw + 0.5));
         th = (l_int32)(L_MAX(1.0, scaley * rh + 0.5));
         boxd = boxCreate(tx, ty, tw, th);
-    }
-    else if (order == L_TR_RO_SC) {
+    } else if (order == L_TR_RO_SC) {
         tx = bx + shiftx;
         ty = by + shifty;
         if (angle == 0.0) {
@@ -371,8 +376,7 @@ BOX       *boxd;
             ry = ty;
             rw = bw;
             rh = bh;
-        }
-        else {
+        } else {
             xdif = tx + 0.5 * bw - xcen;
             ydif = ty + 0.5 * bh - ycen;
             rw = L_ABS(bw * cosa) + L_ABS(bh * sina);
@@ -385,8 +389,7 @@ BOX       *boxd;
         tw = (l_int32)(L_MAX(1.0, scalex * rw + 0.5));
         th = (l_int32)(L_MAX(1.0, scaley * rh + 0.5));
         boxd = boxCreate(tx, ty, tw, th);
-    }
-    else {  /* order == L_SC_RO_TR) */
+    } else {  /* order == L_SC_RO_TR) */
         tx = (l_int32)(scalex * bx + 0.5);
         ty = (l_int32)(scaley * by + 0.5);
         tw = (l_int32)(L_MAX(1.0, scalex * bw + 0.5));
@@ -398,8 +401,7 @@ BOX       *boxd;
             ry = ty;
             rw = tw;
             rh = th;
-        }
-        else {
+        } else {
             xdif = tx + 0.5 * tw - xcent;
             ydif = ty + 0.5 * th - ycent;
             rw = L_ABS(tw * cosa) + L_ABS(th * sina);
@@ -420,7 +422,7 @@ BOX       *boxd;
 
 /*!
  *  boxaRotateOrth()
- * 
+ *
  *      Input:  boxa
  *              w, h (of image in which the boxa is embedded)
  *              rotation (0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
@@ -444,10 +446,10 @@ BOXA    *boxad;
 
     if (!boxas)
         return (BOXA *)ERROR_PTR("boxas not defined", procName, NULL);
+    if (rotation < 0 || rotation > 3)
+        return (BOXA *)ERROR_PTR("rotation not in {0,1,2,3}", procName, NULL);
     if (rotation == 0)
         return boxaCopy(boxas, L_COPY);
-    if (rotation < 1 || rotation > 3)
-        return (BOXA *)ERROR_PTR("rotation not in {0,1,2,3}", procName, NULL);
 
     n = boxaGetCount(boxas);
     if ((boxad = boxaCreate(n)) == NULL)
@@ -466,7 +468,7 @@ BOXA    *boxad;
 
 /*!
  *  boxRotateOrth()
- * 
+ *
  *      Input:  box
  *              w, h (of image in which the box is embedded)
  *              rotation (0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
@@ -490,10 +492,10 @@ l_int32  bx, by, bw, bh, xdist, ydist;
 
     if (!box)
         return (BOX *)ERROR_PTR("box not defined", procName, NULL);
+    if (rotation < 0 || rotation > 3)
+        return (BOX *)ERROR_PTR("rotation not in {0,1,2,3}", procName, NULL);
     if (rotation == 0)
         return boxCopy(box);
-    if (rotation < 1 || rotation > 3)
-        return (BOX *)ERROR_PTR("rotation not in {0,1,2,3}", procName, NULL);
 
     boxGetGeometry(box, &bx, &by, &bw, &bh);
     if (bw <= 0 || bh <= 0)  /* invalid */
@@ -514,12 +516,14 @@ l_int32  bx, by, bw, bh, xdist, ydist;
  *---------------------------------------------------------------------*/
 /*!
  *  boxaSort()
- * 
+ *
  *      Input:  boxa
- *              sorttype (L_SORT_BY_X, L_SORT_BY_Y, L_SORT_BY_WIDTH,
- *                        L_SORT_BY_HEIGHT, L_SORT_BY_MIN_DIMENSION,
- *                        L_SORT_BY_MAX_DIMENSION, L_SORT_BY_PERIMETER,
- *                        L_SORT_BY_AREA, L_SORT_BY_ASPECT_RATIO)
+ *              sorttype (L_SORT_BY_X, L_SORT_BY_Y,
+ *                        L_SORT_BY_RIGHT, L_SORT_BY_BOT,
+ *                        L_SORT_BY_WIDTH, L_SORT_BY_HEIGHT,
+ *                        L_SORT_BY_MIN_DIMENSION, L_SORT_BY_MAX_DIMENSION,
+ *                        L_SORT_BY_PERIMETER, L_SORT_BY_AREA,
+ *                        L_SORT_BY_ASPECT_RATIO)
  *              sortorder  (L_SORT_INCREASING, L_SORT_DECREASING)
  *              &naindex (<optional return> index of sorted order into
  *                        original array)
@@ -540,7 +544,8 @@ NUMA      *na, *naindex;
     if (pnaindex) *pnaindex = NULL;
     if (!boxas)
         return (BOXA *)ERROR_PTR("boxas not defined", procName, NULL);
-    if (sorttype != L_SORT_BY_X && sorttype != L_SORT_BY_Y && 
+    if (sorttype != L_SORT_BY_X && sorttype != L_SORT_BY_Y &&
+        sorttype != L_SORT_BY_RIGHT && sorttype != L_SORT_BY_BOT &&
         sorttype != L_SORT_BY_WIDTH && sorttype != L_SORT_BY_HEIGHT &&
         sorttype != L_SORT_BY_MIN_DIMENSION &&
         sorttype != L_SORT_BY_MAX_DIMENSION &&
@@ -553,8 +558,8 @@ NUMA      *na, *naindex;
 
         /* Use O(n) binsort if possible */
     n = boxaGetCount(boxas);
-    if (n > MIN_COMPS_FOR_BIN_SORT && 
-        ((sorttype == L_SORT_BY_X) || (sorttype == L_SORT_BY_Y) || 
+    if (n > MIN_COMPS_FOR_BIN_SORT &&
+        ((sorttype == L_SORT_BY_X) || (sorttype == L_SORT_BY_Y) ||
          (sorttype == L_SORT_BY_WIDTH) || (sorttype == L_SORT_BY_HEIGHT) ||
          (sorttype == L_SORT_BY_PERIMETER)))
         return boxaBinSort(boxas, sorttype, sortorder, pnaindex);
@@ -572,6 +577,12 @@ NUMA      *na, *naindex;
         case L_SORT_BY_Y:
             numaAddNumber(na, y);
             break;
+        case L_SORT_BY_RIGHT:
+            numaAddNumber(na, x + w - 1);
+            break;
+        case L_SORT_BY_BOT:
+            numaAddNumber(na, y + h - 1);
+            break;
         case L_SORT_BY_WIDTH:
             numaAddNumber(na, w);
             break;
@@ -598,7 +609,7 @@ NUMA      *na, *naindex;
             numaAddNumber(na, (l_float32)w / (l_float32)h);
             break;
         default:
-            L_WARNING("invalid sort type", procName);
+            L_WARNING("invalid sort type\n", procName);
         }
     }
 
@@ -620,7 +631,7 @@ NUMA      *na, *naindex;
 
 /*!
  *  boxaBinSort()
- * 
+ *
  *      Input:  boxa
  *              sorttype (L_SORT_BY_X, L_SORT_BY_Y, L_SORT_BY_WIDTH,
  *                        L_SORT_BY_HEIGHT, L_SORT_BY_PERIMETER)
@@ -651,7 +662,7 @@ NUMA    *na, *naindex;
     if (pnaindex) *pnaindex = NULL;
     if (!boxas)
         return (BOXA *)ERROR_PTR("boxas not defined", procName, NULL);
-    if (sorttype != L_SORT_BY_X && sorttype != L_SORT_BY_Y && 
+    if (sorttype != L_SORT_BY_X && sorttype != L_SORT_BY_Y &&
         sorttype != L_SORT_BY_WIDTH && sorttype != L_SORT_BY_HEIGHT &&
         sorttype != L_SORT_BY_PERIMETER)
         return (BOXA *)ERROR_PTR("invalid sort type", procName, NULL);
@@ -682,7 +693,7 @@ NUMA    *na, *naindex;
             numaAddNumber(na, w + h);
             break;
         default:
-            L_WARNING("invalid sort type", procName);
+            L_WARNING("invalid sort type\n", procName);
         }
     }
 
@@ -704,7 +715,7 @@ NUMA    *na, *naindex;
 
 /*!
  *  boxaSortByIndex()
- * 
+ *
  *      Input:  boxas
  *              naindex (na that maps from the new boxa to the input boxa)
  *      Return: boxad (sorted), or null on error
@@ -738,7 +749,7 @@ BOXA    *boxad;
 
 /*!
  *  boxaSort2d()
- * 
+ *
  *      Input:  boxas
  *              &naa (<optional return> numaa with sorted indices
  *                    whose values are the indices of the input array)
@@ -748,19 +759,22 @@ BOXA    *boxad;
  *                      onto a boxa of horizontally-aligned boxes; pass 2)
  *              minh1 (components less than this height either join an
  *                     existing boxa or are set aside for pass 2)
- *      Return: boxaa (2d sorted version of boxa), or null on error
+ *      Return: baa (2d sorted version of boxa), or null on error
  *
  *  Notes:
  *      (1) The final result is a sort where the 'fast scan' direction is
  *          left to right, and the 'slow scan' direction is from top
- *          to bottom.  Each boxa in the boxaa represents a sorted set
+ *          to bottom.  Each boxa in the baa represents a sorted set
  *          of boxes from left to right.
- *      (2) Two passes are used to aggregate the boxas, which can corresond
+ *      (2) Three passes are used to aggregate the boxas, which can correspond
  *          to characters or words in a line of text.  In pass 1, only
  *          taller components, which correspond to xheight or larger,
- *          are permitted to start a new boxa, whereas in pass 2,
- *          the remaining vertically-challenged components are allowed
- *          to join an existing boxa or start a new one.
+ *          are permitted to start a new boxa.  In pass 2, the remaining
+ *          vertically-challenged components are allowed to join an
+ *          existing boxa or start a new one.  In pass 3, boxa whose extent
+ *          is overlapping are joined.  After that, the boxes in each
+ *          boxa are sorted horizontally, and finally the boxa are
+ *          sorted vertically.
  *      (3) If delta1 < 0, the first pass allows aggregation when
  *          boxes in the same boxa do not overlap vertically.
  *          The distance by which they can miss and still be aggregated
@@ -782,10 +796,10 @@ boxaSort2d(BOXA    *boxas,
 {
 l_int32  i, index, h, nt, ne, n, m, ival;
 BOX     *box;
-BOXA    *boxa, *boxae, *boxan, *boxat1, *boxat2, *boxav, *boxavs;
-BOXAA   *baa, *baad;
-NUMA    *naindex, *nae, *nan, *nah, *nav, *nat1, *nat2, *nad;
-NUMAA   *naa, *naad;
+BOXA    *boxa, *boxae, *boxan, *boxa1, *boxa2, *boxa3, *boxav, *boxavs;
+BOXAA   *baa, *baa1, *baad;
+NUMA    *naindex, *nae, *nan, *nah, *nav, *na1, *na2, *nad, *namap;
+NUMAA   *naa, *naa1, *naad;
 
     PROCNAME("boxaSort2d");
 
@@ -810,14 +824,12 @@ NUMAA   *naa, *naad;
         if (h < minh1) {  /* save for 2nd pass */
             boxaAddBox(boxae, box, L_INSERT);
             numaAddNumber(nae, i);
-        }
-        else {
+        } else {
             n = boxaaGetCount(baa);
             boxaaAlignBox(baa, box, delta1, &index);
             if (index < n) {  /* append to an existing boxa */
                 boxaaAddBox(baa, index, box, L_INSERT);
-            }
-            else {  /* doesn't align, need new boxa */
+            } else {  /* doesn't align, need new boxa */
                 boxan = boxaCreate(0);
                 boxaAddBox(boxan, box, L_INSERT);
                 boxaaAddBoxa(baa, boxan, L_INSERT);
@@ -831,8 +843,7 @@ NUMAA   *naa, *naad;
     boxaDestroy(&boxa);
     numaDestroy(&naindex);
 
-        /* Second pass: feed in small height boxes;
-         * TODO: this correctly, using local y position! */
+        /* Second pass: feed in small height boxes */
     ne = boxaGetCount(boxae);
     for (i = 0; i < ne; i++) {
         box = boxaGetBox(boxae, i, L_CLONE);
@@ -840,8 +851,7 @@ NUMAA   *naa, *naad;
         boxaaAlignBox(baa, box, delta2, &index);
         if (index < n) {  /* append to an existing boxa */
             boxaaAddBox(baa, index, box, L_INSERT);
-        }
-        else {  /* doesn't align, need new boxa */
+        } else {  /* doesn't align, need new boxa */
             boxan = boxaCreate(0);
             boxaAddBox(boxan, box, L_INSERT);
             boxaaAddBoxa(baa, boxan, L_INSERT);
@@ -852,21 +862,69 @@ NUMAA   *naa, *naad;
         numaaAddNumber(naa, index, ival);
     }
 
-        /* Sort each boxa in the boxaa */
+        /* Third pass: merge some boxa whose extent is overlapping.
+         * Think of these boxa as text lines, where the bounding boxes
+         * of the text lines can overlap, but likely won't have
+         * a huge overlap.
+         * First do a greedy find of pairs of overlapping boxa, where
+         * the two boxa overlap by at least 50% of the smaller, and
+         * the smaller is not more than half the area of the larger.
+         * For such pairs, call the larger one the primary boxa.  The
+         * boxes in the smaller one are appended to those in the primary
+         * in pass 3a, and the primaries are extracted in pass 3b.
+         * In this way, all boxes in the original baa are saved. */
+    n = boxaaGetCount(baa);
+    boxaaGetExtent(baa, NULL, NULL, NULL, &boxa3);
+    boxa1 = boxaHandleOverlaps(boxa3, L_REMOVE_SMALL, 1000, 0.5, 0.5, &namap);
+    boxaDestroy(&boxa1);
+    boxaDestroy(&boxa3);
+    for (i = 0; i < n; i++) {  /* Pass 3a: join selected copies of boxa */
+        numaGetIValue(namap, i, &ival);
+        if (ival >= 0) {  /* join current to primary boxa[ival] */
+            boxa1 = boxaaGetBoxa(baa, i, L_COPY);
+            boxa2 = boxaaGetBoxa(baa, ival, L_CLONE);
+            boxaJoin(boxa2, boxa1, 0, -1);
+            boxaDestroy(&boxa2);
+            boxaDestroy(&boxa1);
+            na1 = numaaGetNuma(naa, i, L_COPY);
+            na2 = numaaGetNuma(naa, ival, L_CLONE);
+            numaJoin(na2, na1, 0, -1);
+            numaDestroy(&na1);
+            numaDestroy(&na2);
+        }
+    }
+    baa1 = boxaaCreate(n);
+    naa1 = numaaCreate(n);
+    for (i = 0; i < n; i++) {  /* Pass 3b: save primary boxa */
+        numaGetIValue(namap, i, &ival);
+        if (ival == -1) {
+            boxa1 = boxaaGetBoxa(baa, i, L_CLONE);
+            boxaaAddBoxa(baa1, boxa1, L_INSERT);
+            na1 = numaaGetNuma(naa, i, L_CLONE);
+            numaaAddNuma(naa1, na1, L_INSERT);
+        }
+    }
+    numaDestroy(&namap);
+    boxaaDestroy(&baa);
+    baa = baa1;
+    numaaDestroy(&naa);
+    naa = naa1;
+
+        /* Sort the boxes in each boxa horizontally */
     m = boxaaGetCount(baa);
     for (i = 0; i < m; i++) {
-        boxat1 = boxaaGetBoxa(baa, i, L_CLONE);
-        boxat2 = boxaSort(boxat1, L_SORT_BY_X, L_SORT_INCREASING, &nah);
-        boxaaReplaceBoxa(baa, i, boxat2);
-        nat1 = numaaGetNuma(naa, i, L_CLONE);
-        nat2 = numaSortByIndex(nat1, nah);
-        numaaReplaceNuma(naa, i, nat2);
-        boxaDestroy(&boxat1);
-        numaDestroy(&nat1);
+        boxa1 = boxaaGetBoxa(baa, i, L_CLONE);
+        boxa2 = boxaSort(boxa1, L_SORT_BY_X, L_SORT_INCREASING, &nah);
+        boxaaReplaceBoxa(baa, i, boxa2);
+        na1 = numaaGetNuma(naa, i, L_CLONE);
+        na2 = numaSortByIndex(na1, nah);
+        numaaReplaceNuma(naa, i, na2);
+        boxaDestroy(&boxa1);
+        numaDestroy(&na1);
         numaDestroy(&nah);
     }
 
-        /* Sort boxa vertically within boxaa, using the first box
+        /* Sort the boxa vertically within boxaa, using the first box
          * in each boxa. */
     m = boxaaGetCount(baa);
     boxav = boxaCreate(m);  /* holds first box in each boxa in baa */
@@ -875,10 +933,10 @@ NUMAA   *naa, *naad;
         *pnaad = naad;
     baad = boxaaCreate(m);
     for (i = 0; i < m; i++) {
-        boxat1 = boxaaGetBoxa(baa, i, L_CLONE);
-        box = boxaGetBox(boxat1, 0, L_CLONE); 
+        boxa1 = boxaaGetBoxa(baa, i, L_CLONE);
+        box = boxaGetBox(boxa1, 0, L_CLONE);
         boxaAddBox(boxav, box, L_INSERT);
-        boxaDestroy(&boxat1);
+        boxaDestroy(&boxa1);
     }
     boxavs = boxaSort(boxav, L_SORT_BY_Y, L_SORT_INCREASING, &nav);
     for (i = 0; i < m; i++) {
@@ -889,6 +947,7 @@ NUMAA   *naa, *naad;
         numaaAddNuma(naad, nad, L_INSERT);
     }
 
+
 /*    fprintf(stderr, "box count = %d, numaa count = %d\n", nt,
             numaaGetNumberCount(naad)); */
 
@@ -908,7 +967,7 @@ NUMAA   *naa, *naad;
 
 /*!
  *  boxaSort2dByIndex()
- * 
+ *
  *      Input:  boxas
  *              naa (numaa that maps from the new baa to the input boxa)
  *      Return: baa (sorted boxaa), or null on error
@@ -1153,64 +1212,78 @@ boxaGetMedian(BOXA  *boxa)
 /*!
  *  boxaaGetExtent()
  *
- *      Input:  boxaa
+ *      Input:  baa
  *              &w  (<optional return> width)
  *              &h  (<optional return> height)
  *              &box (<optional return>, minimum box containing all boxa
  *                    in boxaa)
+ *              &boxa (<optional return>, boxa containing all boxes in each
+ *                     boxa in the boxaa)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
  *      (1) The returned w and h are the minimum size image
  *          that would contain all boxes untranslated.
+ *      (2) Each box in the returned boxa is the minimum box required to
+ *          hold all the boxes in the respective boxa of baa.
+ *      (3) If there are no valid boxes in a boxa, the box corresponding
+ *          to its extent has all fields set to 0 (an invalid box).
  */
 l_int32
-boxaaGetExtent(BOXAA    *boxaa,
+boxaaGetExtent(BOXAA    *baa,
                l_int32  *pw,
                l_int32  *ph,
-               BOX     **pbox)
+               BOX     **pbox,
+               BOXA    **pboxa)
 {
-l_int32  i, j, n, m, x, y, w, h, xmax, ymax, xmin, ymin, found;
-BOXA    *boxa;
+l_int32  i, n, x, y, w, h, xmax, ymax, xmin, ymin, found;
+BOX     *box1;
+BOXA    *boxa, *boxa1;
 
     PROCNAME("boxaaGetExtent");
 
-    if (!pw && !ph && !pbox)
+    if (!pw && !ph && !pbox && !pboxa)
         return ERROR_INT("no ptrs defined", procName, 1);
-    if (pbox) *pbox = NULL;
     if (pw) *pw = 0;
     if (ph) *ph = 0;
-    if (!boxaa)
-        return ERROR_INT("boxaa not defined", procName, 1);
+    if (pbox) *pbox = NULL;
+    if (pboxa) *pboxa = NULL;
+    if (!baa)
+        return ERROR_INT("baa not defined", procName, 1);
 
-    n = boxaaGetCount(boxaa);
+    n = boxaaGetCount(baa);
     if (n == 0)
-        return ERROR_INT("no boxa in boxaa", procName, 1);
+        return ERROR_INT("no boxa in baa", procName, 1);
 
+    boxa = boxaCreate(n);
     xmax = ymax = 0;
     xmin = ymin = 100000000;
     found = FALSE;
     for (i = 0; i < n; i++) {
-        boxa = boxaaGetBoxa(boxaa, i, L_CLONE);
-        m = boxaGetCount(boxa);
-        for (j = 0; j < m; j++) {
-            boxaGetBoxGeometry(boxa, j, &x, &y, &w, &h);
-            if (w <= 0 || h <= 0)
-                continue;
-            found = TRUE;
+        boxa1 = boxaaGetBoxa(baa, i, L_CLONE);
+        boxaGetExtent(boxa1, NULL, NULL, &box1);
+        boxaDestroy(&boxa1);
+        boxGetGeometry(box1, &x, &y, &w, &h);
+        if (w > 0 && h > 0) {  /* a valid extent box */
+            found = TRUE;  /* found at least one valid extent box */
             xmin = L_MIN(xmin, x);
             ymin = L_MIN(ymin, y);
             xmax = L_MAX(xmax, x + w);
             ymax = L_MAX(ymax, y + h);
         }
+        boxaAddBox(boxa, box1, L_INSERT);
     }
-    if (!found)
-        return ERROR_INT("no valid boxes in boxaa", procName, 1);
+    if (found == FALSE)  /* no valid extent boxes */
+        xmin = ymin = 0;
+
     if (pw) *pw = xmax;
     if (ph) *ph = ymax;
     if (pbox)
-      *pbox = boxCreate(xmin, ymin, xmax - xmin, ymax - ymin);
-
+        *pbox = boxCreate(xmin, ymin, xmax - xmin, ymax - ymin);
+    if (pboxa)
+        *pboxa = boxa;
+    else
+        boxaDestroy(&boxa);
     return 0;
 }
 
@@ -1218,21 +1291,21 @@ BOXA    *boxa;
 /*!
  *  boxaaFlattenToBoxa()
  *
- *      Input:  boxaa
- *              &naindex  (<optional return> the boxa index in the boxaa)
+ *      Input:  baa
+ *              &naindex  (<optional return> the boxa index in the baa)
  *              copyflag  (L_COPY or L_CLONE)
  *      Return: boxa, or null on error
  *
  *  Notes:
- *      (1) This 'flattens' the boxaa to a boxa, taking the boxes in
+ *      (1) This 'flattens' the baa to a boxa, taking the boxes in
  *          order in the first boxa, then the second, etc.
  *      (2) If a boxa is empty, we generate an invalid, placeholder box
- *          of zero size.  This is useful when converting from a boxaa
+ *          of zero size.  This is useful when converting from a baa
  *          where each boxa has either 0 or 1 boxes, and it is necessary
  *          to maintain a 1:1 correspondence between the initial
  *          boxa array and the resulting box array.
  *      (3) If &naindex is defined, we generate a Numa that gives, for
- *          each box in the boxaa, the index of the boxa to which it belongs.
+ *          each box in the baa, the index of the boxa to which it belongs.
  */
 BOXA *
 boxaaFlattenToBoxa(BOXAA   *baa,
@@ -1266,8 +1339,7 @@ NUMA    *naindex;
             boxaAddBox(boxa, box, L_INSERT);
             if (pnaindex)
                 numaAddNumber(naindex, i);  /* save 'row' number */
-        }
-        else {
+        } else {
             for (j = 0; j < m; j++) {
                 box = boxaGetBox(boxat, j, copyflag);
                 boxaAddBox(boxa, box, L_INSERT);
@@ -1285,21 +1357,24 @@ NUMA    *naindex;
 /*!
  *  boxaaFlattenAligned()
  *
- *      Input:  boxaa
+ *      Input:  baa
  *              num (number extracted from each)
+ *              fillerbox (<optional> that fills if necessary)
  *              copyflag  (L_COPY or L_CLONE)
  *      Return: boxa, or null on error
  *
  *  Notes:
- *      (1) This 'flattens' the boxaa to a boxa, taking the first @num
+ *      (1) This 'flattens' the baa to a boxa, taking the first @num
  *          boxes from each boxa.
- *      (2) If less than @num boxes are in a boxa, we add invalid placeholder
- *          boxes to preserve the alignment between the input boxaa
- *          and the output boxa.
+ *      (2) In each boxa, if there are less than @num boxes, we preserve
+ *          the alignment between the input baa and the output boxa
+ *          by inserting one or more fillerbox(es) or, if @fillerbox == NULL,
+ *          one or more invalid placeholder boxes.
  */
 BOXA *
 boxaaFlattenAligned(BOXAA   *baa,
                     l_int32  num,
+                    BOX     *fillerbox,
                     l_int32  copyflag)
 {
 l_int32  i, j, m, n, mval, nshort;
@@ -1324,9 +1399,13 @@ BOX     *box;
             box = boxaGetBox(boxat, j, copyflag);
             boxaAddBox(boxad, box, L_INSERT);
         }
-        for (j = 0; j < nshort; j++) {  /* add placeholders if necessary */
-            box = boxCreate(0, 0, 0, 0);
-            boxaAddBox(boxad, box, L_INSERT);
+        for (j = 0; j < nshort; j++) {  /* add fillers if necessary */
+            if (fillerbox) {
+                boxaAddBox(boxad, fillerbox, L_COPY);
+            } else {
+                box = boxCreate(0, 0, 0, 0);  /* invalid placeholder box */
+                boxaAddBox(boxad, box, L_INSERT);
+            }
         }
         boxaDestroy(&boxat);
     }
@@ -1341,11 +1420,11 @@ BOX     *box;
  *      Input:  boxa
  *              num (number put into each boxa in the baa)
  *              copyflag  (L_COPY or L_CLONE)
- *      Return: boxaa, or null on error
+ *      Return: baa, or null on error
  *
  *  Notes:
  *      (1) This puts @num boxes from the input @boxa into each of a
- *          set of boxa within an output boxaa.
+ *          set of boxa within an output baa.
  *      (2) This assumes that the boxes in @boxa are in sets of @num each.
  */
 BOXAA *
@@ -1366,9 +1445,9 @@ BOXAA   *baa;
         return (BOXAA *)ERROR_PTR("invalid copyflag", procName, NULL);
 
     n = boxaGetCount(boxa);
-    nbaa = (n + num - 1) / num;
-    if (n / num != nbaa)
-        L_ERROR("inconsistent alignment: n / num not an integer", procName);
+    nbaa = n / num;
+    if (num * nbaa != n)
+        L_ERROR("inconsistent alignment: num doesn't divide n\n", procName);
     baa = boxaaCreate(nbaa);
     for (i = 0, index = 0; i < nbaa; i++) {
         boxat = boxaCreate(num);
@@ -1385,10 +1464,9 @@ BOXAA   *baa;
 
 /*!
  *  boxaaAlignBox()
- * 
- *      Input:  boxaa
- *              box (to be aligned with the last of one of the boxa
- *                   in boxaa, if possible)
+ *
+ *      Input:  baa
+ *              box (to be aligned with the bext boxa in the baa, if possible)
  *              delta (amount by which consecutive components can miss
  *                     in overlap and still be included in the array)
  *              &index (of boxa with best overlap, or if none match,
@@ -1396,8 +1474,8 @@ BOXAA   *baa;
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
- *      (1) This is not greedy; it finds the boxa whose last box has
- *          the biggest overlap with the input box.
+ *      (1) This is not greedy.  It finds the boxa whose vertical
+ *          extent has the closest overlap with the input box.
  */
 l_int32
 boxaaAlignBox(BOXAA    *baa,
@@ -1406,6 +1484,7 @@ boxaaAlignBox(BOXAA    *baa,
               l_int32  *pindex)
 {
 l_int32  i, n, m, y, yt, h, ht, ovlp, maxovlp, maxindex;
+BOX     *boxt;
 BOXA    *boxa;
 
     PROCNAME("boxaaAlignBox");
@@ -1423,10 +1502,12 @@ BOXA    *boxa;
     for (i = 0; i < n; i++) {
         boxa = boxaaGetBoxa(baa, i, L_CLONE);
         if ((m = boxaGetCount(boxa)) == 0) {
-            L_WARNING("no boxes in boxa", procName);
+            L_WARNING("no boxes in boxa\n", procName);
             continue;
         }
-        boxaGetBoxGeometry(boxa, m - 1, NULL, &yt, NULL, &ht);  /* last one */
+        boxaGetExtent(boxa, NULL, NULL, &boxt);
+        boxGetGeometry(boxt, NULL, &yt, NULL, &ht);
+        boxDestroy(&boxt);
         boxaDestroy(&boxa);
 
             /* Overlap < 0 means the components do not overlap vertically */
