@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -224,7 +235,7 @@ PIXCMAP   *cmap;
  *            h = 120       green
  *            h = 160       cyan
  *            h = 200       blue
- */     
+ */
 l_int32
 convertRGBToHSV(l_int32   rval,
                 l_int32   gval,
@@ -248,16 +259,15 @@ l_float32  h;
     delta = max - min;
 
     *pvval = max;
-    if (delta == 0) {   /* gray; no chroma */
+    if (delta == 0) {  /* gray; no chroma */
         *phval = 0;
         *psval = 0;
-    }
-    else {
+    } else {
         *psval = (l_int32)(255. * (l_float32)delta / (l_float32)max + 0.5);
         if (rval == max)  /* between magenta and yellow */
-            h = (l_float32)(gval - bval) / (l_float32)delta; 
+            h = (l_float32)(gval - bval) / (l_float32)delta;
         else if (gval == max)  /* between yellow and cyan */
-            h = 2. + (l_float32)(bval - rval) / (l_float32)delta; 
+            h = 2. + (l_float32)(bval - rval) / (l_float32)delta;
         else  /* between cyan and magenta */
             h = 4. + (l_float32)(rval - gval) / (l_float32)delta;
         h *= 40.0;
@@ -282,7 +292,7 @@ l_float32  h;
  *  Notes:
  *      (1) See convertRGBToHSV() for valid input range of HSV values
  *          and their interpretation in color space.
- */     
+ */
 l_int32
 convertHSVToRGB(l_int32   hval,
                 l_int32   sval,
@@ -303,8 +313,7 @@ l_float32 h, f, s;
         *prval = vval;
         *pgval = vval;
         *pbval = vval;
-    }
-    else {
+    } else {
         if (hval < 0 || hval > 240)
             return ERROR_INT("invalid hval", procName, 1);
         if (hval == 240)
@@ -352,7 +361,7 @@ l_float32 h, f, s;
             return 1;
         }
     }
-  
+
     return 0;
 }
 
@@ -470,13 +479,13 @@ PIX       *pixt, *pixd;
             maxrg = L_MAX(rval, gval);
             max = L_MAX(maxrg, bval);
             delta = max - min;
-            if (delta == 0)  /* gray; no chroma */
+            if (delta == 0) {  /* gray; no chroma */
                 hval = 0;
-            else {
+            } else {
                 if (rval == max)  /* between magenta and yellow */
-                    fh = (l_float32)(gval - bval) / (l_float32)delta; 
+                    fh = (l_float32)(gval - bval) / (l_float32)delta;
                 else if (gval == max)  /* between yellow and cyan */
-                    fh = 2. + (l_float32)(bval - rval) / (l_float32)delta; 
+                    fh = 2. + (l_float32)(bval - rval) / (l_float32)delta;
                 else  /* between cyan and magenta */
                     fh = 4. + (l_float32)(rval - gval) / (l_float32)delta;
                 fh *= 40.0;
@@ -670,8 +679,7 @@ PIX       *pixt, *pixd;
     if (hstart < hend) {
         for (i = hstart; i <= hend; i++)
             hlut[i] = 1;
-    }
-    else {  /* wrap */
+    } else {  /* wrap */
         for (i = hstart; i < 240; i++)
             hlut[i] = 1;
         for (i = 0; i <= hend; i++)
@@ -768,8 +776,7 @@ PIX       *pixt, *pixd;
     if (hstart < hend) {
         for (i = hstart; i <= hend; i++)
             hlut[i] = 1;
-    }
-    else {
+    } else {
         for (i = hstart; i < 240; i++)
             hlut[i] = 1;
         for (i = 0; i <= hend; i++)
@@ -1250,7 +1257,7 @@ PTA      *pta;
             pixaAddPix(*ppixa, pixt2, L_INSERT);
             pixt3 = pixConvertTo32(pixt1);
             pixRenderHashBoxArb(pixt3, box, 6, 2, L_NEG_SLOPE_LINE,
-                                1, 255, 100, 100); 
+                                1, 255, 100, 100);
             pixaAddPix(*ppixa, pixt3, L_INSERT);
             pixDestroy(&pixt1);
         }
@@ -1262,13 +1269,12 @@ PTA      *pta;
             if (ymax - eheight < 0) {  /* overlap to bottom */
                 box = boxCreate(xmax - ewidth, 240 + ymax - eheight,
                                 2 * ewidth + 1, eheight - ymax);
-            }
-            else if (ymax + eheight > 239) {  /* overlap to top */
+            } else if (ymax + eheight > 239) {  /* overlap to top */
                 box = boxCreate(xmax - ewidth, 0, 2 * ewidth + 1,
                                 ymax + eheight - 239);
-            }
-            else
+            } else {
                 box = NULL;
+            }
             if (box) {
                 pixClearInRect(pixw, box);
                 boxDestroy(&box);
@@ -1495,7 +1501,7 @@ PIXCMAP   *cmap;
  *            Y [16 ... 235]
  *            U [16 ... 240]
  *            V [16 ... 240]
- */     
+ */
 l_int32
 convertRGBToYUV(l_int32   rval,
                 l_int32   gval,
@@ -1538,7 +1544,7 @@ l_float32  norm;
  *      (3) The YUV gamut is larger than the RBG gamut; many YUV values
  *          will result in an invalid RGB value.  We clip individual
  *          r,g,b components to the range [0, 255], and do not test input.
- */     
+ */
 l_int32
 convertYUVToRGB(l_int32   yval,
                 l_int32   uval,
@@ -1631,4 +1637,3 @@ l_int32   i, ncolors, rval, gval, bval, yval, uval, vval;
     }
     return 0;
 }
-
