@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -529,13 +540,12 @@ BOX  *box;
     if (connectivity == 4) {
         if ((box = pixSeedfill4BB(pixs, stack, x, y)) == NULL)
             return (BOX *)ERROR_PTR("box not made", procName, NULL);
-    }
-    else if (connectivity == 8) {
+    } else if (connectivity == 8) {
         if ((box = pixSeedfill8BB(pixs, stack, x, y)) == NULL)
             return (BOX *)ERROR_PTR("box not made", procName, NULL);
-    }
-    else
+    } else {
         return (BOX *)ERROR_PTR("connectivity not 4 or 8", procName, NULL);
+    }
 
     return box;
 }
@@ -567,7 +577,7 @@ BOX  *box;
  *          will overrun the stack and crash.   The stack utility
  *          here grows dynamically as needed, and the fillseg
  *          structures that are not in use are stored in another
- *          stack for reuse.  It should be noted that the 
+ *          stack for reuse.  It should be noted that the
  *          overhead in the function calls (vs. macros) is negligible.
  */
 BOX *
@@ -619,12 +629,12 @@ BOX       *box;
         line = data + y * wpl;
 
             /* A segment of scanline y - dy for x1 <= x <= x2 was
-             * previously filled.  We now explore adjacent pixels 
+             * previously filled.  We now explore adjacent pixels
              * in scan line y.  There are three regions: to the
              * left of x1 - 1, between x1 and x2, and to the right of x2.
              * These regions are handled differently.  Leaks are
              * possible expansions beyond the previous segment and
-             * going back in the -dy direction.  These can happen 
+             * going back in the -dy direction.  These can happen
              * for x < x1 - 1 and for x > x2 + 1.  Any "leak" segments
              * are plugged with a push in the -dy (opposite) direction.
              * And any segments found anywhere are always extended
@@ -660,7 +670,7 @@ BOX       *box;
         return (BOX *)ERROR_PTR("box not made", procName, NULL);
     return box;
 }
-            
+
 
 /*!
  *  pixSeedfill8BB()
@@ -733,12 +743,12 @@ BOX       *box;
         line = data + y * wpl;
 
             /* A segment of scanline y - dy for x1 <= x <= x2 was
-             * previously filled.  We now explore adjacent pixels 
+             * previously filled.  We now explore adjacent pixels
              * in scan line y.  There are three regions: to the
              * left of x1, between x1 and x2, and to the right of x2.
              * These regions are handled differently.  Leaks are
              * possible expansions beyond the previous segment and
-             * going back in the -dy direction.  These can happen 
+             * going back in the -dy direction.  These can happen
              * for x < x1 and for x > x2.  Any "leak" segments
              * are plugged with a push in the -dy (opposite) direction.
              * And any segments found anywhere are always extended
@@ -761,7 +771,7 @@ BOX       *box;
             if (x > x2)   /* leak on right? */
                 pushFillsegBB(stack, x2 + 1, x - 1, y, -dy,
                               ymax, &minx, &maxx, &miny, &maxy);
-    skip:   for (x++; x <= x2 + 1 && 
+    skip:   for (x++; x <= x2 + 1 &&
                       x <= xmax &&
                       (GET_DATA_BIT(line, x) == 0); x++)
                 ;
@@ -774,7 +784,7 @@ BOX       *box;
         return (BOX *)ERROR_PTR("box not made", procName, NULL);
     return box;
 }
-            
+
 
 /*!
  *  pixSeedfill()
@@ -872,12 +882,12 @@ l_uint32  *data, *line;
         line = data + y * wpl;
 
             /* A segment of scanline y - dy for x1 <= x <= x2 was
-             * previously filled.  We now explore adjacent pixels 
+             * previously filled.  We now explore adjacent pixels
              * in scan line y.  There are three regions: to the
              * left of x1 - 1, between x1 and x2, and to the right of x2.
              * These regions are handled differently.  Leaks are
              * possible expansions beyond the previous segment and
-             * going back in the -dy direction.  These can happen 
+             * going back in the -dy direction.  These can happen
              * for x < x1 - 1 and for x > x2 + 1.  Any "leak" segments
              * are plugged with a push in the -dy (opposite) direction.
              * And any segments found anywhere are always extended
@@ -907,7 +917,7 @@ l_uint32  *data, *line;
 
     return 0;
 }
-            
+
 
 /*!
  *  pixSeedfill8()
@@ -965,12 +975,12 @@ l_uint32  *data, *line;
         line = data + y * wpl;
 
             /* A segment of scanline y - dy for x1 <= x <= x2 was
-             * previously filled.  We now explore adjacent pixels 
+             * previously filled.  We now explore adjacent pixels
              * in scan line y.  There are three regions: to the
              * left of x1, between x1 and x2, and to the right of x2.
              * These regions are handled differently.  Leaks are
              * possible expansions beyond the previous segment and
-             * going back in the -dy direction.  These can happen 
+             * going back in the -dy direction.  These can happen
              * for x < x1 and for x > x2.  Any "leak" segments
              * are plugged with a push in the -dy (opposite) direction.
              * And any segments found anywhere are always extended
@@ -990,7 +1000,7 @@ l_uint32  *data, *line;
             pushFillseg(stack, xstart, x - 1, y, dy, ymax);
             if (x > x2)   /* leak on right? */
                 pushFillseg(stack, x2 + 1, x - 1, y, -dy, ymax);
-    skip:   for (x++; x <= x2 + 1 && 
+    skip:   for (x++; x <= x2 + 1 &&
                       x <= xmax &&
                       (GET_DATA_BIT(line, x) == 0); x++)
                 ;
@@ -1000,7 +1010,7 @@ l_uint32  *data, *line;
 
     return 0;
 }
-            
+
 
 
 /*-----------------------------------------------------------------------*
@@ -1044,7 +1054,7 @@ L_STACK  *auxstack;
     PROCNAME("pushFillsegBB");
 
     if (!stack) {
-        L_ERROR(procName, "stack not defined");
+        L_ERROR("stack not defined\n", procName);
         return;
     }
 
@@ -1055,16 +1065,16 @@ L_STACK  *auxstack;
 
     if (y + dy >= 0 && y + dy <= ymax) {
         if ((auxstack = stack->auxstack) == NULL) {
-            L_ERROR("auxstack not defined", procName);
+            L_ERROR("auxstack not defined\n", procName);
             return;
         }
 
             /* Get a fillseg to use */
-        if (lstackGetCount(auxstack) > 0)
+        if (lstackGetCount(auxstack) > 0) {
             fseg = (FILLSEG *)lstackRemove(auxstack);
-        else {
+        } else {
             if ((fseg = (FILLSEG *)CALLOC(1, sizeof(FILLSEG))) == NULL) {
-                L_ERROR("fillseg not made", procName);
+                L_ERROR("fillseg not made\n", procName);
                 return;
             }
         }
@@ -1109,22 +1119,22 @@ L_STACK  *auxstack;
     PROCNAME("pushFillseg");
 
     if (!stack) {
-        L_ERROR(procName, "stack not defined");
+        L_ERROR("stack not defined\n", procName);
         return;
     }
 
     if (y + dy >= 0 && y + dy <= ymax) {
         if ((auxstack = stack->auxstack) == NULL) {
-            L_ERROR("auxstack not defined", procName);
+            L_ERROR("auxstack not defined\n", procName);
             return;
         }
 
             /* Get a fillseg to use */
-        if (lstackGetCount(auxstack) > 0)
+        if (lstackGetCount(auxstack) > 0) {
             fseg = (FILLSEG *)lstackRemove(auxstack);
-        else {
+        } else {
             if ((fseg = (FILLSEG *)CALLOC(1, sizeof(FILLSEG))) == NULL) {
-                L_ERROR("fillseg not made", procName);
+                L_ERROR("fillseg not made\n", procName);
                 return;
             }
         }
@@ -1141,7 +1151,7 @@ L_STACK  *auxstack;
 
 /*!
  *  popFillseg()
- * 
+ *
  *      Input:  stack
  *              &xleft (<return>)
  *              &xright (<return>)
@@ -1167,11 +1177,11 @@ L_STACK  *auxstack;
     PROCNAME("popFillseg");
 
     if (!stack) {
-        L_ERROR("stack not defined", procName);
+        L_ERROR("stack not defined\n", procName);
         return;
     }
     if ((auxstack = stack->auxstack) == NULL) {
-        L_ERROR("auxstack not defined", procName);
+        L_ERROR("auxstack not defined\n", procName);
         return;
     }
 
