@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -26,11 +37,19 @@
  *    Read/write from/to memory (see warning)
  *          PIX        *pixReadMemGif()
  *          l_int32     pixWriteMemGif()
- *     
- *    This uses the gif library, version 4.1.6.  Do not use 4.1.4.
  *
- *    This module was generously contribued by Antony Dovgal.
- *    He can be contacted at:  tony *AT* daylessday.org
+ *    This uses the gif library, version 4.1.6 or later.
+ *    Do not use 4.1.4.  It has serious problems handling 1 bpp images.
+ *
+ *    The initial version of this module was generously contribued by
+ *    Antony Dovgal.  He can be contacted at:  tony *AT* daylessday.org
+ *
+ *    There are some issues with version 5:
+ *    - valgrind detects uninitialized values used used for writing
+ *      and conditionally jumping in EGifPutScreenDesc().
+ *    - DGifSlurp() crashes on some images, apparently triggered by
+ *      by some GIF extension records.  The latter problem has been
+ *      reported but not resolved as of October 2013.
  */
 
 #include <string.h>
@@ -57,6 +76,13 @@ static PIX * pixInterlaceGIF(PIX  *pixs);
 static const l_int32 InterlacedOffset[] = {0, 4, 2, 1};
 static const l_int32 InterlacedJumps[] = {8, 8, 4, 2};
 
+    /* Basic interface changed in 5.0 (!) */
+#if GIFLIB_MAJOR < 5
+#define GifMakeMapObject         MakeMapObject
+#define GifFreeMapObject         FreeMapObject
+#define DGifOpenFileHandle(a,b)  DGifOpenFileHandle(a)
+#define EGifOpenFileHandle(a,b)  EGifOpenFileHandle(a)
+#endif  /* GIFLIB_MAJOR */
 
 /*---------------------------------------------------------------------*
  *                       Reading gif from file                         *
@@ -89,7 +115,7 @@ SavedImage       si;
     _lseek(fd, 0, SEEK_SET);
 #endif  /* _MSC_VER */
 
-    if ((gif = DGifOpenFileHandle(fd)) == NULL)
+    if ((gif = DGifOpenFileHandle(fd, NULL)) == NULL)
         return (PIX *)ERROR_PTR("invalid file or file not found",
                                 procName, NULL);
 
@@ -120,12 +146,10 @@ SavedImage       si;
     if (si.ImageDesc.ColorMap) {
             /* private cmap for this image */
         gif_cmap = si.ImageDesc.ColorMap;
-    }
-    else if (gif->SColorMap) {
+    } else if (gif->SColorMap) {
             /* global cmap for whole picture */
         gif_cmap = gif->SColorMap;
-    }
-    else {
+    } else {
             /* don't know where to take cmap from */
         DGifCloseFile(gif);
         return (PIX *)ERROR_PTR("color map is missing", procName, NULL);
@@ -166,16 +190,13 @@ SavedImage       si;
                 if (si.RasterBits[i * w + j])
                     SET_DATA_BIT(line, j);
             }
-        }
-        else if (d == 2) {
+        } else if (d == 2) {
             for (j = 0; j < w; j++)
                 SET_DATA_DIBIT(line, j, si.RasterBits[i * w + j]);
-        }
-        else if (d == 4) {
+        } else if (d == 4) {
             for (j = 0; j < w; j++)
                 SET_DATA_QBIT(line, j, si.RasterBits[i * w + j]);
-        }
-        else {  /* d == 8 */
+        } else {  /* d == 8 */
             for (j = 0; j < w; j++)
                 SET_DATA_BYTE(line, j, si.RasterBits[i * w + j]);
         }
@@ -268,11 +289,9 @@ GifByteType     *gif_line;
     d = pixGetDepth(pix);
     if (d == 32) {
         pixd = pixConvertRGBToColormap(pix, 1);
-    }
-    else if (d > 1) {
+    } else if (d > 1) {
         pixd = pixConvertTo8(pix, TRUE);
-    }
-    else {  /* d == 1; make sure there's a colormap */
+    } else {  /* d == 1; make sure there's a colormap */
         pixd = pixClone(pix);
         if (!pixGetColormap(pixd)) {
             cmap = pixcmapCreate(1);
@@ -305,7 +324,7 @@ GifByteType     *gif_line;
     }
 
         /* Save the cmap colors in a gif_cmap */
-    if ((gif_cmap = MakeMapObject(gif_ncolor, NULL)) == NULL) {
+    if ((gif_cmap = GifMakeMapObject(gif_ncolor, NULL)) == NULL) {
         pixDestroy(&pixd);
         return ERROR_INT("failed to create GIF color map", procName, 1);
     }
@@ -314,7 +333,7 @@ GifByteType     *gif_line;
         if (ncolor > 0) {
             if (pixcmapGetColor(cmap, i, &rval, &gval, &bval) != 0) {
                 pixDestroy(&pixd);
-                FreeMapObject(gif_cmap);
+                GifFreeMapObject(gif_cmap);
                 return ERROR_INT("failed to get color from color map",
                                  procName, 1);
             }
@@ -326,9 +345,9 @@ GifByteType     *gif_line;
     }
 
         /* Get the gif file handle */
-    if ((gif = EGifOpenFileHandle(fd)) == NULL) {
+    if ((gif = EGifOpenFileHandle(fd, NULL)) == NULL) {
+        GifFreeMapObject(gif_cmap);
         pixDestroy(&pixd);
-        FreeMapObject(gif_cmap);
         return ERROR_INT("failed to create GIF image handle", procName, 1);
     }
 
@@ -336,11 +355,11 @@ GifByteType     *gif_line;
     if (EGifPutScreenDesc(gif, w, h, gif_cmap->BitsPerPixel, 0, gif_cmap)
         != GIF_OK) {
         pixDestroy(&pixd);
-        FreeMapObject(gif_cmap);
+        GifFreeMapObject(gif_cmap);
         EGifCloseFile(gif);
         return ERROR_INT("failed to write screen description", procName, 1);
     }
-    FreeMapObject(gif_cmap); /* not needed after this point */
+    GifFreeMapObject(gif_cmap); /* not needed after this point */
 
     if (EGifPutImageDesc(gif, 0, 0, w, h, FALSE, NULL) != GIF_OK) {
         pixDestroy(&pixd);
@@ -348,7 +367,7 @@ GifByteType     *gif_line;
         return ERROR_INT("failed to image screen description", procName, 1);
     }
 
-    data = pixGetData(pixd);	
+    data = pixGetData(pixd);
     wpl = pixGetWpl(pixd);
     if (d != 1 && d != 2 && d != 4 && d != 8) {
         pixDestroy(&pixd);
@@ -398,7 +417,7 @@ GifByteType     *gif_line;
          * to read comments. */
     if ((text = pixGetText(pix)) != NULL) {
         if (EGifPutComment(gif, text) != GIF_OK)
-            L_WARNING("gif comment not written", procName);
+            L_WARNING("gif comment not written\n", procName);
     }
 
     FREE(gif_line);
@@ -419,8 +438,10 @@ GifByteType     *gif_line;
  *      Return: pix, or null on error
  *
  *  Notes:
- *      (1) Of course, we are cheating here -- writing the data out
- *          to file and then reading it back in as a gif format.
+ *      (1) Of course, we are cheating here -- writing the data to file
+ *          in gif format and reading it back in.  We can't use the
+ *          GNU runtime extension fmemopen() to avoid writing to a file
+ *          because libgif doesn't have a file stream interface!
  *      (2) This should not be assumed to be safe from a sophisticated
  *          attack, even though we have attempted to make the filename
  *          difficult to guess by embedding the process number and the
@@ -441,23 +462,24 @@ PIX *
 pixReadMemGif(const l_uint8  *cdata,
               size_t          size)
 {
-char     *tname;
-l_uint8  *data;
-PIX      *pix;
+char  *fname;
+PIX   *pix;
 
     PROCNAME("pixReadMemGif");
 
     if (!cdata)
         return (PIX *)ERROR_PTR("cdata not defined", procName, NULL);
+    L_WARNING("writing to a temp file, not directly to memory\n", procName);
 
-    data = (l_uint8 *)cdata;  /* we're really not going to change this */
-    tname = genTempFilename("/tmp/", "mem.gif", 1, 1);
-    l_binaryWrite(tname, "w", data, size);
-    pix = pixRead(tname);
-    remove(tname);
-    FREE(tname);
-    if (!pix)
-        return (PIX *)ERROR_PTR("pix not read", procName, NULL);
+        /* Write to a temp file */
+    fname = genTempFilename("/tmp/", "mem.gif", 1, 1);
+    l_binaryWrite(fname, "w", (l_uint8 *)cdata, size);
+
+        /* Read back from the file */
+    pix = pixRead(fname);
+    lept_rm(NULL, fname);
+    lept_free(fname);
+    if (!pix) L_ERROR("pix not read\n", procName);
     return pix;
 }
 
@@ -478,28 +500,28 @@ pixWriteMemGif(l_uint8  **pdata,
                size_t    *psize,
                PIX       *pix)
 {
-char     *tname;
-l_uint8  *data;
-size_t    nbytes;
+char  *fname;
 
     PROCNAME("pixWriteMemGif");
 
     if (!pdata)
         return ERROR_INT("&data not defined", procName, 1 );
+    *pdata = NULL;
     if (!psize)
         return ERROR_INT("&size not defined", procName, 1 );
+    *psize = 0;
     if (!pix)
         return ERROR_INT("&pix not defined", procName, 1 );
+    L_WARNING("writing to a temp file, not directly to memory\n", procName);
 
-    tname = genTempFilename("/tmp/", "mem.gif", 1, 1);
-    pixWrite(tname, pix, IFF_GIF);
-    data = l_binaryRead(tname, &nbytes);
-    remove(tname);
-    FREE(tname);
-    if (!data)
-        return ERROR_INT("data not returned", procName, 1 );
-    *pdata = data;
-    *psize = nbytes;
+        /* Write to a temp file */
+    fname = genTempFilename("/tmp/", "mem.gif", 1, 1);
+    pixWrite(fname, pix, IFF_GIF);
+
+        /* Read back into memory */
+    *pdata = l_binaryRead(fname, psize);
+    lept_rm(NULL, fname);
+    lept_free(fname);
     return 0;
 }
 
@@ -507,4 +529,3 @@ size_t    nbytes;
 /* -----------------------------------------------------------------*/
 #endif    /* HAVE_LIBGIF || HAVE_LIBUNGIF  */
 /* -----------------------------------------------------------------*/
-
