@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -61,6 +72,39 @@ l_int32 selectDefaultPdfEncoding(PIX *pix, l_int32 *ptype)
 
 /* ----------------------------------------------------------------------*/
 
+l_int32 convertUnscaledFilesToPdf(const char *dirname, const char *substr,
+                                  const char *title, const char *fileout)
+{
+    return ERROR_INT("function not present", "convertUnscaledFilesToPdf", 1);
+}
+
+/* ----------------------------------------------------------------------*/
+
+l_int32 saConvertUnscaledFilesToPdf(SARRAY *sa, const char *title,
+                                    const char *fileout)
+{
+    return ERROR_INT("function not present", "saConvertUnscaledFilesToPdf", 1);
+}
+
+/* ----------------------------------------------------------------------*/
+
+l_int32 saConvertUnscaledFilesToPdfData(SARRAY *sa, const char *title,
+                                        l_uint8 **pdata, size_t *pnbytes)
+{
+    return ERROR_INT("function not present",
+                     "saConvertUnscaledFilesToPdfData", 1);
+}
+
+/* ----------------------------------------------------------------------*/
+
+l_int32 convertUnscaledToPdfData(const char *fname, const char *title,
+                                 l_uint8 **pdata, size_t *pnbytes)
+{
+    return ERROR_INT("function not present", "convertUnscaledToPdfData", 1);
+}
+
+/* ----------------------------------------------------------------------*/
+
 l_int32 pixaConvertToPdf(PIXA *pixa, l_int32 res, l_float32 scalefactor,
                          l_int32 type, l_int32 quality,
                          const char *title, const char *fileout)
@@ -83,8 +127,8 @@ l_int32 convertToPdf(const char *filein,
                      l_int32 type, l_int32 quality,
                      const char *fileout,
                      l_int32 x, l_int32 y, l_int32 res,
-                     L_PDF_DATA **plpd, l_int32 position,
-                     const char *title)
+                     const char *title,
+                     L_PDF_DATA **plpd, l_int32 position)
 {
     return ERROR_INT("function not present", "convertToPdf", 1);
 }
@@ -95,8 +139,8 @@ l_int32 convertImageDataToPdf(l_uint8 *imdata, size_t size,
                               l_int32 type, l_int32 quality,
                               const char *fileout,
                               l_int32 x, l_int32 y, l_int32 res,
-                              L_PDF_DATA **plpd, l_int32 position,
-                              const char *title)
+                              const char *title,
+                              L_PDF_DATA **plpd, l_int32 position)
 {
     return ERROR_INT("function not present", "convertImageDataToPdf", 1);
 }
@@ -107,8 +151,8 @@ l_int32 convertToPdfData(const char *filein,
                          l_int32 type, l_int32 quality,
                          l_uint8 **pdata, size_t *pnbytes,
                          l_int32 x, l_int32 y, l_int32 res,
-                         L_PDF_DATA **plpd, l_int32 position,
-                         const char *title)
+                         const char *title,
+                         L_PDF_DATA **plpd, l_int32 position)
 {
     return ERROR_INT("function not present", "convertToPdfData", 1);
 }
@@ -119,8 +163,8 @@ l_int32 convertImageDataToPdfData(l_uint8 *imdata, size_t size,
                                   l_int32 type, l_int32 quality,
                                   l_uint8 **pdata, size_t *pnbytes,
                                   l_int32 x, l_int32 y, l_int32 res,
-                                  L_PDF_DATA **plpd, l_int32 position,
-                                  const char *title)
+                                  const char *title,
+                                  L_PDF_DATA **plpd, l_int32 position)
 {
     return ERROR_INT("function not present", "convertImageDataToPdfData", 1);
 }
@@ -130,8 +174,8 @@ l_int32 convertImageDataToPdfData(l_uint8 *imdata, size_t size,
 l_int32 pixConvertToPdf(PIX *pix, l_int32 type, l_int32 quality,
                         const char *fileout,
                         l_int32 x, l_int32 y, l_int32 res,
-                        L_PDF_DATA **plpd, l_int32 position,
-                        const char *title)
+                        const char *title,
+                        L_PDF_DATA **plpd, l_int32 position)
 {
     return ERROR_INT("function not present", "pixConvertToPdf", 1);
 }
@@ -141,8 +185,8 @@ l_int32 pixConvertToPdf(PIX *pix, l_int32 type, l_int32 quality,
 l_int32 pixConvertToPdfData(PIX *pix, l_int32 type, l_int32 quality,
                             l_uint8 **pdata, size_t *pnbytes,
                             l_int32 x, l_int32 y, l_int32 res,
-                            L_PDF_DATA **plpd, l_int32 position,
-                            const char *title)
+                            const char *title,
+                            L_PDF_DATA **plpd, l_int32 position)
 {
     return ERROR_INT("function not present", "pixConvertToPdfData", 1);
 }
@@ -167,9 +211,19 @@ l_int32 convertSegmentedFilesToPdf(const char *dirname, const char *substr,
 
 /* ----------------------------------------------------------------------*/
 
+BOXAA * convertNumberedMasksToBoxaa(const char *dirname, const char *substr,
+                                    l_int32 numpre, l_int32 numpost)
+{
+    return (BOXAA *)ERROR_PTR("function not present",
+                              "convertNumberedMasksToBoxaa", NULL);
+}
+
+/* ----------------------------------------------------------------------*/
+
 l_int32 convertToPdfSegmented(const char *filein, l_int32 res, l_int32 type,
                               l_int32 thresh, BOXA *boxa, l_int32 quality,
-                              l_float32 scalefactor, const char *fileout)
+                              l_float32 scalefactor, const char *title,
+                              const char *fileout)
 {
     return ERROR_INT("function not present", "convertToPdfSegmented", 1);
 }
@@ -178,8 +232,8 @@ l_int32 convertToPdfSegmented(const char *filein, l_int32 res, l_int32 type,
 
 l_int32 pixConvertToPdfSegmented(PIX *pixs, l_int32 res, l_int32 type,
                                  l_int32 thresh, BOXA *boxa, l_int32 quality,
-                                 l_float32 scalefactor, const char *fileout,
-                                 const char *title)
+                                 l_float32 scalefactor, const char *title,
+                                 const char *fileout)
 {
     return ERROR_INT("function not present", "pixConvertToPdfSegmented", 1);
 }
@@ -189,6 +243,7 @@ l_int32 pixConvertToPdfSegmented(PIX *pixs, l_int32 res, l_int32 type,
 l_int32 convertToPdfDataSegmented(const char *filein, l_int32 res,
                                   l_int32 type, l_int32 thresh, BOXA *boxa,
                                   l_int32 quality, l_float32 scalefactor,
+                                  const char *title,
                                   l_uint8 **pdata, size_t *pnbytes)
 {
     return ERROR_INT("function not present", "convertToPdfDataSegmented", 1);
@@ -199,8 +254,8 @@ l_int32 convertToPdfDataSegmented(const char *filein, l_int32 res,
 l_int32 pixConvertToPdfDataSegmented(PIX *pixs, l_int32 res, l_int32 type,
                                      l_int32 thresh, BOXA *boxa,
                                      l_int32 quality, l_float32 scalefactor,
-                                     l_uint8 **pdata, size_t *pnbytes,
-                                     const char *title)
+                                     const char *title,
+                                     l_uint8 **pdata, size_t *pnbytes)
 {
     return ERROR_INT("function not present", "pixConvertToPdfDataSegmented", 1);
 }
@@ -242,6 +297,8 @@ l_int32 saConcatenatePdfToData(SARRAY *sa, l_uint8 **pdata, size_t *pnbytes)
     return ERROR_INT("function not present", "saConcatenatePdfToData", 1);
 }
 
+/* ----------------------------------------------------------------------*/
+
 l_int32 ptraConcatenatePdfToData(L_PTRA *pa_data, SARRAY *sa,
                                  l_uint8 **pdata, size_t *pnbytes)
 {
