@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -76,7 +87,7 @@
  *  Any pix requiring less than 0.5 MB or more than 8 MB of memory will
  *  not come from the memory store.  Instead, it will be dynamically
  *  allocated and freed after use.
- *             
+ *
  *  How is this implemented?
  *
  *  At setup, the full data block size is computed and allocated.
@@ -171,7 +182,7 @@ L_PTRAA          *paa;
         return ERROR_INT("numalloc not defined", procName, 1);
     numaGetSum(numalloc, &nchunks);
     if (nchunks > 1000.0)
-        L_WARNING_FLOAT("There are %.0f chunks", procName, nchunks);
+        L_WARNING("There are %.0f chunks\n", procName, nchunks);
 
     if ((pms = (L_PIX_MEM_STORE *)CALLOC(1, sizeof(L_PIX_MEM_STORE))) == NULL)
         return ERROR_INT("pms not made", procName, 1);
@@ -211,7 +222,7 @@ L_PTRAA          *paa;
     if ((firstptr = (l_uint32 **)CALLOC(nlevels, sizeof(l_uint32 *))) == NULL)
         return ERROR_INT("calloc fail for firstptr", procName, 1);
     pms->firstptr = firstptr;
-    
+
     data = baseptr;
     for (i = 0; i < nlevels; i++) {
         if ((pa = ptraCreate(alloca[i])) == NULL)
@@ -223,7 +234,7 @@ L_PTRAA          *paa;
             data += sizes[i] / 4;
         }
     }
-            
+
     if (logfile) {
         pms->memused = (l_int32 *)CALLOC(nlevels, sizeof(l_int32));
         pms->meminuse = (l_int32 *)CALLOC(nlevels, sizeof(l_int32));
@@ -306,20 +317,19 @@ L_PTRA           *pa;
     if (level < 0) {  /* size range invalid; must alloc */
         if ((data = pmsGetAlloc(nbytes)) == NULL)
             return (void *)ERROR_PTR("data not made", procName, NULL);
-    }
-    else {  /* get from store */
+    } else {  /* get from store */
         pa = ptraaGetPtra(pms->paa, level, L_HANDLE_ONLY);
         data = ptraRemoveLast(pa);
         if (data && pms->logfile) {
-            pms->memused[level]++; 
-            pms->meminuse[level]++; 
-            if (pms->meminuse[level] > pms->memmax[level]) 
+            pms->memused[level]++;
+            pms->meminuse[level]++;
+            if (pms->meminuse[level] > pms->memmax[level])
                 pms->memmax[level]++;
         }
         if (!data) {  /* none left at this level */
             data = pmsGetAlloc(nbytes);
             if (pms->logfile)
-                pms->memempty[level]++; 
+                pms->memempty[level]++;
         }
     }
 
@@ -343,22 +353,22 @@ L_PTRA           *pa;
     PROCNAME("pmsCustomDealloc");
 
     if ((pms = CustomPMS) == NULL) {
-        L_ERROR("pms not defined", procName);
+        L_ERROR("pms not defined\n", procName);
         return;
     }
 
     if (pmsGetLevelForDealloc(data, &level) == 1) {
-        L_ERROR("level not found", procName);
+        L_ERROR("level not found\n", procName);
         return;
     }
 
-    if (level < 0)  /* no logging; just free the data */
+    if (level < 0) {  /* no logging; just free the data */
         FREE(data);
-    else {  /* return the data to the store */
+    } else {  /* return the data to the store */
         pa = ptraaGetPtra(pms->paa, level, L_HANDLE_ONLY);
         ptraAdd(pa, data);
         if (pms->logfile)
-            pms->meminuse[level]--; 
+            pms->meminuse[level]--;
     }
 
     return;
@@ -377,6 +387,10 @@ L_PTRA           *pa;
  *          is freed like normal memory.
  *      (2) If logging is on, only write out allocs that are as large as
  *          the minimum size handled by the memory store.
+ *      (3) size_t is %lu on 64 bit platforms and %u on 32 bit platforms.
+ *          The C99 platform-independent format specifier for size_t is %zu,
+ *          but windows hasn't conformed, so we are forced to go back to
+ *          C89, use %lu, and cast to get platform-independence.  Ugh.
  */
 void *
 pmsGetAlloc(size_t  nbytes)
@@ -394,7 +408,7 @@ L_PIX_MEM_STORE  *pms;
         return (void *)ERROR_PTR("data not made", procName, NULL);
     if (pms->logfile && nbytes >= pms->smallest) {
         fp = fopenWriteStream(pms->logfile, "a");
-        fprintf(fp, "Alloc %ld bytes at %p\n", nbytes, data);
+        fprintf(fp, "Alloc %lu bytes at %p\n", (unsigned long)nbytes, data);
         fclose(fp);
     }
 
@@ -497,20 +511,18 @@ L_PIX_MEM_STORE  *pms;
 
     fprintf(stderr, "Total number of pix used at each level\n");
     for (i = 0; i < pms->nlevels; i++)
-         fprintf(stderr, " Level %d (%ld bytes): %d\n", i, pms->sizes[i],
-                 pms->memused[i]);
+         fprintf(stderr, " Level %d (%lu bytes): %d\n", i,
+                 (unsigned long)pms->sizes[i], pms->memused[i]);
 
     fprintf(stderr, "Max number of pix in use at any time in each level\n");
     for (i = 0; i < pms->nlevels; i++)
-         fprintf(stderr, " Level %d (%ld bytes): %d\n", i, pms->sizes[i],
-                 pms->memmax[i]);
+         fprintf(stderr, " Level %d (%lu bytes): %d\n", i,
+                 (unsigned long)pms->sizes[i], pms->memmax[i]);
 
     fprintf(stderr, "Number of pix alloc'd because none were available\n");
     for (i = 0; i < pms->nlevels; i++)
-         fprintf(stderr, " Level %d (%ld bytes): %d\n", i, pms->sizes[i],
-                 pms->memempty[i]);
+         fprintf(stderr, " Level %d (%lu bytes): %d\n", i,
+                 (unsigned long)pms->sizes[i], pms->memempty[i]);
 
     return;
 }
-
-
