@@ -1,21 +1,32 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
  *  pngio.c
- *                     
+ *
  *    Read png from file
  *          PIX        *pixReadStreamPng()
  *          l_int32     readHeaderPng()
@@ -26,18 +37,12 @@
  *    Write png to file
  *          l_int32     pixWritePng()  [ special top level ]
  *          l_int32     pixWriteStreamPng()
- *          
- *    Read and write of png to/from RGBA pix
- *          PIX        *pixReadRGBAPng();
- *          l_int32     pixWriteRGBAPng();
+ *          l_int32     pixSetZlibCompression()
  *
- *    Setting flags for special modes
- *          void        l_pngSetStrip16To8()
- *          void        l_pngSetStripAlpha()
- *          void        l_pngSetWriteAlpha()
- *          void        l_pngSetZlibCompression()
+ *    Setting flag for special read mode
+ *          void        l_pngSetReadStrip16To8()
  *
- *    Read/write to memory   [not on windows]
+ *    Read/write to memory
  *          PIX        *pixReadMemPng()
  *          l_int32     pixWriteMemPng()
  *
@@ -57,71 +62,70 @@
  *       bpp == bits/pixel of image in Pix (memory)
  *    where each component is referred to as a "sample".
  *
- *    There are three special flags for determining the number or
- *    size of samples retained or written:
- *    (1) var_PNG_STRIP_16_to_8: default is TRUE.  This strips each
- *        16 bit sample down to 8 bps:
- *         - For 16 bps rgb (16 bps, 3 spp) --> 32 bpp rgb Pix
- *         - For 16 bps gray (16 bps, 1 spp) --> 8 bpp grayscale Pix
- *    (2) var_PNG_STRIP_ALPHA: default is TRUE.  This does not copy
- *        the alpha channel to the pix:
- *         - For 8 bps rgba (8 bps, 4 spp) --> 32 bpp rgb Pix
- *    (3) var_PNG_WRITE_ALPHA: default is FALSE.  The default generates
- *        an RGB png file with 3 spp.  If set to TRUE, this generates
- *        an RGBA png file with 4 spp, and writes the alpha channel.
- *    These are set with accessors.
+ *    For reading and writing rgb and rgba images, we read and write
+ *    alpha if it exists (spp == 4) and do not read or write if
+ *    it doesn't (spp == 3).  The alpha component can be 'removed'
+ *    simply by setting spp to 3.  In leptonica, we make relatively
+ *    little explicit use of the alpha sample.  Note that the alpha
+ *    sample in the image is also called "alpha transparency",
+ *    "alpha component" and "alpha layer."
  *
- *    Two convenience functions are included for reading the alpha
- *    channel (if it exists) into the pix, and for writing out the
- *    alpha sample of a pix to a png file:
- *        pixReadRGBAPng()
- *        pixWriteRGBAPng()
- *    These use two of the special flags, setting to the non-default
- *    value before use and resetting to default afterwards.
- *    In leptonica, we make almost no explicit use of the alpha channel.
- *        
- *    Another special flag, var_ZLIB_COMPRESSION, is used to determine
- *    the compression level.  Default is for standard png compression.
+ *    To change the zlib compression level, use pixSetZlibCompression()
+ *    before writing the file.  The default is for standard png compression.
  *    The zlib compression value can be set [0 ... 9], with
  *         0     no compression (huge files)
  *         1     fastest compression
  *         -1    default compression  (equivalent to 6 in latest version)
  *         9     best compression
- *    If not set, we use the default compression in zlib.
  *    Note that if you are using the defined constants in zlib instead
  *    of the compression integers given above, you must include zlib.h.
  *
- *    Note: All special flags use global constants, so if used with
- *          multi-threaded applications, results can be non-deterministic.
+ *    There is global for determining the size of retained samples:
+ *             var_PNG_STRIP_16_to_8
+ *    and a function l_pngSetReadStrip16To8() for setting it.
+ *    The default is TRUE, which causes pixRead() to strip each 16 bit
+ *    sample down to 8 bps:
+ *     - For 16 bps rgb (16 bps, 3 spp) --> 32 bpp rgb Pix
+ *     - For 16 bps gray (16 bps, 1 spp) --> 8 bpp grayscale Pix
+ *    If the variable is set to FALSE, the 16 bit gray samples
+ *    are saved when read; the 16 bit rgb samples return an error.
+ *    Note: results can be non-deterministic if used with
+ *    multi-threaded applications.
+ *
+ *    On systems like windows without fmemopen() and open_memstream(),
+ *    we write data to a temp file and read it back for operations
+ *    between pix and compressed-data, such as pixReadMemPng() and
+ *    pixWriteMemPng().
  */
 
 #include <string.h>
 #include "allheaders.h"
 
-#ifdef HAVE_CONFIG_H
+#ifdef  HAVE_CONFIG_H
 #include "config_auto.h"
 #endif  /* HAVE_CONFIG_H */
 
 /* --------------------------------------------*/
-#if  HAVE_LIBPNG && HAVE_LIBZ   /* defined in environ.h */
+#if  HAVE_LIBPNG   /* defined in environ.h */
 /* --------------------------------------------*/
 
-#include "zlib.h"
 #include "png.h"
 
-/* ----------------Set defaults for read/write options ----------------- */
-    /* strip 16 bpp --> 8 bpp on reading png; default is for stripping */
+#if  HAVE_LIBZ
+#include "zlib.h"
+#else
+#define  Z_DEFAULT_COMPRESSION (-1)
+#endif  /* HAVE_LIBZ */
+
+/* ------------------ Set default for read option -------------------- */
+    /* Strip 16 bpp --> 8 bpp on reading png; default is for stripping.
+     * If you don't strip, you can't read the gray-alpha spp = 2 images. */
 static l_int32   var_PNG_STRIP_16_TO_8 = 1;
-    /* strip alpha on reading png; default is for stripping */
-static l_int32   var_PNG_STRIP_ALPHA = 1;
-    /* write alpha for 32 bpp images; default is to write only RGB */
-static l_int32   var_PNG_WRITE_ALPHA = 0;
-    /* zlib compression in png; default is for standard compression */
-static l_int32   var_ZLIB_COMPRESSION = Z_DEFAULT_COMPRESSION;
 
 
 #ifndef  NO_CONSOLE_IO
-#define  DEBUG     0
+#define  DEBUG_READ     0
+#define  DEBUG_WRITE    0
 #endif  /* ~NO_CONSOLE_IO */
 
 
@@ -139,20 +143,31 @@ static l_int32   var_ZLIB_COMPRESSION = Z_DEFAULT_COMPRESSION;
  *          at the beginning of the file.
  *      (2) To do sequential reads of png format images from a stream,
  *          use pixReadStreamPng()
+ *      (3) Grayscale-with-alpha pngs (spp = 2) are converted to RGBA
+ *          on read; the returned pix has spp = 4 and equal red, green and
+ *          blue channels.
+ *      (4) spp = 1 with alpha (palette) is converted to RGBA with spp = 4.
+ *      (5) We use the high level png interface, where the transforms are set
+ *          up in advance and the header and image are read with a single
+ *          call.  The more complicated interface, where the header is
+ *          read first and the buffers for the raster image are user-
+ *          allocated before reading the image, works OK for single images,
+ *          but I could not get it to work properly for the successive
+ *          png reads that are required by pixaReadStream().
  */
 PIX *
 pixReadStreamPng(FILE  *fp)
 {
-l_uint8      rval, gval, bval;
-l_int32      i, j, k;
-l_int32      wpl, d, spp, cindex;
+l_int32      rval, gval, bval;
+l_int32      i, j, k, index, ncolors;
+l_int32      wpl, d, spp, cindex, tRNS;
 l_uint32     png_transforms;
 l_uint32    *data, *line, *ppixel;
-int          num_palette, num_text;
+int          num_palette, num_text, num_trans;
 png_byte     bit_depth, color_type, channels;
 png_uint_32  w, h, rowbytes;
 png_uint_32  xres, yres;
-png_bytep    rowptr;
+png_bytep    rowptr, trans;
 png_bytep   *row_pointers;
 png_structp  png_ptr;
 png_infop    info_ptr, end_info;
@@ -199,9 +214,6 @@ PIXCMAP     *cmap;
         png_transforms = PNG_TRANSFORM_STRIP_16;
     else
         png_transforms = PNG_TRANSFORM_IDENTITY;
-        /* To remove alpha channel, use PNG_TRANSFORM_STRIP_ALPHA */
-    if (var_PNG_STRIP_ALPHA == 1)   /* our default */
-        png_transforms |= PNG_TRANSFORM_STRIP_ALPHA;
 
         /* Read it */
     png_read_png(png_ptr, info_ptr, png_transforms, NULL);
@@ -214,15 +226,13 @@ PIXCMAP     *cmap;
     color_type = png_get_color_type(png_ptr, info_ptr);
     channels = png_get_channels(png_ptr, info_ptr);
     spp = channels;
+    tRNS = png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS) ? 1 : 0;
 
-    if (spp == 1)
+    if (spp == 1) {
         d = bit_depth;
-    else if (spp == 2) {
-        d = 2 * bit_depth;
-        L_WARNING("there shouldn't be 2 spp!", procName);
-    }
-    else  /* spp == 3 (rgb), spp == 4 (rgba) */
+    } else {  /* spp == 2 (gray + alpha), spp == 3 (rgb), spp == 4 (rgba) */
         d = 4 * bit_depth;
+    }
 
         /* Remove if/when this is implemented for all bit_depths */
     if (spp == 3 && bit_depth != 8) {
@@ -242,9 +252,9 @@ PIXCMAP     *cmap;
             bval = palette[cindex].blue;
             pixcmapAddColor(cmap, rval, gval, bval);
         }
-    }
-    else
+    } else {
         cmap = NULL;
+    }
 
     if ((pix = pixCreate(w, h, d)) == NULL) {
         png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
@@ -253,17 +263,32 @@ PIXCMAP     *cmap;
     wpl = pixGetWpl(pix);
     data = pixGetData(pix);
     pixSetColormap(pix, cmap);
+    pixSetSpp(pix, spp);
 
-    if (spp == 1) {   /* copy straight from buffer to pix */
+    if (spp == 1 && !tRNS) {  /* copy straight from buffer to pix */
         for (i = 0; i < h; i++) {
             line = data + i * wpl;
             rowptr = row_pointers[i];
             for (j = 0; j < rowbytes; j++) {
-                SET_DATA_BYTE(line, j, rowptr[j]);
+                    SET_DATA_BYTE(line, j, rowptr[j]);
             }
         }
-    }
-    else  {   /* spp == 3 or spp == 4 */
+    } else if (spp == 2) {  /* grayscale + alpha; convert to RGBA */
+        L_INFO("converting (gray + alpha) ==> RGBA\n", procName);
+        for (i = 0; i < h; i++) {
+            ppixel = data + i * wpl;
+            rowptr = row_pointers[i];
+            for (j = k = 0; j < w; j++) {
+                    /* Copy gray value into r, g and b */
+                SET_DATA_BYTE(ppixel, COLOR_RED, rowptr[k]);
+                SET_DATA_BYTE(ppixel, COLOR_GREEN, rowptr[k]);
+                SET_DATA_BYTE(ppixel, COLOR_BLUE, rowptr[k++]);
+                SET_DATA_BYTE(ppixel, L_ALPHA_CHANNEL, rowptr[k++]);
+                ppixel++;
+            }
+        }
+        pixSetSpp(pix, 4);  /* we do not support 2 spp pix */
+    } else if (spp == 3 || spp == 4) {
         for (i = 0; i < h; i++) {
             ppixel = data + i * wpl;
             rowptr = row_pointers[i];
@@ -278,14 +303,81 @@ PIXCMAP     *cmap;
         }
     }
 
-#if  DEBUG
+        /* Special case: requires converting to RGBA */
+    if (spp == 1 && tRNS) {
+        if (!cmap) {
+                /* Trivial case: make fully transparent RGBA image */
+            L_INFO("transparency, 1 spp, no colormap, no transparency array: "
+                   "convention is fully transparent image\n", procName);
+            L_INFO("converting (fully transparent 1 spp) ==> RGBA\n", procName);
+            pixDestroy(&pix);
+            pix = pixCreate(w, h, 32);  /* init to alpha = 0 (transparent) */
+            pixSetSpp(pix, 4);
+        } else {
+            L_INFO("converting (cmap + alpha) ==> RGBA\n", procName);
+
+                /* Grab the transparency array */
+            png_get_tRNS(png_ptr, info_ptr, &trans, &num_trans, NULL);
+            if (!trans) {  /* invalid png file */
+                pixDestroy(&pix);
+                png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
+                return (PIX *)ERROR_PTR("cmap, tRNS, but no transparency array",
+                                        procName, NULL);
+            }
+
+                /* Save the cmap and destroy the pix */
+            cmap = pixcmapCopy(pixGetColormap(pix));
+            ncolors = pixcmapGetCount(cmap);
+            pixDestroy(&pix);
+
+                /* Start over with 32 bit RGBA */
+            pix = pixCreate(w, h, 32);
+            wpl = pixGetWpl(pix);
+            data = pixGetData(pix);
+            pixSetSpp(pix, 4);
+
+#if DEBUG_READ
+            fprintf(stderr, "ncolors = %d, num_trans = %d\n",
+                    ncolors, num_trans);
+            for (i = 0; i < ncolors; i++) {
+                pixcmapGetColor(cmap, i, &rval, &gval, &bval);
+                if (i < num_trans) {
+                    fprintf(stderr, "(r,g,b,a) = (%d,%d,%d,%d)\n",
+                            rval, gval, bval, trans[i]);
+                } else {
+                    fprintf(stderr, "(r,g,b,a) = (%d,%d,%d,<<255>>)\n",
+                            rval, gval, bval);
+                }
+            }
+#endif  /* DEBUG_READ */
+
+                /* Extract the data and convert to RGBA */
+            for (i = 0; i < h; i++) {
+                ppixel = data + i * wpl;
+                rowptr = row_pointers[i];
+                for (j = 0; j < w; j++) {
+                    index = rowptr[j];
+                    pixcmapGetColor(cmap, index, &rval, &gval, &bval);
+                    composeRGBPixel(rval, gval, bval, ppixel);
+                        /* Assume missing entries to be 255 (opaque) according
+                         * to the spec:  http://www.w3.org/TR/PNG/#11tRNS */
+                    SET_DATA_BYTE(ppixel, L_ALPHA_CHANNEL,
+                                  index < num_trans ? trans[index] : 255);
+                    ppixel++;
+                }
+            }
+            pixcmapDestroy(&cmap);
+        }
+    }
+
+#if  DEBUG_READ
     if (cmap) {
         for (i = 0; i < 16; i++) {
             fprintf(stderr, "[%d] = %d\n", i,
                    ((l_uint8 *)(cmap->array))[i]);
         }
     }
-#endif  /* DEBUG */
+#endif  /* DEBUG_READ */
 
         /* If there is no colormap, PNG defines black = 0 and
          * white = 1 by default for binary monochrome.  Therefore,
@@ -300,7 +392,7 @@ PIXCMAP     *cmap;
          * (It also doesn't work if there is a colormap.)
          * If there is a colormap that defines black = 1 and
          * white = 0, we don't need to do anything.
-         * 
+         *
          * How do we check the polarity of the colormap?
          * The colormap determines the values of black and
          * white pixels in the following way:
@@ -341,20 +433,23 @@ PIXCMAP     *cmap;
  *  readHeaderPng()
  *
  *      Input:  filename
- *              &width (<return>)
- *              &height (<return>)
- *              &bps (<return>, bits/sample)
- *              &spp (<return>, samples/pixel)
- *              &iscmap (<optional return>; input NULL to ignore)
+ *              &w (<optional return>)
+ *              &h (<optional return>)
+ *              &bps (<optional return>, bits/sample)
+ *              &spp (<optional return>, samples/pixel)
+ *              &iscmap (<optional return>)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
  *      (1) If there is a colormap, iscmap is returned as 1; else 0.
+ *      (2) For gray+alpha, although the png records bps = 16, we
+ *          consider this as two 8 bpp samples (gray and alpha).
+ *          When a gray+alpha is read, it is converted to 32 bpp RGBA.
  */
 l_int32
 readHeaderPng(const char *filename,
-              l_int32    *pwidth,
-              l_int32    *pheight,
+              l_int32    *pw,
+              l_int32    *ph,
               l_int32    *pbps,
               l_int32    *pspp,
               l_int32    *piscmap)
@@ -366,11 +461,9 @@ FILE    *fp;
 
     if (!filename)
         return ERROR_INT("filename not defined", procName, 1);
-    if (!pwidth || !pheight || !pbps || !pspp)
-        return ERROR_INT("input ptr(s) not defined", procName, 1);
     if ((fp = fopenReadStream(filename)) == NULL)
         return ERROR_INT("image file not found", procName, 1);
-    ret = freadHeaderPng(fp, pwidth, pheight, pbps, pspp, piscmap);
+    ret = freadHeaderPng(fp, pw, ph, pbps, pspp, piscmap);
     fclose(fp);
     return ret;
 }
@@ -380,20 +473,20 @@ FILE    *fp;
  *  freadHeaderPng()
  *
  *      Input:  stream
- *              &width (<return>)
- *              &height (<return>)
- *              &bps (<return>, bits/sample)
- *              &spp (<return>, samples/pixel)
- *              &iscmap (<optional return>; input NULL to ignore)
+ *              &w (<optional return>)
+ *              &h (<optional return>)
+ *              &bps (<optional return>, bits/sample)
+ *              &spp (<optional return>, samples/pixel)
+ *              &iscmap (<optional return>)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
- *      (1) If there is a colormap, iscmap is returned as 1; else 0.
+ *      (1) See readHeaderPng().
  */
 l_int32
 freadHeaderPng(FILE     *fp,
-               l_int32  *pwidth,
-               l_int32  *pheight,
+               l_int32  *pw,
+               l_int32  *ph,
                l_int32  *pbps,
                l_int32  *pspp,
                l_int32  *piscmap)
@@ -403,11 +496,14 @@ l_uint8  *data;
 
     PROCNAME("freadHeaderPng");
 
+    if (pw) *pw = 0;
+    if (ph) *ph = 0;
+    if (pbps) *pbps = 0;
+    if (pspp) *pspp = 0;
+    if (piscmap) *piscmap = 0;
     if (!fp)
         return ERROR_INT("stream not defined", procName, 1);
-    if (!pwidth || !pheight || !pbps || !pspp)
-        return ERROR_INT("input ptr(s) not defined", procName, 1);
-    
+
     nbytes = fnbytesInFile(fp);
     if (nbytes < 40)
         return ERROR_INT("file too small to be png", procName, 1);
@@ -415,7 +511,7 @@ l_uint8  *data;
         return ERROR_INT("CALLOC fail for data", procName, 1);
     if (fread(data, 1, 40, fp) != 40)
         return ERROR_INT("error reading data", procName, 1);
-    ret = sreadHeaderPng(data, pwidth, pheight, pbps, pspp, piscmap);
+    ret = sreadHeaderPng(data, pw, ph, pbps, pspp, piscmap);
     FREE(data);
     return ret;
 }
@@ -425,39 +521,48 @@ l_uint8  *data;
  *  sreadHeaderPng()
  *
  *      Input:  data
- *              &width (<return>)
- *              &height (<return>)
- *              &bps (<return>, bits/sample)
- *              &spp (<return>, samples/pixel)
+ *              &w (<optional return>)
+ *              &h (<optional return>)
+ *              &bps (<optional return>, bits/sample)
+ *              &spp (<optional return>, samples/pixel)
  *              &iscmap (<optional return>; input NULL to ignore)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
- *      (1) If there is a colormap, iscmap is returned as 1; else 0.
+ *      (1) See readHeaderPng().
+ *      (2) png colortypes (see png.h: PNG_COLOR_TYPE_*):
+ *          0:  gray; fully transparent (with tRNS) (1 spp)
+ *          2:  RGB (3 spp)
+ *          3:  colormap; colormap+alpha (with tRNS) (1 spp)
+ *          4:  gray + alpha (2 spp)
+ *          6:  RGBA (4 spp)
+ *          Note:
+ *            0 and 3 have the alpha information in a tRNS chunk
+ *            4 and 6 have separate alpha samples with each pixel.
  */
 l_int32
 sreadHeaderPng(const l_uint8  *data,
-               l_int32        *pwidth,
-               l_int32        *pheight,
+               l_int32        *pw,
+               l_int32        *ph,
                l_int32        *pbps,
                l_int32        *pspp,
                l_int32        *piscmap)
 {
-l_uint8    colortype, bps;
 l_uint16   twobytes;
 l_uint16  *pshort;
+l_int32    colortype, bps, spp;
 l_uint32  *pword;
 
     PROCNAME("sreadHeaderPng");
 
     if (!data)
         return ERROR_INT("data not defined", procName, 1);
-    if (!pwidth || !pheight || !pbps || !pspp)
-        return ERROR_INT("input ptr(s) not defined", procName, 1);
-    *pwidth = *pheight = *pbps = *pspp = 0;
-    if (piscmap)
-      *piscmap = 0;
-    
+    if (pw) *pw = 0;
+    if (ph) *ph = 0;
+    if (pbps) *pbps = 0;
+    if (pspp) *pspp = 0;
+    if (piscmap) *piscmap = 0;
+
         /* Check password */
     if (data[0] != 137 || data[1] != 80 || data[2] != 78 ||
         data[3] != 71 || data[4] != 13 || data[5] != 10 ||
@@ -466,21 +571,34 @@ l_uint32  *pword;
 
     pword = (l_uint32 *)data;
     pshort = (l_uint16 *)data;
-    *pwidth = convertOnLittleEnd32(pword[4]);
-    *pheight = convertOnLittleEnd32(pword[5]);
+    if (pw) *pw = convertOnLittleEnd32(pword[4]);
+    if (ph) *ph = convertOnLittleEnd32(pword[5]);
     twobytes = convertOnLittleEnd16(pshort[12]); /* contains depth/sample  */
                                                  /* and the color type     */
     colortype = twobytes & 0xff;  /* color type */
     bps = twobytes >> 8;   /* bits/sample */
-    *pbps = bps;
-    if (colortype == 2)  /* RGB */
-        *pspp = 3;
-    else if (colortype == 6)  /* RGBA */
-        *pspp = 4;
-    else   /* palette or gray */
-        *pspp = 1;
+
+        /* Special case with alpha that is extracted as RGBA.
+         * Note that the cmap+alpha is also extracted as RGBA,
+         * but only if the tRNS chunk exists, which we can't tell
+         * by this simple parser.*/
+    if (colortype == 4)
+        L_INFO("gray + alpha: will extract as RGBA (spp = 4)\n", procName);
+
+    if (colortype == 2) {  /* RGB */
+        spp = 3;
+    } else if (colortype == 6) {  /* RGBA */
+        spp = 4;
+    } else if (colortype == 4) {  /* gray + alpha */
+        spp = 2;
+        bps = 8;  /* both the gray and alpha are 8-bit samples */
+    } else {  /* gray (0) or cmap (3) or cmap+alpha (3) */
+        spp = 1;
+    }
+    if (pbps) *pbps = bps;
+    if (pspp) *pspp = spp;
     if (piscmap) {
-        if (colortype & 1)  /* palette: see png.h, PNG_COLOR_TYPE_... */
+        if (colortype & 1)  /* palette */
             *piscmap = 1;
         else
             *piscmap = 0;
@@ -520,7 +638,7 @@ png_infop    info_ptr;
         return ERROR_INT("stream not opened", procName, 1);
 
        /* Make the two required structs */
-   if ((png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
+    if ((png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
                    (png_voidp)NULL, NULL, NULL)) == NULL)
         return ERROR_INT("png_ptr not made", procName, 1);
     if ((info_ptr = png_create_info_struct(png_ptr)) == NULL) {
@@ -646,7 +764,7 @@ FILE  *fp;
  *          and 32.  However, it is possible, and in some cases desirable,
  *          to write out a png file using an rgb pix that has 24 bpp.
  *          For example, the open source xpdf SplashBitmap class generates
- *          24 bpp rgb images.  Consequently, we anble writing 24 bpp pix.
+ *          24 bpp rgb images.  Consequently, we enable writing 24 bpp pix.
  *          To generate such a pix, you can make a 24 bpp pix without data
  *          and assign the data array to the pix; e.g.,
  *              pix = pixCreateHeader(w, h, 24);
@@ -655,6 +773,14 @@ FILE  *fp;
  *          from the 32 bpp pix.  Caution: do not call pixSetPadBits(),
  *          because the alignment is wrong and you may erase part of the
  *          last pixel on each line.
+ *      (8) If the pix has a colormap, it is written to file.  In most
+ *          situations, the alpha component is 255 for each colormap entry,
+ *          which is opaque and indicates that it should be ignored.
+ *          However, if any alpha component is not 255, it is assumed that
+ *          the alpha values are valid, and they are written to the png
+ *          file in a tRNS segment.  On readback, the tRNS segment is
+ *          identified, and the colormapped image with alpha is converted
+ *          to a 4 spp rgba image.
  */
 l_int32
 pixWriteStreamPng(FILE      *fp,
@@ -663,11 +789,12 @@ pixWriteStreamPng(FILE      *fp,
 {
 char         commentstring[] = "Comment";
 l_int32      i, j, k;
-l_int32      wpl, d, cmflag;
-l_int32      ncolors;
-l_int32     *rmap, *gmap, *bmap;
+l_int32      wpl, d, spp, cmflag, opaque;
+l_int32      ncolors, compval;
+l_int32     *rmap, *gmap, *bmap, *amap;
 l_uint32    *data, *ppixel;
 png_byte     bit_depth, color_type;
+png_byte     alpha[256];
 png_uint_32  w, h;
 png_uint_32  xres, yres;
 png_bytep   *row_pointers;
@@ -705,40 +832,44 @@ char        *text;
     png_init_io(png_ptr, fp);
 
         /* With best zlib compression (9), get between 1 and 10% improvement
-         * over default (5), but the compression is 3 to 10 times slower.
-         * Our default compression is the zlib default (5). */
-    png_set_compression_level(png_ptr, var_ZLIB_COMPRESSION);
+         * over default (6), but the compression is 3 to 10 times slower.
+         * Use the zlib default (6) as our default compression unless
+         * pix->special falls in the range [10 ... 19]; then subtract 10
+         * to get the compression value.  */
+    compval = Z_DEFAULT_COMPRESSION;
+    if (pix->special >= 10 && pix->special < 20)
+        compval = pix->special - 10;
+    png_set_compression_level(png_ptr, compval);
 
     w = pixGetWidth(pix);
     h = pixGetHeight(pix);
     d = pixGetDepth(pix);
+    spp = pixGetSpp(pix);
     if ((cmap = pixGetColormap(pix)))
         cmflag = 1;
     else
         cmflag = 0;
 
         /* Set the color type and bit depth. */
-    if (d == 32 && var_PNG_WRITE_ALPHA == 1) {
+    if (d == 32 && spp == 4) {
         bit_depth = 8;
         color_type = PNG_COLOR_TYPE_RGBA;   /* 6 */
         cmflag = 0;  /* ignore if it exists */
-    }
-    else if (d == 24 || d == 32) {
+    } else if (d == 24 || d == 32) {
         bit_depth = 8;
         color_type = PNG_COLOR_TYPE_RGB;   /* 2 */
         cmflag = 0;  /* ignore if it exists */
-    }
-    else {
+    } else {
         bit_depth = d;
         color_type = PNG_COLOR_TYPE_GRAY;  /* 0 */
     }
     if (cmflag)
         color_type = PNG_COLOR_TYPE_PALETTE;  /* 3 */
 
-#if  DEBUG
+#if  DEBUG_WRITE
     fprintf(stderr, "cmflag = %d, bit_depth = %d, color_type = %d\n",
             cmflag, bit_depth, color_type);
-#endif  /* DEBUG */
+#endif  /* DEBUG_WRITE */
 
     png_set_IHDR(png_ptr, info_ptr, w, h, bit_depth, color_type,
                  PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE,
@@ -753,8 +884,9 @@ char        *text;
         png_set_pHYs(png_ptr, info_ptr, xres, yres, PNG_RESOLUTION_METER);
 
     if (cmflag) {
-        pixcmapToArrays(cmap, &rmap, &gmap, &bmap);
+        pixcmapToArrays(cmap, &rmap, &gmap, &bmap, &amap);
         ncolors = pixcmapGetCount(cmap);
+        pixcmapIsOpaque(cmap, &opaque);
 
             /* Make and save the palette */
         if ((palette = (png_colorp)(CALLOC(ncolors, sizeof(png_color))))
@@ -765,12 +897,17 @@ char        *text;
             palette[i].red = (png_byte)rmap[i];
             palette[i].green = (png_byte)gmap[i];
             palette[i].blue = (png_byte)bmap[i];
+            alpha[i] = (png_byte)amap[i];
         }
 
         png_set_PLTE(png_ptr, info_ptr, palette, (int)ncolors);
+        if (!opaque)  /* alpha channel has some transparency; assume valid */
+            png_set_tRNS(png_ptr, info_ptr, (png_bytep)alpha,
+                         (int)ncolors, NULL);
         FREE(rmap);
         FREE(gmap);
         FREE(bmap);
+        FREE(amap);
     }
 
         /* 0.4545 is treated as the default by some image
@@ -811,9 +948,9 @@ char        *text;
             (!cmap || (cmap && ((l_uint8 *)(cmap->array))[0] == 0x0))) {
             pixt = pixInvert(NULL, pix);
             pixEndianByteSwap(pixt);
-        }
-        else 
+        } else {
             pixt = pixEndianByteSwapNew(pix);
+        }
         if (!pixt) {
             png_destroy_write_struct(&png_ptr, &info_ptr);
             return ERROR_INT("pixt not made", procName, 1);
@@ -848,8 +985,8 @@ char        *text;
             ppixel = data + i * wpl;
             png_write_rows(png_ptr, (png_bytepp)&ppixel, 1);
         }
-    }
-    else {  /* 32 bpp rgb and rgba */
+    } else {  /* 32 bpp rgb and rgba.  Write out the alpha channel if either
+             * the pix has 4 spp or writing it is requested anyway */
         if ((rowbuffer = (png_bytep)CALLOC(w, 4)) == NULL)
             return ERROR_INT("rowbuffer not made", procName, 1);
         for (i = 0; i < h; i++) {
@@ -858,7 +995,7 @@ char        *text;
                 rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_RED);
                 rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_GREEN);
                 rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_BLUE);
-                if (var_PNG_WRITE_ALPHA == 1)
+                if (spp == 4)
                     rowbuffer[k++] = GET_DATA_BYTE(ppixel, L_ALPHA_CHANNEL);
                 ppixel++;
             }
@@ -878,149 +1015,14 @@ char        *text;
 }
 
 
-/*---------------------------------------------------------------------*
- *                    Read and write of png to RGBA                    *
- *---------------------------------------------------------------------*/
-extern const char *ImageFileFormatExtensions[];
-
 /*!
- *  pixReadRGBAPng()
+ *  pixSetZlibCompression()
  *
- *      Input:  filename (of png file)
- *      Return: pix, or null on error
- *
- *  Notes:
- *      (1) Wrapper to keep the alpha channel of a png, if it exists.
- *      (2) The default behavior of pix read functions is to ignore
- *          the alpha channel.
- *      (3) This always leaves alpha stripping in the same mode as
- *          when this function begins.  So if alpha stripping is in
- *          default mode, this disables it, reads the file (including
- *          the alpha channel), and resets back to stripping.  Otherwise,
- *          it leaves stripping disabled.
- */
-PIX *
-pixReadRGBAPng(const char  *filename)
-{
-l_int32  format;
-PIX     *pix;
-
-    PROCNAME("pixReadRGBAPng");
-
-    if (!filename)
-        return (PIX *)ERROR_PTR("filename not defined", procName, NULL);
-
-        /* Make sure it's a png file */
-    findFileFormat(filename, &format);
-    if (format != IFF_PNG) {
-        L_ERROR_STRING("file format is %s, not png", procName,
-                       ImageFileFormatExtensions[format]);
-        return NULL;
-    }
-
-        /* If alpha channel reading is enabled, just read it */
-    if (var_PNG_STRIP_ALPHA == FALSE)
-        return pixRead(filename);
-
-    l_pngSetStripAlpha(0);
-    pix = pixRead(filename);
-    l_pngSetStripAlpha(1);  /* reset to default */
-    if (!pix) L_ERROR("pix not read", procName);
-    return pix;
-}
-
-
-/*!
- *  pixWriteRGBAPng()
- *
- *      Input:  filename
- *              pix (rgba)
+ *      Input:  pix
+ *              compval (zlib compression value)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
- *      (1) Wrapper to write the alpha sample of a 32 bpp pix to
- *          a png file in rgba format.
- *      (2) The default behavior of pix write to png is to ignore
- *          the alpha sample.
- *      (3) This always leaves alpha writing in the same mode as
- *          when this function begins.  So if alpha writing is in
- *          default mode, this enables it, writes out a rgba png file
- *          that includes the alpha channel, and resets to default.
- *          Otherwise, it leaves alpha writing enabled.
- */
-l_int32
-pixWriteRGBAPng(const char *filename,
-                PIX        *pix)
-{
-l_int32  ret;
-
-    PROCNAME("pixWriteRGBAPng");
-
-    if (!pix)
-        return ERROR_INT("pix not defined", procName, 1);
-    if (!filename)
-        return ERROR_INT("filename not defined", procName, 1);
-
-        /* If alpha channel writing is enabled, just write it */
-    if (var_PNG_WRITE_ALPHA == TRUE)
-        return pixWrite(filename, pix, IFF_PNG);
-
-    l_pngSetWriteAlpha(1);
-    ret = pixWritePng(filename, pix, 0.0);
-    l_pngSetWriteAlpha(0);  /* reset to default */
-    return ret;
-}
-
-
-/*---------------------------------------------------------------------*
- *                   Setting flags for special modes                   *
- *---------------------------------------------------------------------*/
-/*!
- *  l_pngSetStrip16To8()
- *
- *      Input:  flag (1 for stripping 16 bpp to 8 bpp; 0 for leaving 16 bpp)
- *      Return: void
- */
-void
-l_pngSetStrip16To8(l_int32  flag)
-{
-    var_PNG_STRIP_16_TO_8 = flag;
-}
-
-
-/*!
- *  l_pngSetStripAlpha()
- *
- *      Input:  flag (1 for stripping alpha channel on read; 0 for leaving it)
- *      Return: void
- */
-void
-l_pngSetStripAlpha(l_int32  flag)
-{
-    var_PNG_STRIP_ALPHA = flag;
-}
-
-
-/*!
- *  l_pngSetWriteAlpha()
- *
- *      Input:  flag (1 for writing alpha channel; 0 for just writing rgb)
- *      Return: void
- */
-void
-l_pngSetWriteAlpha(l_int32  flag)
-{
-    var_PNG_WRITE_ALPHA = flag;
-}
-
-
-/*!
- *  l_pngSetZlibCompression()
- *
- *      Input:  val (zlib compression value)
- *      Return: void
- *
- *  Notes:
  *      (1) Valid zlib compression values are in the interval [0 ... 9],
  *          where, as defined in zlib.h:
  *            0         Z_NO_COMPRESSION
@@ -1032,26 +1034,47 @@ l_pngSetWriteAlpha(l_int32  flag)
  *      (2) If you use the defined constants in zlib.h instead of the
  *          compression integers given above, you must include zlib.h.
  */
-void
-l_pngSetZlibCompression(l_int32  val)
+l_int32
+pixSetZlibCompression(PIX     *pix,
+                      l_int32  compval)
 {
-    PROCNAME("l_pngSetZlibCompression");
-    if (val < -1 || val > 9) {
-        L_ERROR("Invalid zlib comp val; using default", procName);
-        val = Z_DEFAULT_COMPRESSION;
+    PROCNAME("pixSetZlibCompression");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    if (compval < 0 || compval > 9) {
+        L_ERROR("Invalid zlib comp val; using default\n", procName);
+        compval = Z_DEFAULT_COMPRESSION;
     }
-    var_ZLIB_COMPRESSION = val;
+    pix->special = 10 + compval;  /* valid range [10 ... 19] */
+    return 0;
 }
 
 
+/*---------------------------------------------------------------------*
+ *             Setting flag for stripping 16 bits on reading           *
+ *---------------------------------------------------------------------*/
+/*!
+ *  l_pngSetReadStrip16To8()
+ *
+ *      Input:  flag (1 for stripping 16 bpp to 8 bpp on reading;
+ *                    0 for leaving 16 bpp)
+ *      Return: void
+ */
+void
+l_pngSetReadStrip16To8(l_int32  flag)
+{
+    var_PNG_STRIP_16_TO_8 = flag;
+}
+
 
 /*---------------------------------------------------------------------*
  *                         Read/write to memory                        *
  *---------------------------------------------------------------------*/
 #if HAVE_FMEMOPEN
-
 extern FILE *open_memstream(char **data, size_t *size);
 extern FILE *fmemopen(void *data, size_t size, const char *mode);
+#endif  /* HAVE_FMEMOPEN */
 
 /*!
  *  pixReadMemPng()
@@ -1067,20 +1090,26 @@ PIX *
 pixReadMemPng(const l_uint8  *cdata,
               size_t          size)
 {
-l_uint8  *data;
-FILE     *fp;
-PIX      *pix;
+FILE  *fp;
+PIX   *pix;
 
     PROCNAME("pixReadMemPng");
 
     if (!cdata)
         return (PIX *)ERROR_PTR("cdata not defined", procName, NULL);
 
-    data = (l_uint8 *)cdata;  /* we're really not going to change this */
-    if ((fp = fmemopen(data, size, "r")) == NULL)
+#if HAVE_FMEMOPEN
+    if ((fp = fmemopen((void *)cdata, size, "r")) == NULL)
         return (PIX *)ERROR_PTR("stream not opened", procName, NULL);
+#else
+    L_WARNING("work-around: writing to a temp file\n", procName);
+    fp = tmpfile();
+    fwrite(cdata, 1, size, fp);
+    rewind(fp);
+#endif  /* HAVE_FMEMOPEN */
     pix = pixReadStreamPng(fp);
     fclose(fp);
+    if (!pix) L_ERROR("pix not read\n", procName);
     return pix;
 }
 
@@ -1116,38 +1145,21 @@ FILE    *fp;
     if (!pix)
         return ERROR_INT("&pix not defined", procName, 1 );
 
+#if HAVE_FMEMOPEN
     if ((fp = open_memstream((char **)pdata, psize)) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
     ret = pixWriteStreamPng(fp, pix, gamma);
+#else
+    L_WARNING("work-around: writing to a temp file\n", procName);
+    fp = tmpfile();
+    ret = pixWriteStreamPng(fp, pix, gamma);
+    rewind(fp);
+    *pdata = l_binaryReadStream(fp, psize);
+#endif  /* HAVE_FMEMOPEN */
     fclose(fp);
     return ret;
 }
 
-#else
-
-PIX *
-pixReadMemPng(const l_uint8  *cdata,
-              size_t          size)
-{
-    return (PIX *)ERROR_PTR(
-        "png read from memory not implemented on this platform",
-        "pixReadMemPng", NULL);
-}
-
-
-l_int32
-pixWriteMemPng(l_uint8  **pdata,
-               size_t    *psize,
-               PIX       *pix,
-               l_float32  gamma)
-{
-    return ERROR_INT(
-        "png write to memory not implemented on this platform",
-        "pixWriteMemPng", 1);
-}
-
-#endif  /* HAVE_FMEMOPEN */
-
 /* --------------------------------------------*/
-#endif  /* HAVE_LIBPNG && HAVE_LIBZ */
+#endif  /* HAVE_LIBPNG */
 /* --------------------------------------------*/
