@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -81,48 +92,17 @@ l_int32 pixWriteStreamPng(FILE *fp, PIX *pix, l_float32 gamma)
 
 /* ----------------------------------------------------------------------*/
 
-PIX * pixReadRGBAPng(const char *filename)
+l_int32 pixSetZlibCompression(PIX *pix, l_int32 compval)
+
 {
-    return (PIX * )ERROR_PTR("function not present", "pixReadRGBAPng", NULL);
+    return ERROR_INT("function not present", "pixSetZlibCompression", 1);
 }
 
 /* ----------------------------------------------------------------------*/
 
-l_int32 pixWriteRGBAPng(const char *filename, PIX *pix)
+void l_pngSetReadStrip16To8(l_int32 flag)
 {
-    return ERROR_INT("function not present", "pixWriteRGBAPng", 1);
-}
-
-/* ----------------------------------------------------------------------*/
-
-void l_pngSetStrip16To8(l_int32 flag)
-{
-    L_ERROR("function not present", "l_pngSetSTrip16To8");
-    return;
-}
-
-/* ----------------------------------------------------------------------*/
-
-void l_pngSetStripAlpha(l_int32 flag)
-{
-    L_ERROR("function not present", "l_pngSetStripAlpha");
-    return;
-}
-
-/* ----------------------------------------------------------------------*/
-
-void l_pngSetWriteAlpha(l_int32 flag)
-{
-    L_ERROR("function not present", "l_pngSetWriteAlpha");
-    return;
-}
-
-/* ----------------------------------------------------------------------*/
-
-void l_pngSetZlibCompression(l_int32 val)
-
-{
-    L_ERROR("function not present", "l_pngSetZlibCompression");
+    L_ERROR("function not present", "l_pngSetReadStrip16To8");
     return;
 }
 
