@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -103,7 +114,7 @@ l_int32 convertJpegToPSString(const char *filein, char **poutstr,
 
 /* ----------------------------------------------------------------------*/
 
-char * generateJpegPS(const char *filein, L_COMPRESSED_DATA *cid,
+char * generateJpegPS(const char *filein, L_COMP_DATA *cid,
                       l_float32 xpt, l_float32 ypt, l_float32 wpt,
                       l_float32 hpt, l_int32 pageno, l_int32 endpage)
 {
@@ -112,26 +123,26 @@ char * generateJpegPS(const char *filein, L_COMPRESSED_DATA *cid,
 
 /* ----------------------------------------------------------------------*/
 
-L_COMPRESSED_DATA * pixGenerateJpegData(PIX *pixs, l_int32 ascii85flag,
-                                        l_int32 quality)
+L_COMP_DATA * pixGenerateJpegData(PIX *pixs, l_int32 ascii85flag,
+                                  l_int32 quality)
 {
-    return (L_COMPRESSED_DATA *)ERROR_PTR("function not present",
-                                          "pixGenerateJpegData", NULL);
+    return (L_COMP_DATA *)ERROR_PTR("function not present",
+                                    "pixGenerateJpegData", NULL);
 }
 
 /* ----------------------------------------------------------------------*/
 
-L_COMPRESSED_DATA * l_generateJpegData(const char *fname, l_int32 ascii85flag)
+L_COMP_DATA * l_generateJpegData(const char *fname, l_int32 ascii85flag)
 {
-    return (L_COMPRESSED_DATA *)ERROR_PTR("function not present",
-                                          "l_generateJpegData", NULL);
+    return (L_COMP_DATA *)ERROR_PTR("function not present",
+                                    "l_generateJpegData", NULL);
 }
 
 /* ----------------------------------------------------------------------*/
 
-void compressed_dataDestroy(L_COMPRESSED_DATA  **pcid)
+void l_compdataDestroy(L_COMP_DATA  **pcid)
 {
-    L_ERROR("function not present", "compressedDataDestroy");
+    L_ERROR("function not present", "l_compdataDestroy");
     return;
 }
 
@@ -164,7 +175,7 @@ l_int32 convertG4ToPSString(const char *filein, char **poutstr,
 
 /* ----------------------------------------------------------------------*/
 
-char * generateG4PS(const char *filein, L_COMPRESSED_DATA *cid, l_float32 xpt,
+char * generateG4PS(const char *filein, L_COMP_DATA *cid, l_float32 xpt,
                     l_float32 ypt, l_float32 wpt, l_float32 hpt,
                     l_int32 maskflag, l_int32 pageno, l_int32 endpage)
 {
@@ -173,18 +184,18 @@ char * generateG4PS(const char *filein, L_COMPRESSED_DATA *cid, l_float32 xpt,
 
 /* ----------------------------------------------------------------------*/
 
-L_COMPRESSED_DATA * pixGenerateG4Data(PIX *pixs, l_int32 ascii85flag)
+L_COMP_DATA * pixGenerateG4Data(PIX *pixs, l_int32 ascii85flag)
 {
-    return (L_COMPRESSED_DATA *)ERROR_PTR("function not present",
-                                          "pixGenerateG4Data", NULL);
+    return (L_COMP_DATA *)ERROR_PTR("function not present",
+                                    "pixGenerateG4Data", NULL);
 }
 
 /* ----------------------------------------------------------------------*/
 
-L_COMPRESSED_DATA * l_generateG4Data(const char *fname, l_int32 ascii85flag)
+L_COMP_DATA * l_generateG4Data(const char *fname, l_int32 ascii85flag)
 {
-    return (L_COMPRESSED_DATA *)ERROR_PTR("function not present",
-                                          "l_generateG4Data", NULL);
+    return (L_COMP_DATA *)ERROR_PTR("function not present",
+                                    "l_generateG4Data", NULL);
 }
 
 /* ----------------------------------------------------------------------*/
@@ -224,7 +235,7 @@ l_int32 convertFlateToPSString(const char *filein, char **poutstr,
 
 /* ----------------------------------------------------------------------*/
 
-char * generateFlatePS(const char *filein, L_COMPRESSED_DATA *cid,
+char * generateFlatePS(const char *filein, L_COMP_DATA *cid,
                        l_float32 xpt, l_float32 ypt, l_float32 wpt,
                        l_float32 hpt, l_int32 pageno, l_int32 endpage)
 {
@@ -233,19 +244,34 @@ char * generateFlatePS(const char *filein, L_COMPRESSED_DATA *cid,
 
 /* ----------------------------------------------------------------------*/
 
-L_COMPRESSED_DATA * l_generateFlateData(const char *fname,
-                                        l_int32 ascii85flag)
+L_COMP_DATA * l_generateFlateData(const char *fname, l_int32 ascii85flag)
 {
-    return (L_COMPRESSED_DATA *)ERROR_PTR("function not present",
-                                          "l_generateFlateData", NULL);
+    return (L_COMP_DATA *)ERROR_PTR("function not present",
+                                    "l_generateFlateData", NULL);
 }
 
 /* ----------------------------------------------------------------------*/
 
-L_COMPRESSED_DATA * pixGenerateFlateData(PIX *pixs, l_int32 ascii85flag)
+L_COMP_DATA * pixGenerateFlateData(PIX *pixs, l_int32 ascii85flag)
 {
-    return (L_COMPRESSED_DATA *)ERROR_PTR("function not present",
-                                          "pixGenerateFlateData", NULL);
+    return (L_COMP_DATA *)ERROR_PTR("function not present",
+                                    "pixGenerateFlateData", NULL);
+}
+
+/* ----------------------------------------------------------------------*/
+
+l_int32 l_generateCIData(const char *fname, l_int32 type, l_int32 quality,
+                         l_int32 ascii85, L_COMP_DATA **pcid)
+{
+    return ERROR_INT("function not present", "l_generateCIData", 1);
+}
+
+/* ----------------------------------------------------------------------*/
+
+l_int32 pixGenerateCIData(PIX *pixs, l_int32 type, l_int32 quality,
+                         l_int32 ascii85, L_COMP_DATA **pcid)
+{
+    return ERROR_INT("function not present", "pixGenerateCIData", 1);
 }
 
 /* ----------------------------------------------------------------------*/
